var request = require('request')
process.env.NODE_TLS_REJECT_UNAUTHORIZED = "0";
var ngp = require('../../ecxngp');
var host = null
var hypervisor_path = require('./config').hypervisor_path
var hypervisorvm_path = require('./config').hypervisorvm_path
var storageprofilepath = require('./config').storageprofilepath
var recoverypointer_path = require('./config').recoverypointer_path
var setSessionandHeader = require('./helpfunction').setSessionandHeader
var replaceLocalhost = require('./helpfunction').replaceLocalhost
var processHeaders = require('./helpfunction').processXforwardedHeaders
var applyHeaders = require('./helpfunction').applyXforwardedHeadersToUrl
const json2csv = require('json2csv').parse;

var option = {
    method: 'GET',
    url: '',
    headers:
    {
        'x-endeavour-sessionid': '',
        'x-endeavour-auditrequest': 'yes',
        'Content-Type': 'application/json',
        'Accept': 'application/json'
    }
};
var snapOption = {
    method: 'GET',
    url: '',
    headers:
    {
        'x-endeavour-sessionid': '',
        'x-endeavour-auditrequest': 'yes',
        'Content-Type': 'application/json',
        'Accept': 'application/json'
    },
    qs:
    {
        'filter': ''
    }
};
var timelen = { "SUBHOURLY": 60000, "HOURLY": 60 * 60000, "DAILY": 24 * 60 * 60000, "WEEKLY": 7 * 24 * 60 * 60000, "MONTHLY": 30 * 24 * 60 * 60000 }
var words = { "SUBHOURLY": "minute(s)", "HOURLY": "hour(s)", "DAILY": "day(s)", "WEEKLY": "week(s)", "MONTHLY": "month(s)" }

var hypervisor_addr = {};
var SLAbackschedule = {};
var SLArepschedule = {};
var SLAoffschedule = {};
var SLAarchiveschedule = {};
var replicationerrsession = {};
var offloadsession = {};
var archivesession = {};
var backupresult = {};

function setOptionUrl(url) {
    option.url = replaceLocalhost(url);
}

function getstorageprofile(starturl, resolve, reject) {
    setOptionUrl(starturl);
    function loop() {
        request(option, function (error, response, body) {
            if (!error && response.statusCode == 200) {
                var STP = JSON.parse(body).storageprofiles
                var links = JSON.parse(body).links
                for (var i = 0; i < STP.length; i++) {
                    var subpolicies = STP[i].spec.subpolicy
                    for (var j = 0; j < subpolicies.length; j++) {
                        var subp = subpolicies[j]
                        var index = STP[i].id
                        if (subp.type == 'BACKUP') {
                            if (STP[i].name in SLAbackschedule) {
                                SLAbackschedule[STP[i].name][index] = { 'freq': subp.trigger.frequency, 'schedule': subp.trigger.type }
                            }
                            else {
                                SLAbackschedule[STP[i].name] = {}
                                SLAbackschedule[STP[i].name][index] = { 'freq': subp.trigger.frequency, 'schedule': subp.trigger.type }
                            }
                        }
                        else if (subp.type == 'REPLICATION') {
                            if (STP[i].name in SLArepschedule) {
                                SLArepschedule[STP[i].name][index] = { 'freq': subp.trigger.frequency, 'schedule': subp.trigger.type }
                            }
                            else {
                                SLArepschedule[STP[i].name] = {}
                                SLArepschedule[STP[i].name][index] = { 'freq': subp.trigger.frequency, 'schedule': subp.trigger.type }
                            }
                        }
                        else if (subp.type == 'SPPOFFLOAD') {
                            if (STP[i].name in SLAoffschedule) {
                                SLAoffschedule[STP[i].name][index] = { 'freq': subp.trigger.frequency, 'schedule': subp.trigger.type }
                            }
                            else {
                                SLAoffschedule[STP[i].name] = {}
                                SLAoffschedule[STP[i].name][index] = { 'freq': subp.trigger.frequency, 'schedule': subp.trigger.type }
                            }
                        }
                        else if (subp.type == 'SPPARCHIVE') {
                            if (STP[i].name in SLAarchiveschedule) {
                                SLAarchiveschedule[STP[i].name][index] = { 'freq': subp.trigger.frequency, 'schedule': subp.trigger.type }
                            }
                            else {
                                SLAarchiveschedule[STP[i].name] = {}
                                SLAarchiveschedule[STP[i].name][index] = { 'freq': subp.trigger.frequency, 'schedule': subp.trigger.type }
                            }
                        }
                    }
                }
                if ('nextPage' in links) {
                    setOptionUrl(links.nextPage.href);
                    return loop()
                }
                else {
                    return resolve()
                }
            }
            else {
                const err = new Error("Error when getting storageprofile")
                err.status = 403
                reject(err);
            }
        })
    }
    return loop()
}
function gethypervisor(starturl, resolve, reject) {
    setOptionUrl(starturl);
    function loop() {
        request(option, function (error, response, body) {
            if (!error && response.statusCode == 200) {
                var hyperlists = JSON.parse(body).hypervisors
                var links = JSON.parse(body).links
                for (var i = 0; i < hyperlists.length; i++) {
                    hypervisor_addr[hyperlists[i].id] = hyperlists[i].name
                }
                if ('nextPage' in links) {
                    setOptionUrl(links.nextPage.href);
                    return loop()
                }
                else {
                    return resolve()
                }
            }
            else {
                const err = new Error("Error when getting hypervisor info")
                err.status = 403
                reject(err);
            }
        })
    }
    return loop()
}
function gethypervms(starturl, resolve, reject) {
    setOptionUrl(starturl);
    var vmlists = {};
    function loop() {
        request(option, function (error, response, body) {
            if (!error && response.statusCode == 200) {
                var links = JSON.parse(body).links
                var body = JSON.parse(body);
                for (var i = 0; i < body.children.length; i++) {
                    var vm = body.children[i]
                    var vminfo = {}
                    vminfo['VM'] = vm.properties['name']
                    vminfo['SLAPolicy'] = vm.properties.protectionInfo.storageProfileName
                    vminfo['Hypervisor'] = hypervisor_addr[vm.properties.hypervisorKey]
                    vminfo['HypervisorType'] = vm.properties.hypervisorType
                    vminfo['ProtectionTime'] = vm.properties.protectionInfo.protectionTime
                    vminfo['Location'] = vm.properties.location
                    vminfo['Frequency'] = vminfo['SLAPolicy'] in SLAbackschedule && vm.properties.protectionInfo.storageProfileId in SLAbackschedule[vminfo['SLAPolicy']] ? SLAbackschedule[vminfo['SLAPolicy']][vm.properties.protectionInfo.storageProfileId].freq : null
                    vminfo['Schedule'] = vminfo['SLAPolicy'] in SLAbackschedule && vm.properties.protectionInfo.storageProfileId in SLAbackschedule[vminfo['SLAPolicy']] ? SLAbackschedule[vminfo['SLAPolicy']][vm.properties.protectionInfo.storageProfileId].schedule : null
                    vmlists[vm.properties.pk] = vminfo
                }
                if ('nextPage' in links) {
                    setOptionUrl(links.nextPage.href);
                    return loop()
                }
                else {
                    return resolve(vmlists)
                }
            }
            else {
                console.error(error)
                const err = new Error("Error while getting sucessful backup records")
                err.status = 403
                return reject(err);
            }
        })
    }
    return loop()
}
function getrecoveryvms(vmlists, starturl, resolve, reject) {
    backupresult.compliance = []
    backupresult.ucompliance = []
    setOptionUrl(starturl);
    function loop() {
        request(option, function (error, response, body) {
            if (!error && response.statusCode == 200) {
                var links = JSON.parse(body).links
                var body = JSON.parse(body)
                for (var i = 0; i < body.children.length; i++) {
                    var vm = body.children[i]
                    if (vm.properties.errObj.protectionInfo.subPolicyType == 'REPLICATION') {
                        replicationerrsession[vm.properties.pk] = vm.properties.sessionId
                        continue
                    }
                    else if (vm.properties.errObj.protectionInfo.subPolicyType == 'SPPOFFLOAD') {
                        offloadsession[vm.properties.pk] = vm.properties.sessionId
                        continue
                    }
                    else if (vm.properties.errObj.protectionInfo.subPolicyType == 'SPPARCHIVE') {
                        archivesession[vm.properties.pk] = vm.properties.sessionId
                        continue
                    }

                    var vminfo = {}
                    vminfo['VM'] = vm.properties.errObj.name
                    vminfo['SLAPolicy'] = vm.properties.errObj.protectionInfo.storageProfileName
                    if (vm.properties.pk in vmlists) {
                        var sucvm = vmlists[vm.properties.pk]
                        if (vm.properties.errObj.protectionInfo.protectionTime < sucvm.ProtectionTime) {
                            if (Date.now() - timelen[sucvm.Schedule] * sucvm.Frequency < sucvm.ProtectionTime) {
                                backupresult.compliance.push(sucvm)
                            }
                            else {
                                sucvm['FailureReason'] = "Defined RPO for SLA Policy " + sucvm.SLAPolicy + " to run Every " + sucvm.Frequency + " " + words[sucvm.Schedule] + "not meet."
                                backupresult.ucompliance.push(sucvm)
                            }

                        }
                        else {
                            vminfo['Hypervisor'] = sucvm.Hypervisor
                            vminfo['HypervisorType'] = sucvm.HypervisorType
                            vminfo['ProtectionTime'] = sucvm.ProtectionTime
                            vminfo['Location'] = sucvm.Location
                            vminfo['Frequency'] = sucvm.Frequency
                            vminfo['Schedule'] = sucvm.Schedule
                            vminfo['FailureReason'] = vm.properties.errObj.protectionInfo.lastErrorMessage
                            backupresult.ucompliance.push(vminfo)
                        }
                        delete vmlists[vm.properties.pk]
                    }
                    else {
                        vminfo['Hypervisor'] = hypervisor_addr[vm.properties.errObj.hypervisorKey]
                        vminfo['HypervisorType'] = vm.properties.errObj.hypervisorType
                        vminfo['ProtectionTime'] = 'No Succesful Runs'
                        vminfo['Location'] = vm.properties.errObj.location
                        vminfo['Frequency'] = vminfo['SLAPolicy'] in SLAbackschedule && vm.properties.errObj.protectionInfo.storageProfileId in SLAbackschedule[vminfo['SLAPolicy']] ? SLAbackschedule[vminfo['SLAPolicy']][vm.properties.errObj.protectionInfo.storageProfileId].freq : null
                        vminfo['Schedule'] = vminfo['SLAPolicy'] in SLAbackschedule && vm.properties.errObj.protectionInfo.storageProfileId in SLAbackschedule[vminfo['SLAPolicy']] ? SLAbackschedule[vminfo['SLAPolicy']][vm.properties.errObj.protectionInfo.storageProfileId].schedule : null
                        vminfo['FailureReason'] = vm.properties.errObj.protectionInfo.lastErrorMessage
                        backupresult.ucompliance.push(vminfo)
                    }
                }
                if ('nextPage' in links) {
                    setOptionUrl(links.nextPage.href);
                    return loop()
                }
                else {
                    for (var i in vmlists) {
                        if (Date.now() - timelen[vmlists[i].Schedule] * vmlists[i].Frequency < vmlists[i].ProtectionTime) {
                            backupresult.compliance.push(vmlists[i])
                        }
                        else {
                            vmlists[i]['FailureReason'] = "Defined RPO for SLA Policy " + vmlists[i].SLAPolicy + " to run Every " + vmlists[i].Frequency + " " + words[vmlists[i].Schedule] + " not meet."
                            backupresult.ucompliance.push(vmlists[i])
                        }
                    }
                    return resolve()
                }
            }
            else {
                console.error(error)
                const err = new Error("Error while getting failed backup records")
                err.status = 403
                reject(err);
            }
        })
    }
    return loop()
}
function getvmcompliances(starturl, resolve, reject) {
    setOptionUrl(starturl);
    var replicationresult = {}
    var offloadresult = {}
    var archiveresult = {}

    offloadresult.compliance = []
    offloadresult.ucompliance = []
    archiveresult.compliance = []
    archiveresult.ucompliance = []
    replicationresult.compliance = []
    replicationresult.ucompliance = []
    function loop() {
        request(option, function (error, response, body) {
            if (!error && response.statusCode == 200) {
                var links = JSON.parse(body).links
                var body = JSON.parse(body)
                var promisearray = []
                for (var i = 0; i < body.children.length; i++) {
                    var promise = new Promise(function (resolve, reject) {
                        var vm = body.children[i]
                        var vminfo = {}
                        vminfo['VM'] = vm.properties['name']
                        vminfo['SLAPolicy'] = vm.properties.protectionInfo.storageProfileName
                        vminfo['Hypervisor'] = hypervisor_addr[vm.properties.hypervisorKey]
                        vminfo['HypervisorType'] = vm.properties.hypervisorType
                        vminfo['Location'] = vm.properties.location
                        //if below condition is true, that means this particualr VM has replication assigned under a parituclar SLA.
                        if (vm.properties.protectionInfo.storageProfileName in SLArepschedule && vm.properties.protectionInfo.storageProfileId in SLArepschedule[vm.properties.protectionInfo.storageProfileName]) {
                            vminfo['RepFrequency'] = vminfo['SLAPolicy'] in SLArepschedule && vm.properties.protectionInfo.storageProfileId in SLArepschedule[vminfo['SLAPolicy']] ? SLArepschedule[vminfo['SLAPolicy']][vm.properties.protectionInfo.storageProfileId].freq : null
                            vminfo['RepSchedule'] = vminfo['SLAPolicy'] in SLArepschedule && vm.properties.protectionInfo.storageProfileId in SLArepschedule[vminfo['SLAPolicy']] ? SLArepschedule[vminfo['SLAPolicy']][vm.properties.protectionInfo.storageProfileId].schedule : null
                        }
                        if (vm.properties.protectionInfo.storageProfileName in SLAoffschedule && vm.properties.protectionInfo.storageProfileId in SLAoffschedule[vm.properties.protectionInfo.storageProfileName]) {
                            vminfo['OffFrequency'] = vminfo['SLAPolicy'] in SLAoffschedule && vm.properties.protectionInfo.storageProfileId in SLAoffschedule[vminfo['SLAPolicy']] ? SLAoffschedule[vminfo['SLAPolicy']][vm.properties.protectionInfo.storageProfileId].freq : null
                            vminfo['OffSchedule'] = vminfo['SLAPolicy'] in SLAoffschedule && vm.properties.protectionInfo.storageProfileId in SLAoffschedule[vminfo['SLAPolicy']] ? SLAoffschedule[vminfo['SLAPolicy']][vm.properties.protectionInfo.storageProfileId].schedule : null

                        }
                        if (vm.properties.protectionInfo.storageProfileName in SLAarchiveschedule && vm.properties.protectionInfo.storageProfileId in SLAarchiveschedule[vm.properties.protectionInfo.storageProfileName]) {
                            vminfo['ArchiveFrequency'] = vminfo['SLAPolicy'] in SLAarchiveschedule && vm.properties.protectionInfo.storageProfileId in SLAarchiveschedule[vminfo['SLAPolicy']] ? SLAarchiveschedule[vminfo['SLAPolicy']][vm.properties.protectionInfo.storageProfileId].freq : null
                            vminfo['ArchiveSchedule'] = vminfo['SLAPolicy'] in SLAarchiveschedule && vm.properties.protectionInfo.storageProfileId in SLAarchiveschedule[vminfo['SLAPolicy']] ? SLAarchiveschedule[vminfo['SLAPolicy']][vm.properties.protectionInfo.storageProfileId].schedule : null

                        }
                        var filter1 = {};
                        filter1['property'] = "replicatedSnapshot.backupStorageVolumeId";
                        filter1['op'] = "IN";
                        filter1['value'] = [];

                        var filter2 = {};
                        filter2['property'] = "replicatedSnapshot.storageType";
                        filter2['op'] = "IN";
                        filter2['value'] = [];

                        for (var j = 0; j < vm.properties.protectionInfo.storageSnapshots.length; j++) {
                            filter1['value'].push(vm.properties.protectionInfo.storageSnapshots[j].storageVolumeId)
                            filter2['value'].push(vm.properties.protectionInfo.storageSnapshots[j].storageType)
                        }
                        filter2['value'].push("cloud", "archive")


                        snapOption.qs.filter = JSON.stringify([filter1, filter2])
                        request(snapOption, function (error, response, body) {
                            if (!error && response.statusCode == 200) {
                                var childlists = JSON.parse(body).children
                                var reprpotime = Date.now() - timelen[vminfo.RepSchedule] * vminfo.RepFrequency
                                var offrpotime = Date.now() - timelen[vminfo.OffSchedule] * vminfo.OffFrequency
                                var archiverpotime = Date.now() - timelen[vminfo.ArchiveSchedule] * vminfo.ArchiveFrequency
                                var repcount = 0
                                var offcount = 0
                                var archivecount = 0
                                var reptime = 0
                                var offtime = 0
                                var archivetime = 0
                                var repfail = false
                                var offfail = false
                                var archivefail = false
                                for (var k = 0; k < childlists.length; k++) {
                                    //need to differentiate offload, archive and replication
                                    if (repfail && offfail && archivefail) break
                                    if (childlists[k].properties.replicatedSnapshot.storageType == 'vsnap') {
                                        if (childlists[k].properties.sessionId < reprpotime ||
                                            (childlists[k].properties.pk in replicationerrsession && child_lists[k].properties.sessionId < replicationerrsession[childlists[k].properties.pk])) {
                                            repfail = true
                                        }
                                        else {
                                            if (reptime == 0) {
                                                reptime = childlists[k].properties.sessionId
                                            }
                                            else {
                                                reptime = Math.min(childlists[k].properties.sessionId, reptime)
                                            }

                                            repcount++;
                                        }
                                    }
                                    else if (childlists[k].properties.replicatedSnapshot.storageType == 'cloud') {
                                        if (childlists[k].properties.sessionId < offrpotime || (childlists[k].properties.pk in offloadsession && child_lists[k].properties.sessionId < offloadsession[childlists[k].properties.pk])) {
                                            offfail = true
                                        }
                                        else {
                                            if (offtime == 0) {
                                                offtime = childlists[k].properties.sessionId
                                            }
                                            else {
                                                offtime = Math.min(childlists[k].properties.sessionId, offtime)
                                            }

                                            offcount++;
                                        }
                                    }
                                    else if (childlists[k].properties.replicatedSnapshot.storageType == 'archive') {
                                        if (childlists[k].properties.sessionId < archiverpotime || (childlists[k].properties.pk in archivesession && child_lists[k].properties.sessionId < archivesession[childlists[k].properties.pk])) {
                                            archivefail = true
                                        }
                                        else {
                                            if (archivetime == 0) {
                                                archivetime = childlists[k].properties.sessionId
                                            }
                                            else {
                                                archivetime = Math.min(childlists[k].properties.sessionId, offtime)
                                            }

                                            archivecount++;
                                        }
                                    }
                                }
                                if ('RepSchedule' in vminfo) {
                                    let vminfo_copy = JSON.parse(JSON.stringify(vminfo))
                                    if (repcount < filter1.value.length && repfail) {
                                        vminfo_copy['ProtectionTime'] = null
                                        vminfo_copy['FailureReason'] = "Defined RPO for SLA Policy " + vminfo_copy.SLAPolicy + " to run Every " + vminfo_copy.RepFrequency + " " + words[vminfo_copy.RepSchedule] + " not meet."
                                        replicationresult.ucompliance.push(vminfo_copy)
                                    }
                                    else if (repcount >= filter1.value.length && !repfail) {
                                        vminfo_copy['ProtectionTime'] = reptime
                                        replicationresult.compliance.push(vminfo_copy)
                                    }
                                }
                                if ('OffSchedule' in vminfo) {
                                    if (offcount < filter1.value.length && offfail) {
                                        vminfo['ProtectionTime'] = null
                                        vminfo['FailureReason'] = "Defined RPO for SLA Policy " + vminfo.SLAPolicy + " to run Every " + vminfo.OffFrequency + " " + words[vminfo.OffSchedule] + " not meet."
                                        offloadresult.ucompliance.push(vminfo)
                                    }
                                    else if (offcount >= filter1.value.length && !offfail) {
                                        vminfo['ProtectionTime'] = offtime
                                        offloadresult.compliance.push(vminfo)
                                    }
                                }
                                if ('ArchiveSchedule' in vminfo) {
                                    if (archivecount < filter1.value.length && archivefail) {
                                        vminfo['ProtectionTime'] = null
                                        vminfo['FailureReason'] = "Defined RPO for SLA Policy " + vminfo.SLAPolicy + " to run Every " + vminfo.OffFrequency + " " + words[vminfo.ArchiveSchedule] + " not meet."
                                        archiveresult.ucompliance.push(vminfo)
                                    }
                                    else if (archivecount >= filter1.value.length && !archivefail) {
                                        vminfo['ProtectionTime'] = offtime
                                        archiveresult.compliance.push(vminfo)
                                    }
                                }
                                resolve()
                            }
                            else {
                                console.error(error)
                                const err = new Error("Error when getting snapshots")
                                err.status = 403
                                reject(err);
                            }
                        })
                    })
                    promisearray.push(promise)
                }
                return Promise.all(promisearray).then(function () {
                    if ('nextPage' in links) {
                        setOptionUrl(links.nextPage.href);
                        return loop()
                    }
                    else {
                        return resolve({ 'backup2vsnap': backupresult, 'replication': replicationresult, 'offload': offloadresult, 'archive': archiveresult })
                    }
                }).catch(function (error) {
                    console.error(error)
                    const err = new Error("Error while getting replication and offload compliance")
                    err.status = 403
                    reject(err);
                })

            }
            else {
                console.error(error)
                const err = new Error("Error when getting sucessful backup records")
                err.status = 403
                reject(err);
            }
        })
    }
    return loop()
}

function getcompliance(res, req, next) {
    processHeaders(req, option);
    processHeaders(req, snapOption);
    setSessionandHeader(res, req)
        .then(function (value) {
            option.headers['x-endeavour-sessionid'] = value[0]
            snapOption.headers['x-endeavour-sessionid'] = value[0]
            host = value[1]
            snapOption.url = replaceLocalhost(value[1] + '/api/endeavour/catalog/recovery/storagesnapshotreplication?embed=(children(properties))');
            hypervisor_addr = {};
            SLAbackschedule = {};
            SLArepschedule = {};
            SLAoffschedule = {};
            replicationerrsession = {};
            offloadsession = {};
            backupresult = {};
        })
        .then(function () {
            return new Promise(function (resolve, reject) {
                getstorageprofile(host + storageprofilepath, resolve, reject)
            })
        })
        .then(function () {
            return new Promise(function (resolve, reject) {
                gethypervisor(host + hypervisor_path, resolve, reject)
            })
        })
        .then(function () {
            return new Promise(function (resolve, reject) {
                gethypervms(host + hypervisorvm_path, resolve, reject)
            })
        })
        .then(function (vmlists) {
            return new Promise(function (resolve, reject) {
                getrecoveryvms(vmlists, host + recoverypointer_path, resolve, reject)
            })
        })
        .then(function () {
            return new Promise(function (resolve, reject) {
                getvmcompliances(host + hypervisorvm_path, resolve, reject)
            })
        })
        .then(function (result) {
            if ('download' in req.query && req.query['download'] == 'true') {
                res.setHeader('Content-Disposition', 'attachment')
                res.json(result)

            }
            else {
                res.json(result)
            }
        })
        .catch(function (error) {
            console.log(error)
            next(error)
        })
}

module.exports =
{
    getcompliance: getcompliance,
};

