if (!('contains' in String.prototype)) {
	  String.prototype.contains = function(str, startIndex) {
	    return ''.indexOf.call(this, str, startIndex) !== -1;
	  };
	}

if (typeof String.prototype.startsWith != 'function') {
    String.prototype.startsWith = function(prefix) {
        return this.slice(0, prefix.length) == prefix;
    };
}
		 
if (typeof String.prototype.endsWith != 'function') {
    String.prototype.endsWith = function(suffix) {
        return this.slice(-suffix.length) == suffix;
    };
}


var ngp = require('./ecxngp');   

/*
  Set 'skip' in create_policy if ALL of the subtype X slapolicies already exists. Could be multiple policies/jobs to create. I think you should push all of these from is_policy to 
  create_policy.
*/

var funcSteps = [                                                                                  // generalized user handling + provider registration
	{                                                                                              
		continue: 'next',                                                                          
		name: 'opt_usermode',                                                                      // hypervisor user is->reg_provider, else -> reg_createuser
		verb: 'GET',
		path: '/api/identity/user',
		ip: null,                                                                                                                                       
		port: null, 
		type: null,
		func: null,                                                                                 
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
    },    
	{                                                                                              
		continue: 'next',                                                                          // set by previous step [ next | skip | fail ]     
		name: 'opt_createuser',                                                                    // -> reg_provider
		verb: 'POST',
		path: '/api/identity/user',                                                            
		ip: null, 
		port: null, 
		type: null,
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	},     
	{                                                                                              // NB: nothing in this block will change
		continue: 'next',                                                                          // set by previous step [ next | skip | fail ]     
		name: 'opt_apply',   
		verb: 'POST',
		path: null,                                                   
		ip: null, 
		port: null, 
		type: null,
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
    },
	{                                                                                              
		continue: 'next',                                                                          // set to skip if reg_provider succeeds
		name: 'opt_deleteuser',                                                                    // if driven delete any user identities that were created along the way
		verb: 'DELETE',    
		path: null,   
		ip: null, 
		port: null, 
		type: null,
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	},     
];

/*
Yes sir.  And if I do not give you neither username nor password, you don’t even need to do a thing other than copying the post body to the call to ECX core service.

What you're telling me here would seem to imply that I can simply ignore enableFH and enableLogTruncate. That is, they no longer determine whether or not to create a (perhaps) new user or to use a perhaps extant user. 
 
Instead, username+password -> create user (or use existing one should I get a duplicate identity exception). And, mere username -> use extant user (if it exists).
*/



// NB: rollback logic herein could be a candidate for failsafe logic as in regprovider.js
// If neither username nor password, just pass the entire payload to the POST call to core ECX.  
// You only need to add business logic when credential info (username + password, or just username in href form) is present.

const applyOptStepFuncs = {
	opt_usermode: function(step, funcStream, ecxSessionId, oFromNg, cb){                           
		if ( ! oFromNg.options.username && ! oFromNg.options.password )                            // FTI no need for create user step (BTW: password by itself is an error)
		{
			step.cb(ecxSessionId, null);                                                                 
			funcStream[0].continue = 'skip';                                                       // zap create user step
			ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb); 	                           
		}
		else 
		{
			oFromNg.ao = true;                                                                     // tell userMode where to find user info 
			ngp.userMode(step, funcStream, ecxSessionId, oFromNg, cb);                             // NB: do locally, specific to applyOptions
		}
	},
	opt_createuser: function(step, funcStream, ecxSessionId, oFromNg, cb){ 
		oFromNg.ao = true;                                                                                                
		ngp.createUser(step, funcStream, ecxSessionId, oFromNg, cb);                               // create 'regular' (hypervisor or whatever) user as needed
	},
	opt_apply: function(step, funcStream, ecxSessionId, oFromNg, cb){                              //@@ Needs to know to set opProperties in the POST body
		applyOptions(step, funcStream, ecxSessionId, oFromNg, cb);		
	},
	opt_deleteuser: function(step, funcStream, ecxSessionId, oFromNg, cb){                         //@@ if this allowed to run, then MUST rollback any VE user too
		ngp.regDeleteUser(step, funcStream, ecxSessionId, oFromNg, cb);
	},
};


exports.init = init;


function init(req, res, providerType)                                    
{
	var ngState = ngp.getSessionState(req, res);

	if ( ngState.stale )                                                                           // FTI we got bounced while client remained up. AND response already sent.
		return;                                                                                    // !!! R U CERTAIN THAT RESPONSE SENT IN ALL CASES HERE?

	var ecxSid = ngState.ecxSid;
    res.ecxSid = ecxSid;

	var stepsStream = JSON.parse(JSON.stringify(funcSteps));        

	var regStep = null;

	stepsStream.forEach(function(step){                                                           
		if ( regStep )
			return;

		if ( step.name == 'opt_apply' )
		{
			regStep = step;
		}
	});
	
	regStep.path = req.url.replace(ngp.apiFirstStep , '/api/');                                     // NB: proper URL path against which to POST
	

	stepsStream.forEach(function(step){
		step.ip = ngp.localIp;
		step.port = ngp.ecxExternal;

		step.func = applyOptStepFuncs[step.name];                                                  // hook up the step's real workhorse
		
		step.cb = function(ecxSid, data){ 
			//@#@ ngp.logMsg('info', "Finished step '" + ngState.currentFunc + '(' + this.name + ")' in session ID '" + ecxSid + "'");
			ngp.logMsg('info', "Finished step '" + ngp.getCurrentFunc(step.reqId, ecxSid) + '(' + this.name + ")' in session ID '" + ecxSid + "'");

			if ( data )
			{
				ngp.logMsg('debug', "Data returned by step '" + ngState.currentFunc + '(' + this.name + ")' in session ID '" + ecxSid + "'=" + JSON.stringify(data));
			}
		};
	});

	ngp.createRequestStore(stepsStream, ecxSid, req.reqId, 'applyoptions_' + providerType);  

	ngState.currentFunc = 'applyoptions_' + providerType;                                 
	return stepsStream;
}


function applyOptions(step, funcStream, ecxSessionId, oFromNg, cb)
{
	var scrubbedNgInput = ngp.scrubPostBody(oFromNg, true);
	ngp.logMsg('warn', "Input from client for provider options in session ID='" + ecxSessionId + "'=" + JSON.stringify(scrubbedNgInput));

	var ngState = ngp.ecxSidHash[ecxSessionId];

	var postBody = getOptionsPostBody(ecxSessionId, step, ngState, oFromNg);

	ngp.sendRequestToEcx(step.verb, ngp.ecxExternal, step.path, ecxSessionId, postBody, function(optO){ 
		if ( optO.e || optO.response.statusCode != 201 )   
		{
			var failResult = ngp.getGenericEcxErrorResponse(optO, ecxSessionId, step); 

			//@#@ var createdIdentities = ngp.getEndpointState(ecxSessionId, 'createdIdentities');
			var createdIdentities = ngp.getRequestState(step.reqId, ecxSessionId, 'createdIdentities');

			ngp.logMsg('warn', "CIDs=" + JSON.stringify(createdIdentities));
			
			if ( ! createdIdentities || createdIdentities.length < 1 )                                    // FTI no identities created along the way
			{
				if ( funcStream.length > 0 )                                              
					funcStream[0].continue = 'skip'; 
		
				ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, true);       // boolean arg causes ssfr not to set 'fail' (we've already set 'skip')				
			}
			else
			{
				ngp.setSerialStreamResult(ecxSessionId, failResult);                                        // final status
				step.cb(ecxSessionId, optO.rO);                                                  
				ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb); 	                                // let run rollback of created users
			}
		}
		else                                                                                            // EHI provider registered
		{
			ngp.logMsg('debug', "Registration OK response=" + JSON.stringify(optO.rO));  
	
			var successResult = {
				statusCode: optO.response.statusCode,
				response: optO.rO,
			};

			ngp.setSerialStreamResult(ecxSessionId, successResult);                                       // NB: let caller see result of successful update

			if ( funcStream.length > 0 )                                                                  // FTI there is a next step (always first element in what remains of array)
			{
				funcStream[0].continue = 'skip';                                                            // cause skip of next serial step (deletion of identities)                        
			}

			step.cb(ecxSessionId, optO.rO);                                                  
			ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb); 					
		}
	});
}


function getOptionsPostBody(ecxSessionId, step, ngState, oFromNg)                 
{
	delete oFromNg.options.username;
	delete oFromNg.options.password;

	//@#@ if ( ! ngp.getEndpointState(ecxSessionId, 'userHref') )                                                 // FTI we skipped any user logic
	if ( ! ngp.getRequestState(step.reqId, ecxSessionId, 'userHref') )                                                 // FTI we skipped any user logic
	
		return oFromNg;

	oFromNg.options.user = {
		//@#@ href: ngp.getEndpointState(ecxSessionId, 'userHref'),
		href: ngp.getRequestState(step.reqId, ecxSessionId, 'userHref'),
	};

	return oFromNg;
}

