if (!('contains' in String.prototype)) {
    String.prototype.contains = function(str, startIndex) {
      return ''.indexOf.call(this, str, startIndex) !== -1;
    };
  }

if (typeof String.prototype.startsWith != 'function') {
  String.prototype.startsWith = function(prefix) {
      return this.slice(0, prefix.length) == prefix;
  };
}

if (typeof String.prototype.endsWith != 'function') {
  String.prototype.endsWith = function(suffix) {
      return this.slice(-suffix.length) == suffix;
  };
}

var ngp = require('../ecxngp');
var url = require('url');
var HashMap = require('hashmap');

const catalogProtectionTplFile = ngp.PUBLICDIR + '/catalogppoltpl.json';
var catalogProtectionTpl = null;

getConfiguration();

function getConfiguration()
{
    try{
        var catalogProtectionTemp = ngp.fs.readFileSync(catalogProtectionTplFile, 'utf8');
        catalogProtectionTpl = JSON.parse(catalogProtectionTemp);
    }catch(e){
        ngp.logMsg('error', "Failed to read catalog proctection template file because of error = " + e.toString());
        return;
    }
}

var catalogApplySlasFuncSteps = [
	{
		continue: 'next',
		name: 'catalog_applysla_getpolicyjobcreatelist',
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	},
	{
		continue: 'next',
		name: 'catalog_applysla_createpolicyjob',
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	},
	{
		continue: 'next',
		name: 'catalog_applysla_linkresources',
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	},
	{
		continue: 'next',
		name: 'catalog_delete_unused_sla_resources',
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	}	
];

const catalogApplySlasFuncs = {
	catalog_applysla_getpolicyjobcreatelist: function(step, funcStream, ecxSessionId, oFromNg, cb){
		catalogApplySlaGetPolicyJobCreateList(step, funcStream, ecxSessionId, oFromNg, cb);
	},
	catalog_applysla_createpolicyjob: function(step, funcStream, ecxSessionId, oFromNg, cb){
		catalogApplySlaCreatePolicyJob(step, funcStream, ecxSessionId, oFromNg, cb);
	},
	catalog_applysla_linkresources: function(step, funcStream, ecxSessionId, oFromNg, cb){
		catalogApplySlaLinkResources(step, funcStream, ecxSessionId, oFromNg, cb);
	},
	catalog_delete_unused_sla_resources: function(step, funcStream, ecxSessionId, oFromNg, cb){
		catalogDeleteUnusedSlaResources(step, funcStream, ecxSessionId, oFromNg, cb);
	}
};

/**
 * Creates the SLA policy list for function catalogApplySlaCreatePolicies()
 *
 * @param step
 * @param funcStream
 * @param ecxSessionId
 * @param oFromNg
 * @param cb
 */
function catalogApplySlaGetPolicyJobCreateList(step, funcStream, ecxSessionId, oFromNg, cb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Creating catalog Policy/Job create list ==>');
	ngp.addDoubleSpace();
	
	if ( !(Array.isArray(oFromNg['slapolicies'])) || !(oFromNg['subtype']) ){
		ngp.logMsg('error', '		INVALID REQUEST BODY');
		var failResult = {
			statusCode: 400
		};
		ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);
	}

    ngp.ngpCoreHelper.getPolicies(ecxSessionId, function(getPoliciesResObj){
        if (getPoliciesResObj.e || getPoliciesResObj.response.statusCode != 200) {
            ngp.logMsg('error', '		Get policies error  ==> ' + JSON.stringify(getPoliciesResObj.rO));
            var failResult = {
                statusCode: 500,
                response: {
                    response: getPoliciesResObj.rO
                },
            };
            ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);
        }else{
			var policyNameList = {};

            if (oFromNg['slapolicies'].length > 0){
                ngp.logMsg('info', '		Adding SLA policies from POST body');
                oFromNg['slapolicies'].forEach(function(slapolicy){
                    policyNameList[oFromNg.subtype + '_' + slapolicy.name] = 1;
                    ngp.logMsg('info', '			Added Policy/Job name = ' + oFromNg.subtype + '_' + slapolicy.name);
                });

                ngp.logMsg('info', '		Purging Policy/Job create list');
                getPoliciesResObj.rO.policies.forEach(function(policy){
                    if (policyNameList[policy.name])
                    {
                        ngp.logMsg('info', '			Policy/Job pair ' + policy.name + ' already exists, deleting from list.');
                        delete policyNameList[policy.name];
                        return;
                    }
                });

                ngp.logMsg('info', '		Policy/Job pairs to create = ' + JSON.stringify(policyNameList));

                if ( Object.keys(policyNameList).length < 1 ){
                    if ( funcStream.length > 0 )
                        funcStream[0].continue = 'skip';

                    step.cb(ecxSessionId, null);
                    ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb);
                }else{
                    ngp.setRequestState(step.reqId, ecxSessionId, 'catalogCreatePolicyNames', policyNameList);
                    step.cb(ecxSessionId, null);
                    ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb);
                    return;
                }
            }else if (oFromNg['slapolicies'].length < 1){
				ngp.logMsg('info', '		No SLAs are supposed to be associated with catalog backup');
				ngp.logMsg('info', '		Any associated policies and jobs need to be deleted');

				ngp.ngpCoreHelper.getJobs(ecxSessionId, function(getJobsResObj){
					if (getJobsResObj.e || getJobsResObj.response.statusCode != 200) {
						ngp.logMsg('error', '		Get jobs error  ==> ' + JSON.stringify(getJobsResObj.rO));
						var failResult = {
							statusCode: 500,
							response: {
								response: getJobsResObj.rO
							},
						};
						ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);
					}else{
						
						var policyInfoMap = new HashMap();
						var policyIdArray = [];

						getPoliciesResObj.rO.policies.forEach(function(policy){
							if (policy['name'].startsWith(oFromNg['subtype'] + '_')){
								ngp.logMsg('info', '			Policy/Job pair ' + policy.name + ' exists, adding to delete list.');
								//Creating array of policy id's and passing downstream to avoid additional call to policy api
								policyIdArray.push(policy['id'])
								//Creating map to get job name of associated policy and delete
								policyInfoMap.set(policy['id'], policy['name']);
							}
						});
		
						if (policyIdArray.length == 0 ){
							var successResult = {
								statusCode: 201
							};
							ngp.setStepSuccessResult(step, funcStream, ecxSessionId, oFromNg, cb, successResult);
						}else{
							ngp.setRequestState(step.reqId, ecxSessionId, 'deleteCatalogPolicyAndJobFailure', false);
							deleteCatalogPolicyAndJob(step, ecxSessionId, policyIdArray, policyInfoMap, getJobsResObj.rO.jobs, function(deleteCatalogPolicyAndJobResObj){
								var catalogPolicyJobDeletionState = ngp.getRequestState(step.reqId, ecxSessionId, 'deleteCatalogPolicyAndJobFailure');
								if (catalogPolicyJobDeletionState){
									ngp.logMsg('error', '		Failure in deletion of catalog policy and job ==>');
									var failResult = {
										statusCode: 500,
										response: {
											rc: 'Failure in deletion of catalog SLA resources.'
										}											
									};									
									ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);
								}else{
									var successResult = {
										statusCode: 200										
									};															
									ngp.setStepSuccessResult(step, funcStream, ecxSessionId, oFromNg, cb, successResult);
								}			
							});
						}
					}
				});
			}
        }
    });
}

/**
 * Create Catalog Polices and Jobs
 *
 * @param step
 * @param funcStream
 * @param ecxSessionId
 * @param oFromNg
 * @param cb
 */
function catalogApplySlaCreatePolicyJob(step, funcStream, ecxSessionId, oFromNg, cb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Creating catalog Policy/Job ==>');
    ngp.addDoubleSpace();
	var createPolNamesHash = ngp.getRequestState(step.reqId, ecxSessionId, 'catalogCreatePolicyNames');
    //ngp.logMsg('warn', 'POLNAMESHASH=' + JSON.stringify(createPolNamesHash));
    //Example output:  POLNAMESHASH={"catalog_test2-sla1":1}

    var slapHash = ngp.arrayToHash(oFromNg.slapolicies, 'name');
    //Example output: slapHash {"test2-sla1":{"href":"http://IP:8082/api/spec/storageprofile/2102","id":"2102","name":"test2-sla1"}}

	var pjEntries = [];

	var rollback = {
		policies: [],
		jobs: [],
	};

	for ( var policyName in createPolNamesHash )
	{
		var temp = policyName.split('_');
		var entityType = temp.shift();
		var slapMetal = temp.join('_');

		var slap = slapHash[slapMetal];

		var catalogProtectionTplClone = JSON.parse(JSON.stringify(catalogProtectionTpl));
		catalogProtectionTplClone.name = policyName;
		catalogProtectionTplClone.subType = entityType;
		catalogProtectionTplClone.spec.storageworkflow[0].id = slap.id;
		catalogProtectionTplClone.spec.storageworkflow[0].name = slap.name;
		catalogProtectionTplClone.spec.storageworkflow[0].href = slap.href;

		delete catalogProtectionTplClone.spec.storageworkflow[0].trigger;

		var pjEntry = {
			policyName: policyName,
			policyCreated: false,
			policyId: null,
			jobCreated: false,
			jobId: null,
			policyPostBody: catalogProtectionTplClone,
			policyRollbackAttempted: false,
			policyRollbackSuccessful: false,
			jobRollbackAttempted: false,
			jobRollbackSuccessful: false,
		};

		pjEntries.push(pjEntry);
	}

    var numPoliciesToCreate = pjEntries.length;
    ngp.logMsg('info', '		Number of Policy/Job pairs to create: ' + numPoliciesToCreate);

	var finishedPjEntries = [];
	var crSummaryResults = [];


	createCatalogPolicyAndJob(pjEntries, step, ecxSessionId, finishedPjEntries, crSummaryResults, function(failResult){
		if (failResult){
			ngp.logMsg('warn', '		Detected failure of catalog protection policy and job creation in session ' + ecxSessionId + '; attempting rollback');

			var rolledbackPolJobsAr = [];

			rollbackCatalogPolicyAndJob(finishedPjEntries, ecxSessionId, rolledbackPolJobsAr, function(){
				var failResult = {
					statusCode: 500,
					response: {
						rollbackResults: rolledbackPolJobsAr,
					},
				};

				ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);
			});
		}else{
			var rollbackState = {
				createdPolJobsAr: finishedPjEntries,
				rolledbackPolJobsAr: rolledbackPolJobsAr,
			};

			ngp.setRequestState(step.reqId, ecxSessionId, 'catalogPolicyJobCreationResults', crSummaryResults);
			ngp.setRequestState(step.reqId, ecxSessionId, 'catalogFinishedPjEntries', finishedPjEntries);

			step.cb(ecxSessionId, null);
			ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb);
		}
	});
}

function createCatalogPolicyAndJob(pjEntries, step, ecxSessionId, finishedPjEntries, crSummaryResults, policyJobCreatedCb){

	if ( pjEntries.length < 1 ){
		policyJobCreatedCb();
	}else{
		var pjEntry = pjEntries.shift();

        finishedPjEntries.push(pjEntry);

        ngp.ngpCoreHelper.createPolicy(ecxSessionId, pjEntry.policyPostBody, function (policyCreateResObj){
            if ( policyCreateResObj.e || policyCreateResObj.response.statusCode != 201){
                ngp.logMsg('error', '		Catalog protection policy create error ==> ' + JSON.stringify(policyCreateResObj.rO));
                var failResult = ngp.getGenericEcxErrorResponse(policyCreateResObj, ecxSessionId, step);
				policyJobCreatedCb(failResult);
            }else{
				crSummaryResults.push(policyCreateResObj.rO);

                pjEntry.policyId = policyCreateResObj.rO.id;
                pjEntry.policyCreated = true;

                ngp.logMsg('info', '		Catalog protection policy creation response = ' + JSON.stringify(policyCreateResObj.rO));

                var jobTplClone = JSON.parse(JSON.stringify(ngp.jobTpl));

                jobTplClone.name = pjEntry.policyName;
                jobTplClone.description += pjEntry.policyName;
                jobTplClone.policyId = policyCreateResObj.rO.id;

                ngp.ngpCoreHelper.createJob(ecxSessionId, jobTplClone, function(jobCreateResObj){
                    if ( jobCreateResObj.e || jobCreateResObj.response.statusCode != 201 ){
                        ngp.logMsg('error', '		Catalog protection job create error ==> ' + JSON.stringify(jobCreateResObj.rO));
                        var failResult = ngp.getGenericEcxErrorResponse(jobCreateResObj, ecxSessionId, step);
                        policyJobCreatedCb(failResult);
                    }else{
                        crSummaryResults.push(jobCreateResObj.rO);

                        pjEntry.jobId = jobCreateResObj.rO.id;
                        pjEntry.jobCreated = true;

                        ngp.logMsg('warn', '		Catalog protection Job creation response = ' + JSON.stringify(jobCreateResObj.rO));

                        //Recurse until all Policy/Job pairs are created
                        createCatalogPolicyAndJob(pjEntries, step, ecxSessionId, finishedPjEntries, crSummaryResults, policyJobCreatedCb);
                    }
                });
            }
        });
    }
}

/**
 * Apply SLA's on catalog
 *
 * @param step
 * @param funcStream
 * @param ecxSessionId
 * @param oFromNg
 * @param cb
 */
function catalogApplySlaLinkResources(step, funcStream, ecxSessionId, oFromNg, cb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Applying SLAs for catalog ==>');
	ngp.addDoubleSpace();

	//Need to add this to oFromNg under property name storageProfiles
	var slapArClone = JSON.parse(JSON.stringify(oFromNg.slapolicies));
	delete oFromNg.slapolicies;
	oFromNg.storageProfiles = slapArClone;

	ngp.ngpCoreHelper.catalogApplySlas(ecxSessionId, oFromNg, function(catalogApplySlasResObj){
		if ( catalogApplySlasResObj.e || catalogApplySlasResObj.response.statusCode != 201 ){

			var catalogFinishedPjEntries = ngp.getRequestState(step.reqId, ecxSessionId, 'catalogFinishedPjEntries');

			if ( ! catalogFinishedPjEntries || catalogFinishedPjEntries.length < 1 )
			{
				var failResult = ngp.getGenericEcxErrorResponse(catalogApplySlasResObj, ecxSessionId, step);
				ngp.logMsg('warn', '	Detected failure of applying slas of Policy/Job in session ' + ecxSessionId + '; NOTHING TO ROLLBACK');
				ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult);
				return;
			}

			ngp.logMsg('warn', '	Detected failure of applying slas of Policy/Job in session ' + ecxSessionId + '; attempting rollback');

			//Rollback in case of failure in applying sla's
			var rolledbackPolJobsAr = [];
			rollbackCatalogPolicyAndJob(catalogFinishedPjEntries, ecxSessionId, rolledbackPolJobsAr, function(){
				var failResult = {
					statusCode: 500,
					response: {
						rollbackResults: rolledbackPolJobsAr,
					},
				};

				ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult);
			});
		}else{
			var crSummaryResults = ngp.getRequestState(step.reqId, ecxSessionId, 'catalogPolicyJobCreationResults');

			var successResult = {
				statusCode: catalogApplySlasResObj.response.statusCode,
				response: crSummaryResults,
			};

			ngp.setStepSuccessResult(step, funcStream, ecxSessionId, oFromNg, cb, successResult);
		}
	});
}

/**
 * Keep only newly applied SLA resources, delete unused SLA policies and jobs
 * 
 * @param step 
 * @param funcStream 
 * @param ecxSessionId 
 * @param oFromNg 
 * @param cb 
 */
function catalogDeleteUnusedSlaResources(step, funcStream, ecxSessionId, oFromNg, cb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Cleanup unused SLA resources ==>');
	ngp.addDoubleSpace();

    ngp.ngpCoreHelper.getPolicies(ecxSessionId, function(getPoliciesResObj){
        if (getPoliciesResObj.e || getPoliciesResObj.response.statusCode != 200) {
            ngp.logMsg('error', '		Get policies error  ==> ' + JSON.stringify(getPoliciesResObj.rO));
            var failResult = {
                statusCode: 500,
                response: {
                    response: getPoliciesResObj.rO
                },
            };
            ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);
        }else{	

			ngp.ngpCoreHelper.getJobs(ecxSessionId, function(getJobsResObj){
				if (getJobsResObj.e || getJobsResObj.response.statusCode != 200) {
					ngp.logMsg('error', '		Get jobs error  ==> ' + JSON.stringify(getJobsResObj.rO));
					var failResult = {
						statusCode: 500,
						response: {
							response: getJobsResObj.rO
						},
					};
					ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);
				}else{			
					var policyInfoMap = new HashMap();			
					var policyIdArray = [];
					var slasFromUI = [];

					oFromNg['storageProfiles'].forEach(function(slapolicy){
						slasFromUI.push(oFromNg['subtype'] + '_' + slapolicy['name']);								
					});
					
					if (slasFromUI.length > 0)
						ngp.logMsg('info', '		Catalog SLAs resources to retain: ' + slasFromUI.toString());

					ngp.addDoubleSpace();
					getPoliciesResObj.rO.policies.forEach(function(policy){
						if (slasFromUI.indexOf(policy['name']) === -1){
							if (policy['name'].startsWith(oFromNg['subtype'] + '_')){
								policyIdArray.push(policy['id'])
								policyInfoMap.set(policy['id'], policy['name']);
								ngp.logMsg('info', '		Catalog SLAs resources to delete: ' + policy['name']);
							}
						}
					});

					if (policyIdArray.length > 0){						
						ngp.setRequestState(step.reqId, ecxSessionId, 'deleteCatalogPolicyAndJobFailure', false);
						deleteCatalogPolicyAndJob(step, ecxSessionId, policyIdArray, policyInfoMap, getJobsResObj.rO.jobs, function(deleteCatalogPolicyAndJobResObj){
							var catalogPolicyJobDeletionState = ngp.getRequestState(step.reqId, ecxSessionId, 'deleteCatalogPolicyAndJobFailure');
							if (catalogPolicyJobDeletionState){
								ngp.logMsg('error', '		Failure in deletion of catalog policy and job ==>');
								var failResult = {
									statusCode: 500,
									response: {
										rc: 'Failure in deletion of unused catalog SLA resources.'
									}	
								};
								ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);
							}else{
								var successResult = {
									statusCode: 200										
								};															
								ngp.setStepSuccessResult(step, funcStream, ecxSessionId, oFromNg, cb, successResult);
							}			
						});						
					}else{
						var successResult = {
							statusCode: 200										
						};															
						ngp.setStepSuccessResult(step, funcStream, ecxSessionId, oFromNg, cb, successResult);						
					}					
				}
			});
		}
	});
}

/**
 * Delete catalog policy and job
 * 
 * @param step
 * @param ecxSessionId 
 * @param policyToDeleteArray 
 * @param policyToDeleteMap 
 * @param jobs 
 * @param deleteCatalogPolicyAndJobCb 
 */
function deleteCatalogPolicyAndJob(step, ecxSessionId, policyToDeleteArray, policyToDeleteMap, jobs, deleteCatalogPolicyAndJobCb){
	if (policyToDeleteArray.length < 1){
		ngp.logMsg('info', '	Done deleting associated policies and jobs of catalog');
		ngp.addDoubleSpace();
		deleteCatalogPolicyAndJobCb();
	}else{
		ngp.addDoubleSpace();
		ngp.logMsg('info', '	Deleting Policy/Job for catalog ==>');
		ngp.addDoubleSpace();
	}
	
	var policyId = policyToDeleteArray.shift();
	var policyName = policyToDeleteMap.get(policyId);

	var jobId = null;
	var jobName = null;

	for (var i = 0; i < jobs.length ; i++) {
		if (jobs[i]['name'] == policyName){
			jobId = jobs[i]['id'];
			jobName = jobs[i]['name'];
			ngp.logMsg('info', '			Found job associated with policy ' + policyName + '[' + policyId + '] ;deleting job ' + jobName + '[' +  jobId + ']');
			break;
		}
	}
	
	if (jobId){
		ngp.ngpCoreHelper.deleteJob(ecxSessionId, jobId, function(deleteJobCb){
			if ( deleteJobCb.e || ( deleteJobCb.response.statusCode != 204 && deleteJobCb.response.statusCode != 404 && deleteJobCb.response.statusCode != 500 ) ){
				ngp.logMsg('error', '    		Deletion of job ' + jobName + '[' + jobId +  '] failed because of error = ' + JSON.stringify(deleteJobCb.rO));
				ngp.setRequestState(step.reqId, ecxSessionId, 'deleteCatalogPolicyAndJobFailure', true);
				deleteCatalogPolicyAndJob(step, ecxSessionId, policyToDeleteArray, policyToDeleteMap, jobs, deleteCatalogPolicyAndJobCb);
			}else{
				ngp.ngpCoreHelper.deletePolicy(ecxSessionId, policyId, function(deletePolicyCb){
					if ( deletePolicyCb.e || ( deletePolicyCb.response.statusCode != 204 && deletePolicyCb.response.statusCode != 404 && deletePolicyCb.response.statusCode != 500 ) ){
						ngp.logMsg('error', '    		Deletion of policy ' + policyName + '[' + policyId  + '] failed because of error = ' + JSON.stringify(deletePolicyCb.rO));
						ngp.setRequestState(step.reqId, ecxSessionId, 'deleteCatalogPolicyAndJobFailure', true);
						deleteCatalogPolicyAndJob(step, ecxSessionId, policyToDeleteArray, policyToDeleteMap, jobs, deleteCatalogPolicyAndJobCb);
					}else{
						deleteCatalogPolicyAndJob(step, ecxSessionId, policyToDeleteArray, policyToDeleteMap, jobs, deleteCatalogPolicyAndJobCb);
					}
				});
			}
		});
	}
}

/**
 * Rollback created Policy/Job pairs incase of failure
 *
 * @param finishedPjEntries
 * @param ecxSessionId
 * @param rolledbackPolJobsAr
 * @param policiesRolledbackCb
 */
function rollbackCatalogPolicyAndJob(finishedPjEntries, ecxSessionId, rolledbackPolJobsAr, policiesRolledbackCb)
{
	if ( finishedPjEntries.length < 1 ){
		policiesRolledbackCb();
	}else{
		var pjEntry = finishedPjEntries.shift();

		if ( ! pjEntry.policyCreated ){
			ngp.logMsg('warn', "		Policy/Job " + pjEntry.policyName + " were never created; nothing to rollback");
			rollbackCatalogPolicyAndJob(finishedPjEntries, ecxSessionId, rolledbackPolJobsAr, policiesRolledbackCb);
		}else{
			ngp.ngpCoreHelper.deleteJob(ecxSessionId, pjEntry.jobId, function(jobDelResObj){
				pjEntry.jobRollbackAttempted = true;

				if (jobDelResObj.e || ( jobDelResObj.response.statusCode != 204 && jobDelResObj.response.statusCode != 404 && jobDelResObj.response.statusCode != 500 )) {
					ngp.logMsg('warn', "		Rollback deletion of catalog protection job " + pjEntry.jobName + " failed because of error = " + JSON.stringify(jobDelResObj.rO));
				}else{
					pjEntry.jobRollbackSuccessful = true;

					ngp.ngpCoreHelper.deletePolicy(ecxSessionId, pjEntry.policyId, function(deletePolicyResObj){
						if (deletePolicyResObj.e || ( deletePolicyResObj.response.statusCode != 204 && deletePolicyResObj.response.statusCode != 404 && deletePolicyResObj.response.statusCode != 500)){
							ngp.logMsg('warn', "		Rollback deletion of catalog protection policy " + pjEntry.policyName + " failed because of error = " + JSON.stringify(deletePolicyResObj.rO));
							rolledbackPolJobsAr.push(pjEntry);
							rollbackCatalogPolicyAndJob(finishedPjEntries, ecxSessionId, rolledbackPolJobsAr, policiesRolledbackCb);
						}else{
							pjEntry.policyRollbackSuccessful = true;
							rolledbackPolJobsAr.push(pjEntry);
							rollbackCatalogPolicyAndJob(finishedPjEntries, ecxSessionId, rolledbackPolJobsAr, policiesRolledbackCb);
						}
					});
				}
			});
		}
	}
}

exports.catalogApplySlasFuncSteps = catalogApplySlasFuncSteps;
exports.catalogApplySlasFuncs = catalogApplySlasFuncs;