if (!('contains' in String.prototype)) {
    String.prototype.contains = function(str, startIndex) {
        return ''.indexOf.call(this, str, startIndex) !== -1;
    };
}

if (typeof String.prototype.startsWith != 'function') {
    String.prototype.startsWith = function(prefix) {
        return this.slice(0, prefix.length) == prefix;
    };
}

if (typeof String.prototype.endsWith != 'function') {
    String.prototype.endsWith = function(suffix) {
        return this.slice(-suffix.length) == suffix;
    };
}

var ngp = require('../ecxngp');
var catalogProvider = require('./catalog-provider');

var catalogRouter = ngp.express.Router();

const cataloglicitQueriesForPost = {
    action: ['applySLAPolicies','restore']
};

//middleware which is specific to the vadp router
catalogRouter.use(function timeLog(req, res, next) {
    ngp.logMsg('info','Handling /ngp/catalog request Time(%s) METHOD(%s)', Date.now().toString(), req.method.toString());
    next();
});

/*
  ==========================
  ADD CATALOG ENDPOINTS HERE
  ==========================
*/

/*
    ------------------------------------------------------------------------------
    Expected sample POST request body format from UI for action "applySLAPolicies"
    ------------------------------------------------------------------------------
        {
            "subtype": "catalog",
            "version": "1.0",
            "resources": [{
                "href": "http://localhost:8082/api/endeavour/catalog/system",
                "id": "1000",
                "metadataPath": "/"
            }],
            "slapolicies": [{
                "href": "http://localhost:8082/api/spec/storageprofile/2101",
                "id": "2101",
                "name": "test1-sla1"
            }, {
                "href": "http://localhost:8082/api/spec/storageprofile/2102",
                "id": "2102",
                "name": "test2-sla1"
            }, {
                "href": "http://localhost:8082/api/spec/storageprofile/2103",
                "id": "2103",
                "name": "test3-sla1"
            }]
        }

    ---------------------------------------------------------------------
    Expected sample POST request body format from UI for action "restore"
    ---------------------------------------------------------------------


*/
catalogRouter.post('/system', function(req, res) {
    var vadpQueryValidationResult = ngp.validateQs(req, cataloglicitQueriesForPost);

    if ( vadpQueryValidationResult.rc == 2 )
    {
        ngp.logMsg('warn', 'Detected invalid value among keys=' + JSON.stringify(vadpQueryValidationResult.keyVals));
        ngp.returnBadRequestError(res);
        return;
    }

    if ( vadpQueryValidationResult.rc == 0 ){
        ngp.logMsg('warn', 'Detected no query parameters');
        ngp.returnBadRequestError(res);
        return;
    }
    var stepsStream = null;

    if (req.query['action']){        
        if (req.query['action'] == 'applySLAPolicies'){
            ngp.addDoubleSpace();
            ngp.addDoubleSpace();
            ngp.logMsg('info', '++----------------------------------------------------------------------------------------------++');
            ngp.logMsg('info', '++----------------------- STARTING APPLY SLA POLICIES FOR CATALOG ------------------------------++');
            ngp.logMsg('info', '++----------------------------------------------------------------------------------------------++');
            ngp.addDoubleSpace();
            stepsStream = catalogProvider.init(req, res, 'applySLAPolicies');
            ngp.startFunc(req, res, stepsStream);
        }else if (req.query['action'] == 'restore'){
            ngp.addDoubleSpace();
            ngp.addDoubleSpace();
            ngp.logMsg('info', '++----------------------------------------------------------------------------------------------++');
            ngp.logMsg('info', '++---------------------------- STARTING CATALOG RECOVERY ---------------------------------------++');
            ngp.logMsg('info', '++----------------------------------------------------------------------------------------------++');            
            ngp.addDoubleSpace();
            stepsStream = catalogProvider.init(req, res, 'restore');
            ngp.startFunc(req, res, stepsStream);
        }else{
            ngp.returnBadRequestError(res);
            return;
        }
    }
});

module.exports = catalogRouter;