if (!('contains' in String.prototype)) {
	  String.prototype.contains = function(str, startIndex) {
	    return ''.indexOf.call(this, str, startIndex) !== -1;
	  };
	}

if (typeof String.prototype.startsWith != 'function') {
    String.prototype.startsWith = function(prefix) {
        return this.slice(0, prefix.length) == prefix;
    };
}
		 
if (typeof String.prototype.endsWith != 'function') {
    String.prototype.endsWith = function(suffix) {
        return this.slice(-suffix.length) == suffix;
    };
}

var ngp = require('../ecxngp');

/**
 * Delete Report
 * 
 * @param ecxSessionId 
 * @param reportId 
 * @param deleteReportCb 
 */
function deleteReport(ecxSessionId, reportId, deleteReportCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Deleting Report ==> ');
    ngp.logMsg('info', '    reportId(' + reportId + ')');
    var report_api_endpoint = ngp.ngpConstants.REPORT_API + '/' + reportId;
    ngp.sendRequestToEcx('DELETE', ngp.ecxExternal, report_api_endpoint, ecxSessionId, null, function (reportDelResObj) {
       deleteReportCb(reportDelResObj);
    });
}

/**
 * Delete Job
 * 
 * @param ecxSessionId 
 * @param jobId 
 * @param deleteJobCb 
 */
function deleteJob(ecxSessionId, jobId, deleteJobCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Deleting Job ==> ');
    ngp.logMsg('info', '    jobId(' + jobId + ')');
    var job_api_endpoint = ngp.ngpConstants.JOB_API + '/' + jobId;
    ngp.sendRequestToEcx('DELETE', ngp.ecxExternal, job_api_endpoint, ecxSessionId, null, function (jobDelResObj) {
       deleteJobCb(jobDelResObj);
    });
}

/**
 * Delete Policy
 * 
 * @param ecxSessionId 
 * @param policyId 
 * @param deletePolicyCb 
 */
function deletePolicy(ecxSessionId, policyId, deletePolicyCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Deleting Policy ==> ');
    ngp.logMsg('info', '    policyId(' + policyId + ')');
    var policy_api_endpoint = ngp.ngpConstants.POLICY_API + '/' + policyId;
    ngp.sendRequestToEcx('DELETE', ngp.ecxExternal, policy_api_endpoint, ecxSessionId, null, function (policyDelResObj) {
       deletePolicyCb(policyDelResObj);
    });
}

/**
 * Delete Trigger
 * 
 * @param ecxSessionId 
 * @param triggerId 
 * @param deleteTriggerCb 
 */
function deleteTrigger(ecxSessionId, triggerId, deleteTriggerCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Deleting Trigger ==> ');
    ngp.logMsg('info', '    triggerId(' + triggerId + ')');
    var trigger_api_endpoint = ngp.ngpConstants.TRIGGER_API + '/' + triggerId;
    ngp.sendRequestToEcx('DELETE', ngp.ecxExternal, trigger_api_endpoint, ecxSessionId, null, function (triggerDelResObj) {
        deleteTriggerCb(triggerDelResObj);
    });
}

/**
 * Get list of reports
 * 
 * @param ecxSessionId 
 * @param getReportListCb 
 */
function getReportList(ecxSessionId, queryString, getReportListCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Get Report List ==> ');
    var report_api_endpoint = null;
    if( queryString ) {
        report_api_endpoint =  ngp.ngpConstants.REPORT_API + queryString + '&sort=' + ngp.escape('[{"property":"displayName","direction":"ASC"}]');
    }else{
        report_api_endpoint =  ngp.ngpConstants.REPORT_API + '&sort=' + ngp.escape('[{"property":"displayName","direction":"ASC"}]');
    }

    ngp.sendRequestToEcx('GET', ngp.ecxExternal, report_api_endpoint , ecxSessionId, null, function (getReportsResObj) {
        getReportListCb(getReportsResObj);
    });
}

/**
 * Get Report
 *  
 * @param ecxSessionId 
 * @param reportId 
 * @param getReportCb 
 */
function getReport(ecxSessionId, reportId, getReportCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Get Report ==> ');
    var report_api_endpoint = ngp.ngpConstants.REPORT_API + '/' + reportId;
    ngp.sendRequestToEcx('GET', ngp.ecxExternal, report_api_endpoint , ecxSessionId, null, function (getReportResObj) {
        getReportCb(getReportResObj);
    });
}

/**
 * Get Policy
 * 
 * @param ecxSessionId 
 * @param policyId 
 * @param getPolicyCb 
 */
function getPolicy(ecxSessionId, policyId, getPolicyCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Get Policy ==> ');
    var policy_api_endpoint = ngp.ngpConstants.POLICY_API + '/' + policyId;
    ngp.sendRequestToEcx('GET', ngp.ecxExternal, policy_api_endpoint , ecxSessionId, null, function (getPolicyResObj) {
        getPolicyCb(getPolicyResObj);
    });
}

/**
 * Get Jobs
 * 
 * @param ecxSessionId 
 * @param getJobsCb 
 */
function getJobs(ecxSessionId, getJobsCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Get Jobs ==> ');
    ngp.sendRequestToEcx('GET', ngp.ecxExternal, ngp.ngpConstants.JOB_API , ecxSessionId, null, function (getJobsResObj) {
        getJobsCb(getJobsResObj);
    });
}

function getSites(ecxSessionId, getSitesCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Get Sites ==> ');
    ngp.sendRequestToEcx('GET', ngp.ecxExternal, ngp.ngpConstants.SITE_API, ecxSessionId, null, function (getSitesResObj) {
        getSitesCb(getSitesResObj);
    });
}

function getKeys(ecxSessionId, getKeysCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Get Keys ==> ');
    ngp.sendRequestToEcx('GET', ngp.ecxExternal, ngp.ngpConstants.KEY_API, ecxSessionId, null, function (getKeysResObj) {
        getKeysCb(getKeysResObj);
    });
}

function getIdentities(ecxSessionId, getIdentitiesCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Get Identities ==> ');
    ngp.sendRequestToEcx('GET', ngp.ecxExternal, ngp.ngpConstants.IDENTITY_API, ecxSessionId, null, function (getIdentitiesResObj) {
        getIdentitiesCb(getIdentitiesResObj);
    });
}

/**
 * Get Policies
 * 
 * @param ecxSessionId 
 * @param getPolicesCb 
 */
function getPolicies(ecxSessionId, getPolicesCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Get Policies ==> ');
    ngp.sendRequestToEcx('GET', ngp.ecxExternal, ngp.ngpConstants.POLICY_API , ecxSessionId, null, function (getPolicesResObj) {
        getPolicesCb(getPolicesResObj);
    });
}

/**
 * Get Triggers
 * 
 * @param ecxSessionId 
 * @param getTriggerCb 
 */
function getTriggers(ecxSessionId, getTriggerCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Get Triggers ==> ');
    ngp.sendRequestToEcx('GET', ngp.ecxExternal, ngp.ngpConstants.TRIGGER_API, ecxSessionId, null, function(getTriggerResObj){
        getTriggerCb(getTriggerResObj);
    });
}

/**
 * Get Trigger
 * 
 * @param ecxSessionId 
 * @param triggerName 
 * @param getTriggerCb 
 */
function getTrigger(ecxSessionId, triggerName, getTriggerCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Get Trigger ['+ triggerName + '] ==> ');
    var triggerFilter = '[{"property":"name","value":"' + triggerName + '"}]';
    var escFilter = ngp.escape(triggerFilter);
    var trigger_api_endpoint = ngp.ngpConstants.TRIGGER_API + '?filter=' + escFilter;
    ngp.sendRequestToEcx('GET', ngp.ecxExternal , trigger_api_endpoint, ecxSessionId, null, function(getTriggerResObj){
        getTriggerCb(getTriggerResObj);
    });
}


/**
 * Get Smtps
 * 
 * @param ecxSessionId 
 * @param getSmtpsCb 
 */
function getSmtps(ecxSessionId, getSmtpsCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Get Smtps ==> ');
    ngp.sendRequestToEcx('GET', ngp.ecxExternal, ngp.ngpConstants.SMTP_API, ecxSessionId, null, function(getSmtpsResObj){
        getSmtpsCb(getSmtpsResObj);
    });
}

/**
 * Create Trigger
 * 
 * @param ecxSessionId 
 * @param triggerPostBody 
 * @param createTriggerCb 
 */
function createTrigger(ecxSessionId, triggerPostBody, createTriggerCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Creating Trigger ==> ');    
    ngp.sendRequestToEcx('POST', ngp.ecxExternal, ngp.ngpConstants.TRIGGER_API , ecxSessionId, triggerPostBody, function(trigCreateResObj) {
        createTriggerCb(trigCreateResObj);
    });
}

/**
 * Create Policy
 * 
 * @param ecxSessionId 
 * @param policyPostBody 
 * @param createPolicyCb 
 */
function createPolicy(ecxSessionId, policyPostBody, createPolicyCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Creating Policy ==> ');
    ngp.sendRequestToEcx('POST', ngp.ecxExternal, ngp.ngpConstants.POLICY_API, ecxSessionId, policyPostBody, function(policyCreateResObj){
        createPolicyCb(policyCreateResObj);
    });
}

/**
 * Create Job
 * 
 * @param ecxSessionId 
 * @param jobPostBody 
 * @param createJobCb 
 */
function createJob(ecxSessionId, jobPostBody, createJobCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Creating Job ==> ');
    ngp.sendRequestToEcx('POST', ngp.ecxExternal, ngp.ngpConstants.JOB_API, ecxSessionId, jobPostBody , function(jobCreateResObj){
        createJobCb(jobCreateResObj);
    });
}

/**
 * Update Trigger
 * 
 * @param ecxSessionId 
 * @param triggerPutBody 
 * @param updateTriggerCb 
 */
function updateTrigger(ecxSessionId, triggerId, triggerPutBody, updateTriggerCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Updating Trigger ==> ');
    ngp.logMsg('info', '    triggerId(' + triggerId + ')');    
    var trigger_api_endpoint = ngp.ngpConstants.TRIGGER_API + '/' + triggerId;   
    ngp.sendRequestToEcx('PUT', ngp.ecxExternal, trigger_api_endpoint , ecxSessionId, triggerPutBody, function(trigUpdateResObj) {
        updateTriggerCb(trigUpdateResObj);
    });
}

/**
 * Update Policy
 * 
 * @param ecxSessionId 
 * @param policyId 
 * @param policyPostBody 
 * @param updatePolicyCb 
 */
function updatePolicy(ecxSessionId, policyId, policyPostBody, updatePolicyCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Updating Policy ==> ');
    ngp.logMsg('info', '    policyId(' + policyId + ')');    
    var policy_api_endpoint = ngp.ngpConstants.POLICY_API + '/' + policyId;   
    ngp.sendRequestToEcx('PUT', ngp.ecxExternal, policy_api_endpoint , ecxSessionId, policyPostBody, function(policyUpdateResObj) {
        updatePolicyCb(policyUpdateResObj);
    });
}

/**
 * Update Job
 * 
 * @param ecxSessionId 
 * @param jobId 
 * @param jobPostBody 
 * @param updateJobCb 
 */
function updateJob(ecxSessionId, jobId, jobPostBody, updateJobCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Updating Job ==> ');
    ngp.logMsg('info', '    jobId(' + jobId + ')');    
    var job_api_endpoint = ngp.ngpConstants.JOB_API + '/' + jobId;   
    ngp.sendRequestToEcx('PUT', ngp.ecxExternal, job_api_endpoint , ecxSessionId, jobPostBody, function(jobUpdateResObj) {
        updateJobCb(jobUpdateResObj);
    });

}

/**
 * Apply Security Permissions
 * 
 * @param ecxSessionId 
 * @param applySecurityPermissionsPostBody 
 * @param applySecurityPermsCb 
 */
function applySecurityPermissions(ecxSessionId, applySecurityPermissionsPostBody, applySecurityPermsCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Applying Security Permissions ==> ');       
    var security_api_endpoint = ngp.ngpConstants.SECURITY_API + '/permissions?action=add';
    ngp.sendRequestToEcx('POST', ngp.ecxExternal, security_api_endpoint , ecxSessionId, applySecurityPermissionsPostBody , function(applySecurityPermissionsResObj){
        applySecurityPermsCb(applySecurityPermissionsResObj);
    });
}

/**
 * Get Security Permissions
 * 
 * @param ecxSessionId 
 * @param rbacPath 
 * @param getSecurityPermsCb 
 */
function getSecurityPermissions(ecxSessionId, rbacPath, getSecurityPermsCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Getting Security Permissions ==> ');
    ngp.logMsg('info', '    rbacPath(' + rbacPath + ')');
    var security_api_endpoint = ngp.ngpConstants.SECURITY_API + '/permissions?rbacPath=' + rbacPath;
    ngp.sendRequestToEcx('GET', ngp.ecxExternal, security_api_endpoint, ecxSessionId, null, function(getSecurityPermissionsResObj){
        getSecurityPermsCb(getSecurityPermissionsResObj);
    });
}

/**
 * Install VADP proxy
 * 
 * @param  ecxSessionId 
 * @param  oFromNg 
 * @param  installVadpProxyCb 
 */
function installVadpProxy(ecxSessionId, oFromNg, installVadpProxyCb){
    var vadp_api_endpoint = ngp.ngpConstants.VADP_API + '?action=pushinstall';   
    ngp.sendRequestToEcx('POST', ngp.ecxExternal, vadp_api_endpoint , ecxSessionId, oFromNg, function(installVadpProxyResObj) {
        installVadpProxyCb(installVadpProxyResObj);
    });
}

/**
 * Get VADP proxies
 * 
 * @param ecxSessionId 
 * @param getVadpProxiesCb 
 */
function getVadpProxies(ecxSessionId, getVadpProxiesCb){
    ngp.logMsg('info', 'Getting VADP proxies ==> ');
    var vadp_api_endpoint = ngp.ngpConstants.VADP_API + '?installerinfo=false';
    ngp.sendRequestToEcx('GET', ngp.ecxExternal, vadp_api_endpoint, ecxSessionId, null, function(getVadpProxiesResObj) {
        getVadpProxiesCb(getVadpProxiesResObj);
    });
}

/**
 * Register VADP proxy
 * 
 * @param ecxSessionId 
 * @param vadpProxyRegisterPostBody 
 * @param registerVadpProxyCb 
 */
function registerVadpProxy(ecxSessionId, vadpProxyRegisterPostBody, registerVadpProxyCb){
    ngp.logMsg('info', 'Register VADP proxy ==> ');
    //var vadp_api_endpoint = ngp.ngpConstants.VADP_API + '?action=register';
    var vadp_api_endpoint = ngp.ngpConstants.VADP_API;
    ngp.sendRequestToEcx('POST', ngp.ecxExternal, vadp_api_endpoint , ecxSessionId, vadpProxyRegisterPostBody, function(registerVadpProxyResObj) {
        registerVadpProxyCb(registerVadpProxyResObj);
    });
}

function updateVadpProxy(ecxSessionId, vadpProxyUpdatePutBody, stepId, updateVadpProxyCb){
    ngp.logMsg('info', 'Updating VADP proxy ==> ');
    var vadp_api_endpoint = ngp.ngpConstants.VADP_API + '/' + stepId;
    ngp.sendRequestToEcx('PUT', ngp.ecxExternal, vadp_api_endpoint , ecxSessionId, vadpProxyUpdatePutBody, function(updateVadpProxyResObj) {
        updateVadpProxyCb(updateVadpProxyResObj);
    });
}

/**
 * Apply Slas for Catalog
 * 
 * @param ecxSessionId 
 * @param catalogApplySlasPostBody 
 * @param catalogApplySlasCb 
 */
function catalogApplySlas(ecxSessionId, catalogApplySlasPostBody, catalogApplySlasCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Catalog Apply Slas==> ');
    //var catalog_api_endpoint = ngp.ngpConstants.CATALOG_API + '/system?action=applySLAPolicies';
    var catalog_api_endpoint = ngp.ngpConstants.CATALOG_API + '/system?action=applySLAPolicies';
    ngp.sendRequestToEcx('POST', ngp.ecxExternal, catalog_api_endpoint , ecxSessionId, catalogApplySlasPostBody, function(catalogApplySlasResObj) {
        catalogApplySlasCb(catalogApplySlasResObj);
    });
}

/**
 * Get recovery policies for a specific resource type
 * 
 * @param ecxSessionId 
 * @param oFromNg 
 * @param resourceType 
 * @param getRecoveryPolicesCb 
 */
function getRecoveryPolicies(ecxSessionId, oFromNg, resourceType, getRecoveryPolicesCb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'Get [' + resourceType + '] recovery policies ==>');
	var scrubbedNgInput = ngp.scrubPostBody(oFromNg, true);
    ngp.logMsg('warn', "    Input from client for recovery policy in session ID [" + ecxSessionId + "] = " + JSON.stringify(scrubbedNgInput));
    var filter = null;

    switch (resourceType){

        case 'catalog':
            filter = '[{"property":"serviceId","value":"com.syncsort.dp.xsb.serviceprovider.recovery.catalog"},{"property":"subType", "value":"' + oFromNg['subType'] + '"}]';
            break;

        case 'hypervisor':
            filter = '[{"property":"serviceId","value":"com.syncsort.dp.xsb.serviceprovider.recovery.hypervisor"},{"property":"subType", "value":"' + oFromNg['subType'] + '"}]';
            break;            
    }
	
    var escFilter = ngp.escape(filter);
    var policy_api_endpoint = ngp.ngpConstants.POLICY_API + '?filter=' + escFilter;
    ngp.sendRequestToEcx('GET', ngp.ecxExternal, policy_api_endpoint , ecxSessionId, null, function (getRecoveryPolicesResObj) {
        getRecoveryPolicesCb(getRecoveryPolicesResObj);
    });
}


exports.deleteReport = deleteReport;
exports.deleteJob = deleteJob;
exports.deletePolicy = deletePolicy;
exports.deleteTrigger = deleteTrigger;

exports.createJob = createJob;
exports.createPolicy = createPolicy;
exports.createTrigger = createTrigger;
exports.applySecurityPermissions = applySecurityPermissions;

exports.getReport = getReport;
exports.getReportList = getReportList;
exports.getJobs = getJobs;
exports.getSites = getSites;
exports.getKeys = getKeys;
exports.getPolicy = getPolicy;
exports.getIdentities = getIdentities;
exports.getPolicies = getPolicies;
exports.getTriggers = getTriggers;
exports.getSmtps = getSmtps;
exports.getSecurityPermissions = getSecurityPermissions;
exports.getRecoveryPolicies = getRecoveryPolicies;

exports.getTrigger = getTrigger;

exports.updateTrigger = updateTrigger;
exports.updatePolicy = updatePolicy;
exports.updateJob = updateJob;

exports.installVadpProxy = installVadpProxy;
exports.getVadpProxies = getVadpProxies;
exports.registerVadpProxy = registerVadpProxy;
exports.updateVadpProxy = updateVadpProxy;

exports.catalogApplySlas = catalogApplySlas;