if (!('contains' in String.prototype)) {
	  String.prototype.contains = function(str, startIndex) {
	    return ''.indexOf.call(this, str, startIndex) !== -1;
	  };
	}

if (typeof String.prototype.startsWith != 'function') {
    String.prototype.startsWith = function(prefix) {
        return this.slice(0, prefix.length) == prefix;
    };
}
		 
if (typeof String.prototype.endsWith != 'function') {
    String.prototype.endsWith = function(suffix) {
        return this.slice(-suffix.length) == suffix;
    };
}


var ngp = require('./ecxngp');   


var funcSteps = [                                                                                    
	{                                                                                              
		continue: 'next',                                                                            
		name: 'crp_recovmode',                                                                       // figure out onDemand | scheduled
		verb: null,
		path: null,
		ip: null,                                                                                                                                        
		port: null, 
		type: null,
		func: null,                                                                                  // see initialization in regProviderCommon 
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	},     
	{                                                                                                
		continue: 'next',                                                                           
		name: 'crp_ispolicy',
		verb: 'GET',
		path: '/api/endeavour/policy',
		ip: null,                                                                                                                                      
		port: null, 
		type: null,
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	}, 
	{                                                                                                
		continue: 'next',                                                                            
		name: 'crp_getupdatetrigger',
		verb: 'GET',
		path: '/api/endeavour/trigger?filter=[{"property":"name","value":"qqVVqq"}]', 
		ip: null,                                                                                                                                        
		port: null, 
		type: null,
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	},	
	{                                                                                                
		continue: 'next',                                                                            
		name: 'crp_createtrigger',
		verb: 'POST',
		path: '/api/endeavour/trigger',
		ip: null,                                                                                                                                        
		port: null, 
		type: null,
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	}, 	
	{                                                                                               
		continue: 'next',                                                                            // set by previous step [ next | skip | fail ]     
		name: 'crp_create',   
		verb: 'POST',
		path: '/api/endeavour/policy',                                                            
		ip: null, 
		port: null, 
		type: null,
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	}, 
	{                                                                                              // MAY be skipped by create step above
		continue: 'next',                                                                               
		name: 'crp_rollback',   
		verb: null,
		path: null,                                                            
		ip: null, 
		port: null, 
		type: null,
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
    }, 	
];


const crRecovPolStepFuncs = {
	crp_recovmode: function(step, funcStream, ecxSessionId, oFromNg, cb){ 
		ngp.recoveryMode(step, funcStream, ecxSessionId, oFromNg, cb);  
	},    
	crp_ispolicy: function(step, funcStream, ecxSessionId, oFromNg, cb){                           // skipped if recovery mode indicates unscheduled (an on demand job), i.e., don't bother looking for it
		ngp.isRecoveryPolicy(step, funcStream, ecxSessionId, oFromNg, cb);  
	},
	crp_getupdatetrigger: function(step, funcStream, ecxSessionId, oFromNg, cb){          
		getAndUpdateTrigger(step, funcStream, ecxSessionId, oFromNg, cb);      	                    
  },	
	crp_createtrigger: function(step, funcStream, ecxSessionId, oFromNg, cb){          
		ngp.createTrigger(step, funcStream, ecxSessionId, oFromNg, cb);      	                       // called always, but will bail ASAP if no rMode state or ! rMode.scheduled
  },	
	crp_create: function(step, funcStream, ecxSessionId, oFromNg, cb){ 
		createPolicy(step, funcStream, ecxSessionId, oFromNg, cb);     	                             // but for errors, done always, either scheduled | on demand (run now)	                               
	},
	crp_rollback: function(step, funcStream, ecxSessionId, oFromNg, cb){ 
		rollback(step, funcStream, ecxSessionId, oFromNg, cb);     		                               
    },	
};


exports.init = init;


function init(req, res, providerType)                                    
{
	var ngState = ngp.getSessionState(req, res);

	if ( ngState.stale )                                                                           // FTI we got bounced while client remained up. AND response already sent.
		return;                                                                                      // !!! R U CERTAIN THAT RESPONSE SENT IN ALL CASES HERE?

	var ecxSid = ngState.ecxSid;
	res.ecxSid = ecxSid;

	var stepsStream = JSON.parse(JSON.stringify(funcSteps));                                    

	stepsStream.forEach(function(step){
		step.ip = ngp.localIp;
		step.port = ngp.ecxExternal;

		step.func = crRecovPolStepFuncs[step.name];                                                  // hook up the step's real workhorse

		step.cb = function(ecxSid, data){ 
			//@#@ ngp.logMsg('info', "Finished step '" + ngState.currentFunc + '(' + this.name + ")' in session ID '" + ecxSid + "'");
			ngp.logMsg('info', "Finished step '" + ngp.getCurrentFunc(step.reqId, ecxSid) + '(' + this.name + ")' in session ID '" + ecxSid + "'");

			if ( data )
			{
				ngp.logMsg('debug', "Data returned by step '" + ngState.currentFunc + '(' + this.name + ")' in session ID '" + ecxSid + "'=" + JSON.stringify(data));
			}
			
			/* !! KEEP; see block comment below
			if ( step.cb.rollback )
			{
				ngp.logMsg('warn', 'AMAZING');
				step.cb.rollback();
			}
			*/
		};


		/* KEEP: EXAMPLE OF HOW TO PIGGY-BACK ON THE *STEP* CALLBACK. Use case: generally, triggers are created first and if that fails, then there's nothing to roll back because policy/job pair would come
		   next. But if you want to deviate from that sequence, e.g., policy, job, trigger, then in order to keep a GENERIC createTrigger (one unaware of need for rollback) in ecxngp, we need a facility like
		   this to hook the failure in createTrigger at step end and implement rollback - or anything else.

		if ( step.name == 'XXXXcrp_createtrigger' )
		{
			step.cb.rollback = function(ecxSid, data){
				ngp.logMsg('info', "FAR OUT!");
				process.exit(2);
			};
		}
		*/

	});

	ngp.createRequestStore(stepsStream, ecxSid, req.reqId, 'create_' + 'hypervisorrecoverypolicy'); 

	ngState.currentFunc = 'create_' + 'hypervisorrecoverypolicy';                                 
	return stepsStream;
}

/*
	The aporia-inducing logic in this case is as follows:

	a. if the client's POST body contains no .trigger property, then cause the trigger creation step to be skipped and drive the step logic. Otherwise:
	b. GET from SPP the triggers array containing, perhaps, the triggers belonging to this restore job
	c. if that array is empty, then create the trigger in keeping with the intent expressed by the presence of oFromNg.trigger
	d. if that array is not empty (it contains exactly 1 trigger), then update it.
*/

function getAndUpdateTrigger(step, funcStream, ecxSessionId, oFromNg, cb)
{
		if ( ! oFromNg.trigger )                                                                // FTI we're done here, BUT first cause skip of trigger creation
		{
				if ( funcStream.length > 0 )                                                             
						funcStream[0].continue = 'skip';  
			
				step.cb(ecxSessionId, null);                                                                
				ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb);     
				return;                    
		}

		step.path = step.path.replace('qqVVqq', require('querystring').escape(oFromNg.name));   // trigger name is restore job name

		var currentStepPath = step.path;
		var currentStepPathSplitForFilter = currentStepPath.split('=');
		var currentStepPathFilter = currentStepPathSplitForFilter[1];
		
		var fixedFilterOption = ngp.escape(currentStepPathFilter);
		var fixedStepPath = currentStepPathSplitForFilter[0] + '=' + fixedFilterOption;
				
		ngp.sendRequestToEcx(step.verb, ngp.ecxExternal, fixedStepPath, ecxSessionId, null, function(triggerObj){     // get 1 trigger
			if ( triggerObj.e || (triggerObj.response.statusCode != 200 && triggerObj.response.statusCode != 404) ) // serious error or status represents an error
			{
				var failResult = ngp.getGenericEcxErrorResponse(triggerObj, ecxSessionId, step);
				ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false); 
			}   
			else if ( triggerObj.rO.triggers.length < 1 )
			{
					ngp.logMsg('debug', "No trigger found for job=" + oFromNg.name);
					step.cb(ecxSessionId, null);                                                                
					ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb); 
			}
			else                                                                                  // EHI found trigger, update it, SKIP CREATE STEP
			{
				ngp.logMsg('debug', "Found and will update trigger for job=" + oFromNg.name);

				var putBody = {  
						name: oFromNg.name,
						type: "Schedule",
						properties:oFromNg.trigger,
			  };

				var triggerId = triggerObj.rO.triggers[0].id;

				ngp.sendRequestToEcx('PUT', ngp.ecxExternal, '/api/endeavour/trigger/' + triggerId, ecxSessionId, putBody, function(putTrigO){    // update trigger
					if ( putTrigO.e || putTrigO.response.statusCode != 200 )
					{
							var failResult = ngp.getGenericEcxErrorResponse(putTrigO, ecxSessionId, step);
							ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false); 
					}
					else
					{
							if ( funcStream.length > 0 )                                                             
									funcStream[0].continue = 'skip';  

							step.cb(ecxSessionId, null);                                                                
							ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb);                                									
					}
				});
		}
	});			
}


var createPolicy = function(step, funcStream, ecxSessionId, oFromNg, cb)                           
{
    var crPolicySteps = [];                                                                                                                                         
	
		var rollback = {
			policies: [],
			jobs: [],
			};

		var triggerId = ngp.getRequestState(step.reqId, ecxSessionId, 'triggerId');                                 // from earlier step

		var rMode = ngp.getRequestState(step.reqId, ecxSessionId, 'rMode');      
	
		if ( ! rMode.scheduled )
			{
			oFromNg.name = 'onDemandRestore_' + (new Date).getTime();
		}

//!! NB: *not* setting description either at top level nor within subpolicy element

    var hyrpolCommonTpl = JSON.parse(JSON.stringify(ngp.hyrpolTpl));                              // this is common part of hypervisor recovery POST body
    hyrpolCommonTpl.name = oFromNg.name;
    hyrpolCommonTpl.subType = oFromNg.subType;
    hyrpolCommonTpl.spec = oFromNg.spec;
	hyrpolCommonTpl.spec.subpolicy[0].name = oFromNg.name;
	hyrpolCommonTpl.script = oFromNg.script;
	
    var pjEntry = {
        policyName: oFromNg.name,
        policyCreated: false,
        jobCreated: false,
        triggerId: triggerId,                                                                     // if non-null, then trigger was created
				policyPostBody: hyrpolCommonTpl,
        policyRollbackAttempted: false,
        policyRollbackSuccessful: false,
        jobEntity: null,                                                                          // what ECX returned
        jobRollbackAttempted: false,
				jobRollbackSuccessful: false,	                                                         		// NB: set on 204 or 404 from DELETE
				triggerRollbackAttempted: false,
				triggerRollbackSuccessful: false,
				entitiesCreated: 0,                                                                       // we do not count trigger we may have created in an earlier step
    };

		var crSummaryResults = [];                                                                    // will contain results of successful policy/job creation in pairs; goes back to client
	
		if ( triggerId )
			//@#@ crSummaryResults.push(ngp.getEndpointState(ecxSessionId, 'trigger'));                       // pro forma: return to caller each of the entities that we create, in order of creation
			crSummaryResults.push(ngp.getRequestState(step.reqId, ecxSessionId, 'trigger'));                       // pro forma: return to caller each of the entities that we create, in order of creation
			

		createPolicyAndJob(pjEntry, step, ecxSessionId, crSummaryResults, triggerId, function(failResult){
			if ( pjEntry.entitiesCreated != 2 )                                                         // NB: FTI failed to create at least one of PJ
			//if ( true )                                                                               //!! swap with previous 'if' to force rollback
			{
				ngp.logMsg('warn', 'Detected failure of policy and job creation in session ' + ecxSessionId + '; attempting rollback'); 
				ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false); 				

				//@#@ ngp.setEndpointState(ecxSessionId, 'rollbackState', pjEntry);
//				ngp.setRequestState(step.reqId, ecxSessionId, 'rollbackState', pjEntry);
//
//				step.cb(ecxSessionId, null);                                                                
//				ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false); 
			}
			else                                                                                        // EHI PJ created; skip rollback; launch on demand job if need be
			{
				if ( funcStream.length > 0 )                                                             
					funcStream[0].continue = 'skip';  

				if ( rMode.scheduled )
				{
						var successResult = {
						statusCode: 201,
						response: crSummaryResults,                                                           // [triggerObj (if we created one), policyObj, jobObj]
					};
					
						ngp.setStepSuccessResult(step, funcStream, ecxSessionId, oFromNg, cb, successResult);	
				}
				else                                                                                      // EHI run the job now  
				{
					var startLink = pjEntry.jobEntity.links.start.href;                                     // one of our internal links should not need escaping
												
					ngp.sendRequestToEcx(step.verb, ngp.ecxExternal, startLink, ecxSessionId, null, function(startObj){
						if ( startObj.e || ( startObj.response.statusCode != 201 && startObj.response.statusCode != 200 ) )    // FTI failed to start the on demand recovery job. NB: NO ROLLBACK in this case
						{
							var failResult = ngp.getGenericEcxErrorResponse(startObj, ecxSessionId, step);
							ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false); 
						} 
						else                                                                                  // !! REVISIT; should I at least return this in an array? see note 193
						{
							var successResult = {
								statusCode: 201,
								response: startObj.rO,    
							};
							
							ngp.setStepSuccessResult(step, funcStream, ecxSessionId, oFromNg, cb, successResult); 
						}
					});
				}
			}		
		});
}


function createPolicyAndJob(crPolicyStep, driverStep, ecxSessionId, crSummaryResults, triggerId, policiesCreatedCb)   
{
	ngp.sendRequestToEcx(driverStep.verb, ngp.ecxExternal, driverStep.path, ecxSessionId, crPolicyStep.policyPostBody, function(hyrprObj){    // Create POLICY
		if ( hyrprObj.e || hyrprObj.response.statusCode != 201 )                                 // failed to create the ECX policy  
		{
			var failResult = {
				statusCode: hyrprObj.response.statusCode,
				response: {rc: (JSON.stringify(hyrprObj.rO.description).replace(/\"/g, ""))}																
			};

			policiesCreatedCb(failResult);			                                             // return to rollback first jobs, then policies
		}   
		else                                                                                     // the sole policy was created
		{
			crSummaryResults.push(hyrprObj.rO);                                                  // save created policy object to give back to caller

			crPolicyStep.policyId = hyrprObj.rO.id;
			crPolicyStep.policyCreated = true;
			crPolicyStep.entitiesCreated++;

			ngp.logMsg('debug', "Created hypervisor recovery policy=" + JSON.stringify(hyrprObj.rO));  
			var jobTplClone = JSON.parse(JSON.stringify(ngp.jobTpl));

			jobTplClone.name = crPolicyStep.policyName;
			jobTplClone.description += crPolicyStep.policyName;
			jobTplClone.policyId = hyrprObj.rO.id;

			if ( triggerId )
				jobTplClone.triggerIds.push(triggerId);

			ngp.sendRequestToEcx(driverStep.verb, ngp.ecxExternal, '/api/endeavour/job', ecxSessionId, jobTplClone, function(hyrpjObj){    // create JOB
				if ( hyrpjObj.e || hyrpjObj.response.statusCode != 201 )
				{
					var failResult = ngp.getGenericEcxErrorResponse(hyrpjObj, ecxSessionId, driverStep);
					policiesCreatedCb(failResult);
				}
				else
				{
					crSummaryResults.push(hyrpjObj.rO);                                          // save to give back to caller 
					crPolicyStep.jobEntity = hyrpjObj.rO;                                        // need this in case on demand, via its start link
					crPolicyStep.jobId = hyrpjObj.rO.id;
					crPolicyStep.jobCreated = true;
					crPolicyStep.entitiesCreated++;

					ngp.logMsg('debug', "Created hypervisor recovery job=" + JSON.stringify(hyrpjObj.rO));  
					
					policiesCreatedCb();
				}
			});
		}
	});		
}


/*
  rollback always attempts to delete job, policy, and trigger in that order regardless of whether or not they were created. When one of these entities was not created its ID proeprty in rollbackState will
  be 'undefined'. Attempting to delete an entity with ID set to 'undefined' returns a 500. We only log an error, a failure to rollback, if the DELETE's response.statusCode is neither 204, nor 404, nor 500. 
  
  Otherwise we treat the rollback as successful. In the 500 case caused by 'undefined', if the entity was never created then it's OK to pretend that it was rolled back...
*/

function rollback(step, funcStream, ecxSessionId, oFromNg, cb)              
{
	//@#@ var rollbackState = ngp.getEndpointState(ecxSessionId, 'rollbackState');
	var rollbackState = ngp.getRequestState(step.reqId, ecxSessionId, 'rollbackState');

	var rollbackResult = {};

// NB: to simplify this logic, we always attempt to delete the job, then the policy. If the job wasn't created, we'll simply get a 404. 
	
	ngp.sendRequestToEcx('DELETE', ngp.ecxExternal, '/api/endeavour/job/' + rollbackState.jobId, ecxSessionId, null, function(jrObj){    // !! DELETE JOB, then its POLICY
		rollbackState.jobRollbackAttempted = true;
		
		if ( jrObj.e || ( jrObj.response.statusCode != 204 && jrObj.response.statusCode != 404 && jrObj.response.statusCode != 500 ) )    // NB: tolerate, perhaps unwisely, a not found !!! IF PASSING NULL .id then add 404 here
		{
			ngp.logMsg('warn', "Rollback deletion of job " + rollbackState.policyName + " failed because of error=" + JSON.stringify(jrObj.rO));  
			rollbackResult.jobId = rollbackState.jobId;
		}
		else
			rollbackState.jobRollbackSuccessful = true;

// FTI job may or may not have been deleted. Now nuke its policy.

		ngp.sendRequestToEcx('DELETE', ngp.ecxExternal, '/api/endeavour/policy/' + rollbackState.policyId, ecxSessionId, null, function(prObj){    // delete policy
			rollbackState.policyRollbackAttempted = true;				
			
			if ( prObj.e || ( prObj.response.statusCode != 204 && prObj.response.statusCode != 404 && prObj.response.statusCode != 500 ) ) 
			{
				ngp.logMsg('warn', "Rollback deletion of policy " + rollbackState.policyName + " failed because of error=" + JSON.stringify(jrObj.rO));  
				rollbackResult.policyId = rollbackState.policyId;
			}
			else
			{
				rollbackState.policyRollbackSuccessful = true;
			}

			ngp.sendRequestToEcx('DELETE', ngp.ecxExternal, '/api/endeavour/trigger/' + rollbackState.triggerId, ecxSessionId, null, function(trObj){
				rollbackState.triggerRollbackAttempted = true;	

				if ( trObj.e || ( trObj.response.statusCode != 204 && trObj.response.statusCode != 404 && trObj.response.statusCode != 500 ) ) 
				{
					ngp.logMsg('warn', "Rollback deletion of trigger for policy " + rollbackState.policyName + " failed because of error=" + JSON.stringify(trObj.rO));  
					rollbackResult.triggerId = rollbackState.triggerId;					
				}
				else
				{
					rollbackState.triggerRollbackSuccessful = true;
				}
				
				var result = {
					statusCode: 500,  //!! IS THIS FINAL STATUS FOR CREATION FAILURE? MAYBE IT SHOULDN'T BE : MAYBE CREATEPOLICY'S results, set via intermediate status, should govern
					                  //!! AND HERE  SIMPLY TRY TO UNF..K THINGS AND JUST WRITE TO LOG....
					response: {
						failedRollbacks: rollbackResult,
					}, 
				};

				ngp.logMsg('warn', "FINAL RBS=" + JSON.stringify(rollbackState));  
				ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, result, false); 				
			});
		});
	});		
}

