if (!('contains' in String.prototype)) {
  String.prototype.contains = function(str, startIndex) {
    return ''.indexOf.call(this, str, startIndex) !== -1;
  };
}

if (typeof String.prototype.startsWith != 'function') {
  String.prototype.startsWith = function(prefix) {
      return this.slice(0, prefix.length) == prefix;
  };
}
   
if (typeof String.prototype.endsWith != 'function') {
  String.prototype.endsWith = function(suffix) {
      return this.slice(-suffix.length) == suffix;
  };
}


var ngp = require('./ecxngp');   

//!! create restore file policy is always an 'on demand' (unscheduled) case. Thus we always want to create the policy and job (no trigger in this case)
var funcSteps = [                                                                                    
  {                                                                                                
    continue: 'next',                                                                                 
    name: 'crp_create',   
    verb: 'POST',
    path: '/api/endeavour/policy',                                                            
    ip: null, 
    port: null, 
    type: null,
    cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
  }, 
  {                                                                                                // MAYBE skipped by create step above, maybe not, i.e., let it always run and decide if there's RB to do
    continue: 'next',                                                                               
    name: 'crp_rollback',   
    verb: null,
    path: null,                                                            
    ip: null, 
    port: null, 
    type: null,
    cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
    }, 	
];


const crRestoreFilePolStepFuncs = {
    crp_create: function(step, funcStream, ecxSessionId, oFromNg, cb){ 
      createPolicy(step, funcStream, ecxSessionId, oFromNg, cb);     	                             // but for errors, done always, either scheduled | on demand (run now)	                               
    },
    crp_rollback: function(step, funcStream, ecxSessionId, oFromNg, cb){ 
      rollback(step, funcStream, ecxSessionId, oFromNg, cb);     		                               
    },	
};


exports.init = init;


function init(req, res, providerType)                                    
{
    var ngState = ngp.getSessionState(req, res);

    if ( ngState.stale )                                                                             // FTI we got bounced while client remained up. AND response already sent.
      return;                                                                                     

    var ecxSid = ngState.ecxSid;
    res.ecxSid = ecxSid;

    var stepsStream = JSON.parse(JSON.stringify(funcSteps));                                    

    stepsStream.forEach(function(step){
      step.ip = ngp.localIp;
      step.port = ngp.ecxExternal;

      step.func = crRestoreFilePolStepFuncs[step.name];                                              

      step.cb = function(ecxSid, data){ 
			//@#@ ngp.logMsg('info', "Finished step '" + ngState.currentFunc + '(' + this.name + ")' in session ID '" + ecxSid + "'");
		  	ngp.logMsg('info', "Finished step '" + ngp.getCurrentFunc(step.reqId, ecxSid) + '(' + this.name + ")' in session ID '" + ecxSid + "'");

        if ( data )
        {
          ngp.logMsg('debug', "Data returned by step '" + ngState.currentFunc + '(' + this.name + ")' in session ID '" + ecxSid + "'=" + JSON.stringify(data));
        }
      };
  });

  ngp.createRequestStore(stepsStream, ecxSid, req.reqId, 'create_' + 'restorefilepolicy');      

  ngState.currentFunc = 'create_' + 'restorefilepolicy';                                 
  return stepsStream;
}


var createPolicy = function(step, funcStream, ecxSessionId, oFromNg, cb)                           
{
  var scrubbedNgInput = ngp.scrubPostBody(oFromNg, true);
  ngp.logMsg('warn', "Input from client for create file restore policy in session ID='" + ecxSessionId + "'=" + JSON.stringify(scrubbedNgInput));
  
  var crPolicySteps = [];                                                                                                                                         

  var rollback = {
    policies: [],
    jobs: [],
  };

  oFromNg.name = 'onDemandRestoreFile_' + (new Date).getTime();

  var rfpolTpl = JSON.parse(JSON.stringify(ngp.rfpolTpl));                                      // skeletal POST body
  rfpolTpl.name = oFromNg.name;
  rfpolTpl.subType = oFromNg.subType;
  rfpolTpl.spec = oFromNg.spec;
  rfpolTpl.spec.notification = [];
  rfpolTpl.spec.subpolicy[0].name = oFromNg.name;
  rfpolTpl.spec.subpolicy[0].description = "";                                                  //!! no idea
  rfpolTpl.spec.subpolicy[0].type = "restore";    
  rfpolTpl.spec.subpolicy[0].option.autocleanup = true;
  rfpolTpl.spec.subpolicy[0].option.continueonerror = false;
  rfpolTpl.spec.subpolicy[0].option.protocolpriority = 'iSCSI';

  var pjEntry = {
      policyName: oFromNg.name,
      policyCreated: false,
      jobCreated: false,
      policyPostBody: rfpolTpl,
      policyRollbackAttempted: false,
      policyRollbackSuccessful: false,
      jobEntity: null,                                                                          // what ECX returned
      jobRollbackAttempted: false,
      jobRollbackSuccessful: false,	                                                         		// NB: set on 204 or 404 from DELETE

      entitiesCreated: 0,                                                                       // 2 if all cool
  };

  var crSummaryResults = [];                                                                    // will contain results of successful policy/job creation in pairs; goes back to client

   createPolicyAndJob(pjEntry, step, ecxSessionId, crSummaryResults, function(failResult){
    if ( pjEntry.entitiesCreated != 2 )                                                         // NB: FTI failed to create at least one of PJ
    //if ( true )                                                                               //!! swap with previous 'if' to force rollback
    {
      ngp.logMsg('warn', 'Detected failure of policy and job creation in session ' + ecxSessionId + '; attempting rollback'); 
      
      //@#@ ngp.setEndpointState(ecxSessionId, 'rollbackState', pjEntry);
      ngp.setRequestState(step.reqId, ecxSessionId, 'rollbackState', pjEntry);

      step.cb(ecxSessionId, null);                                                                
      ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb);                                // let next step run
    }
    else                                                                                        // EHI PJ created; skip rollback; launch on demand job
    {
      if ( funcStream.length > 0 )                                                             
        funcStream[0].continue = 'skip';  

      var startLink = pjEntry.jobEntity.links.start.href;                                       // one of our internal links should not need escaping
                    
      ngp.sendRequestToEcx(step.verb, ngp.ecxExternal, startLink, ecxSessionId, null, function(startObj){
        if ( startObj.e || ( startObj.response.statusCode != 201 && startObj.response.statusCode != 200 ) )    // FTI failed to start the on demand recovery job. NB: NO ROLLBACK in this case
        {
          var failResult = ngp.getGenericEcxErrorResponse(startObj, ecxSessionId, step);
          ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false); 
        } 
        else                                                                                    // !! REVISIT; should I at least return this in an array? see note 193
        {
          var successResult = {
            statusCode: 201,
            response: startObj.rO,    
          };
          
          ngp.setStepSuccessResult(step, funcStream, ecxSessionId, oFromNg, cb, successResult); 
        }
      });
    }		
  });
}


function createPolicyAndJob(crPolicyStep, driverStep, ecxSessionId, crSummaryResults, policiesCreatedCb)   
{
    ngp.sendRequestToEcx(driverStep.verb, ngp.ecxExternal, driverStep.path, ecxSessionId, crPolicyStep.policyPostBody, function(hyrprObj){    // Create POLICY
      if ( hyrprObj.e || hyrprObj.response.statusCode != 201 )                                      // failed to create the ECX policy  
      {
        var failResult = ngp.getGenericEcxErrorResponse(hyrprObj, ecxSessionId, driverStep);
        policiesCreatedCb(failResult);			                                                        // return to rollback first jobs, then policies
      }   
      else                                                                                          // the sole policy was created
      {
        crSummaryResults.push(hyrprObj.rO);                                                         // save created policy object to give back to caller

        crPolicyStep.policyId = hyrprObj.rO.id;
        crPolicyStep.policyCreated = true;
        crPolicyStep.entitiesCreated++;

        ngp.logMsg('debug', "Created hypervisor restore file policy=" + JSON.stringify(hyrprObj.rO));  
        var jobTplClone = JSON.parse(JSON.stringify(ngp.jobTpl));

        jobTplClone.name = crPolicyStep.policyName;
        jobTplClone.description += crPolicyStep.policyName;
        jobTplClone.policyId = hyrprObj.rO.id;

        ngp.sendRequestToEcx(driverStep.verb, ngp.ecxExternal, '/api/endeavour/job', ecxSessionId, jobTplClone, function(hyrpjObj){    // create JOB
          if ( hyrpjObj.e || hyrpjObj.response.statusCode != 201 )
          {
            var failResult = ngp.getGenericEcxErrorResponse(hyrpjObj, ecxSessionId, driverStep);
            policiesCreatedCb(failResult);
          }
          else
          {
            crSummaryResults.push(hyrpjObj.rO);                                          // save to give back to caller 
            crPolicyStep.jobEntity = hyrpjObj.rO;                                        // need this in case on demand, via its start link
            crPolicyStep.jobId = hyrpjObj.rO.id;
            crPolicyStep.jobCreated = true;
            crPolicyStep.entitiesCreated++;

            ngp.logMsg('debug', "Created hypervisor restore file job=" + JSON.stringify(hyrpjObj.rO));  
            
            policiesCreatedCb();
          }
        });
      }
    });		
}


/*
  rollback always attempts to delete job and policy regardless of whether or not they were created. When one of these entities was not created its ID proeprty in rollbackState will
  be 'undefined'. Attempting to delete an entity with ID set to 'undefined' returns a 500. We only log an error, a failure to rollback, if the DELETE's response.statusCode is neither 204, nor 404, nor 500. 

  Otherwise we treat the rollback as successful. In the 500 case caused by 'undefined', if the entity was never created then it's OK to pretend that it was rolled back...
*/

function rollback(step, funcStream, ecxSessionId, oFromNg, cb)              
{
    //@#@ var rollbackState = ngp.getEndpointState(ecxSessionId, 'rollbackState');
    var rollbackState = ngp.getRequestState(step.reqId, ecxSessionId, 'rollbackState');

    var rollbackResult = {};

    // NB: to simplify this logic, we always attempt to delete the job, then the policy. If the job wasn't created, we'll simply get a 404. 

    ngp.sendRequestToEcx('DELETE', ngp.ecxExternal, '/api/endeavour/job/' + rollbackState.jobId, ecxSessionId, null, function(jrObj){    // !! DELETE JOB, then its POLICY
      rollbackState.jobRollbackAttempted = true;
      
      if ( jrObj.e || ( jrObj.response.statusCode != 204 && jrObj.response.statusCode != 404 && jrObj.response.statusCode != 500 ) )    // NB: tolerate, perhaps unwisely, a not found !!! IF PASSING NULL .id then add 404 here
      {
        ngp.logMsg('warn', "Rollback deletion of job " + rollbackState.policyName + " failed because of error=" + JSON.stringify(jrObj.rO));  
        rollbackResult.jobId = rollbackState.jobId;
      }
      else
        rollbackState.jobRollbackSuccessful = true;

    // FTI job may or may not have been deleted. Now nuke its policy.

      ngp.sendRequestToEcx('DELETE', ngp.ecxExternal, '/api/endeavour/policy/' + rollbackState.policyId, ecxSessionId, null, function(prObj){    // delete policy
        rollbackState.policyRollbackAttempted = true;				
        
        if ( prObj.e || ( prObj.response.statusCode != 204 && prObj.response.statusCode != 404 && prObj.response.statusCode != 500 ) ) 
        {
          ngp.logMsg('warn', "Rollback deletion of policy " + rollbackState.policyName + " failed because of error=" + JSON.stringify(jrObj.rO));  
          rollbackResult.policyId = rollbackState.policyId;
        }
        else
        {
          rollbackState.policyRollbackSuccessful = true;

          var result = {
            statusCode: 500,  
            response: {
              failedRollbacks: rollbackResult,
            }, 
          };

          //!! REVISIT: if rollback succeeds you should probably return the ECX error what caused us to drive rollback in 1st place
          ngp.logMsg('warn', "FINAL RBS=" + JSON.stringify(rollbackState));  
          ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, result, false); 	
        }
      });
    });		
}

