if (!('contains' in String.prototype)) {
	  String.prototype.contains = function(str, startIndex) {
	    return ''.indexOf.call(this, str, startIndex) !== -1;
	  };
	}

if (typeof String.prototype.startsWith != 'function') {
    String.prototype.startsWith = function(prefix) {
        return this.slice(0, prefix.length) == prefix;
    };
}
		 
if (typeof String.prototype.endsWith != 'function') {
    String.prototype.endsWith = function(suffix) {
        return this.slice(-suffix.length) == suffix;
    };
}

    
var ngp = require('./ecxngp');   

var functionSteps = [                                                                                  
    {                                                                                              
        continue: 'next',                                                                              
        name: 'get_appServer',                                                                    
        verb: 'GET',
        path: '/api/appserver/',                                                            
        ip: null, 
        port: null, 
        type: null,
        cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
    },     
    {                                                                                              
      continue: 'next',                                                                              
      name: 'del_appServer',                                                                    
      verb: 'DELETE',
      path: '/api/appserver/',                                                            
      ip: null, 
      port: null, 
      type: null,
      cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
    },     
    {                                                                                              
      continue: 'next',                                                                            
      name: 'del_key',   
      verb: 'DELETE',
      path: '/api/identity/key/',                                                            
      ip: null, 
      port: null, 
      type: null,
      cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
    },
];

const appServerDeleteStepFuncs = {
    get_appServer: function(step, funcStream, ecxSessionId, oFromNg, cb){                                   
      getAppServer(step, funcStream, ecxSessionId, oFromNg, cb);    
    }, 
    del_appServer: function(step, funcStream, ecxSessionId, oFromNg, cb){                                   
        deleteAppServer(step, funcStream, ecxSessionId, oFromNg, cb);    
    }, 
    del_key: function(step, funcStream, ecxSessionId, oFromNg, cb){                                   
      deleteKey(step, funcStream, ecxSessionId, oFromNg, cb);                
    }
};


exports.init = init;


function init(req, res, subType)                                                                       
{
    var ngState = ngp.getSessionState(req, res);
    
    if ( ngState.stale )                                                                                 
      return;                                                                                            

    var funcSteps = null;
    var genericFuncName = null;

    if ( req.method == 'DELETE' )
    {
        genericFuncName = 'delete_appServerKey';
        funcSteps = JSON.parse(JSON.stringify(functionSteps));
    } 

    var ecxSid = ngState.ecxSid;
    res.ecxSid = ecxSid;

    var stepsStream = JSON.parse(JSON.stringify(funcSteps));                                    

    stepsStream.forEach(function(step){
      step.ip = ngp.localIp;
      step.port = ngp.ecxExternal;

      step.func = appServerDeleteStepFuncs[step.name];     
      
      step.cb = function(ecxSid, data){ 
        ngp.logMsg('info', "Finished step '" + ngp.getCurrentFunc(step.reqId, ecxSid) + '(' + this.name + ")' in session ID '" + ecxSid + "'");

        if ( data )
        {
          ngp.logMsg('debug', "Data returned by step '" + ngState.currentFunc + '(' + this.name + ")' in session ID '" + ecxSid + "'=" + JSON.stringify(data));
        }
      };
    });

    ngp.createRequestStore(stepsStream, ecxSid, req.reqId, genericFuncName);                            // add reqId to steps, create store for the req & set currentFunc to genericFuncName
    ngp.setRequestState(req.reqId, ecxSid, 'crudelUrl', req.url);                                       // used by delete_userrpool steps
    
    ngState.currentFunc = genericFuncName;                                                               
    return stepsStream;
}

function getAppServer(step, funcStream, ecxSessionId, oFromNg, cb)                                       
{
    var crudelUrl = ngp.getRequestState(step.reqId, ecxSessionId, 'crudelUrl');  
    var appServerIdURL = crudelUrl.split('/');
    var appServerId = appServerIdURL[appServerIdURL.length-1];

    ngp.sendRequestToEcx(step.verb, ngp.ecxExternal, step.path + appServerId, ecxSessionId, null, function(appServerInfo){   
        if ( appServerInfo.e || ( appServerInfo.response.statusCode != 200 ) ) {
            ngp.logMsg('warn', 'Failed to get the app server Info ' + appServerId + '  because of error=' + JSON.stringify(appServerInfo.rO)); 
            var failResult = ngp.getGenericEcxErrorResponse(appServerInfo, ecxSessionId);          
            ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);
        } else {
            ngp.setRequestState(step.reqId, ecxSessionId, 'appServerDeletionId', JSON.stringify(appServerInfo.rO['id']));
            ngp.setRequestState(step.reqId, ecxSessionId, 'appServerType', JSON.stringify(appServerInfo.rO['applicationType']));
            ngp.setRequestState(step.reqId, ecxSessionId, 'appServerName', JSON.stringify(appServerInfo.rO['name']));
            
            step.cb(ecxSessionId, null);                                                  
            ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb);
        }
    });    
}

function deleteAppServer(step, funcStream, ecxSessionId, oFromNg, cb)                                       
{
    var appServerId = ngp.getRequestState(step.reqId, ecxSessionId, 'appServerDeletionId');  
    var appServerType = ngp.getRequestState(step.reqId, ecxSessionId, 'appServerType'); 
    
    ngp.sendRequestToEcx(step.verb, ngp.ecxExternal, step.path + appServerId.replace(/['"]+/g, ''), ecxSessionId, null, function(delAppServer0){   
        if ( delAppServer0.e || (delAppServer0.response.statusCode != 204 )) {
            ngp.logMsg('warn', 'Deletion of appserver ID=' + appServerId + ' failed because of error=' + JSON.stringify(delAppServer0.rO)); 
            var failResult = ngp.getGenericEcxErrorResponse(delAppServer0, ecxSessionId);          
            
            ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);      
        } else {
            if ( appServerType.replace(/['"]+/g, '') != 'office365' ) {
                ngp.logMsg('warn', 'Skipping deletion of key because the application type is not office365' ); 
                
                if ( funcStream.length > 0 )                                                            
                funcStream[0].continue = 'skip';                                                        
            
                var successResult = {
                    statusCode: delAppServer0.response.statusCode,
                    response: null,    
                };
            
          	    ngp.setStepSuccessResult(step, funcStream, ecxSessionId, oFromNg, cb, successResult);		
            } else {
                step.cb(ecxSessionId, null);                                                  
                ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb);                          
            }
        }
    });    
}

function deleteKey(step, funcStream, ecxSessionId, oFromNg, cb) {
    
    var appServerName = ngp.getRequestState(step.reqId, ecxSessionId, 'appServerName');                                
    
    ngp.sendRequestToEcx('GET', ngp.ecxExternal, '/api/identity/key', ecxSessionId, null, function(getKeyObj) {   
        if ( getKeyObj.e || ( getKeyObj.response.statusCode != 200 ) ) {
            ngp.logMsg('warn', "Getting keys failed because of error=" + JSON.stringify(getKeyObj.rO));  
            var failResult = ngp.getGenericEcxErrorResponse(getKeyObj, ecxSessionId);          
            
            ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);      
        } else {
            var keys = getKeyObj.rO['keys'];
     
            for(var i=0; i< keys.length; i++) {
                if(keys[i].name == 'exch_key_' + appServerName.replace(/['"]+/g, '')) {
                    var deletionKeyId = keys[i].id;
                    break;
                }
            }
 
            ngp.sendRequestToEcx(step.verb, ngp.ecxExternal, step.path + deletionKeyId, ecxSessionId, null, function(delKey0) { 
                if ( delKey0.e || ( delKey0.response.statusCode != 204 ) ) {
                    ngp.logMsg('warn', 'Deletion of Key ID=' + deletionKeyId + ' failed because of error=' + JSON.stringify(delKey0.rO)); 
                    var failResult = ngp.getGenericEcxErrorResponse(delKey0, ecxSessionId);          
                    
                    ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);
                }
                else {
                    var successResult = {
                        statusCode: delKey0.response.statusCode,
                        response: null,    
                    };
                    
                    ngp.setStepSuccessResult(step, funcStream, ecxSessionId, oFromNg, cb, successResult);	
                }                                                                      
            });           
        }
    });    
}