if (!('contains' in String.prototype)) {
	  String.prototype.contains = function(str, startIndex) {
	    return ''.indexOf.call(this, str, startIndex) !== -1;
	  };
	}

if (typeof String.prototype.startsWith != 'function') {
    String.prototype.startsWith = function(prefix) {
        return this.slice(0, prefix.length) == prefix;
    };
}
		 
if (typeof String.prototype.endsWith != 'function') {
    String.prototype.endsWith = function(suffix) {
        return this.slice(-suffix.length) == suffix;
    };
}

    
var ngp = require('./ecxngp');   

var functionSteps = [                                                                                  
    {                                                                                              
      continue: 'next',                                                                              
      name: 'del_scriptServer',                                                                    
      verb: 'DELETE',
      path: '/api/scriptserver/',                                                            
      ip: null, 
      port: null, 
      type: null,
      cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
    }
];

const scriptServerDeleteStepFuncs = {
    del_scriptServer: function(step, funcStream, ecxSessionId, oFromNg, cb){                                   
        deleteScriptServer(step, funcStream, ecxSessionId, oFromNg, cb);    
    }
};


exports.init = init;


function init(req, res, subType)                                                                       
{
    var ngState = ngp.getSessionState(req, res);
    
    if ( ngState.stale )                                                                                 
      return;                                                                                            

    var funcSteps = null;
    var genericFuncName = null;

    var ecxSid = ngState.ecxSid;
    res.ecxSid = ecxSid;

    if ( req.method == 'DELETE' )
    {
        genericFuncName = 'delete_scriptServer';
        funcSteps = JSON.parse(JSON.stringify(functionSteps));
    } 

    var stepsStream = JSON.parse(JSON.stringify(funcSteps));                                    

    stepsStream.forEach(function(step){
      step.ip = ngp.localIp;
      step.port = ngp.ecxExternal;

      step.func = scriptServerDeleteStepFuncs[step.name];     
      
      step.cb = function(ecxSid, data){ 
        ngp.logMsg('info', "Finished step '" + ngp.getCurrentFunc(step.reqId, ecxSid) + '(' + this.name + ")' in session ID '" + ecxSid + "'");

        if ( data )
        {
          ngp.logMsg('debug', "Data returned by step '" + ngState.currentFunc + '(' + this.name + ")' in session ID '" + ecxSid + "'=" + JSON.stringify(data));
        }
      };
    });

    ngp.createRequestStore(stepsStream, ecxSid, req.reqId, genericFuncName);                            // add reqId to steps, create store for the req & set currentFunc to genericFuncName
    ngp.setRequestState(req.reqId, ecxSid, 'crudelUrl', req.url);                                       // used by delete_userrpool steps
    
    ngState.currentFunc = genericFuncName;                                                               
    return stepsStream;
}

function deleteScriptServer(step, funcStream, ecxSessionId, oFromNg, cb)                                       
{
    var crudelUrl = ngp.getRequestState(step.reqId, ecxSessionId, 'crudelUrl');  
    var scriptServerIdURL = crudelUrl.split('/');
    var scriptServerId = scriptServerIdURL[scriptServerIdURL.length-1];  
    
    ngp.sendRequestToEcx(step.verb, ngp.ecxExternal, step.path + scriptServerId.replace(/['"]+/g, ''), ecxSessionId, null, function(delScriptServer0){   
        if ( delScriptServer0.e || (delScriptServer0.response.statusCode != 204 )) {
            ngp.logMsg('warn', 'Deletion of script server ID=' + scriptServerId + ' failed because of error=' + JSON.stringify(delScriptServer0.rO)); 
            var failResult = ngp.getGenericEcxErrorResponse(delScriptServer0, ecxSessionId);          
            
            ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);      
        } else {
            var successResult = {
                statusCode: delScriptServer0.response.statusCode,
                response: null,    
            };
            
            ngp.setSerialStreamResult(ecxSessionId, successResult); 	
            step.cb(ecxSessionId, null);                                                  
            ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb);                          
        }
    });    
}