if (!('contains' in String.prototype)) {
  String.prototype.contains = function(str, startIndex) {
    return ''.indexOf.call(this, str, startIndex) !== -1;
  };
}

if (typeof String.prototype.startsWith != 'function') {
  String.prototype.startsWith = function(prefix) {
      return this.slice(0, prefix.length) == prefix;
  };
}
   
if (typeof String.prototype.endsWith != 'function') {
  String.prototype.endsWith = function(suffix) {
      return this.slice(-suffix.length) == suffix;
  };
}


var ngp = require('./ecxngp');   


var funcSteps = [                                                                                    // absolutely no calls to SPP; these strux and handshake protocol merely pro forma
{                                                                                                
  continue: 'next',                                                                            
  name: 'get_metrics',
  verb: null,
  path: null,
  ip: null,                                                                                                                                        
  port: null, 
  type: null,
  cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
}, 	
];


const getMetricsFuncs = {
  get_metrics: function(step, funcStream, ecxSessionId, oFromNg, cb){          
    getMetrics(step, funcStream, ecxSessionId, oFromNg, cb);      	                     
  },	
};


exports.init = init;


function init(req, res, providerType)                                    
{
    var ngState = ngp.getSessionState(req, res);

    if ( ngState.stale )                                                                            
      return;  

    var ecxSid = ngState.ecxSid;
    res.ecxSid = ecxSid;

    var stepsStream = JSON.parse(JSON.stringify(funcSteps));                                    

    stepsStream.forEach(function(step){
      step.ip = ngp.localIp;
      step.port = ngp.ecxExternal;

      step.func = getMetricsFuncs[step.name];                                                  

      step.cb = function(ecxSid, data){ 
			//@#@ ngp.logMsg('info', "Finished step '" + ngState.currentFunc + '(' + this.name + ")' in session ID '" + ecxSid + "'");
			ngp.logMsg('info', "Finished step '" + ngp.getCurrentFunc(step.reqId, ecxSid) + '(' + this.name + ")' in session ID '" + ecxSid + "'");

        if ( data )
        {
          ngp.logMsg('debug', "Data returned by step '" + ngState.currentFunc + '(' + this.name + ")' in session ID '" + ecxSid + "'=" + JSON.stringify(data));
        }
      };
    });

    ngp.createRequestStore(stepsStream, ecxSid, req.reqId, 'get_metrics');  

    ngState.currentFunc = 'get_metrics';                                 
    return stepsStream;
}

/*

> si.fsSize(function(d) {console.log(d)} )
Promise { <pending> }
> [ { fs: '/dev/mapper/cl-root',
    type: 'xfs',
    size: 44194762752,
    used: 4266102784,
    use: 9.65,
    mount: '/' },
  { fs: '/dev/sda1',
    type: 'xfs',
    size: 1063256064,
    used: 174166016,
    use: 16.38,
    mount: '/boot' },
  { fs: '/dev/mapper/ecxconfig-ecxconfiglv',
    type: 'ext4',
    size: 19946684416,
    used: 569446400,
    use: 2.85,
    mount: '/data' },
  { fs: '/dev/mapper/ecxcatalog-ecxcataloglv',
    type: 'ext4',
    size: 51653410816,
    used: 349626368,
    use: 0.68,
    mount: '/data2' } ]

{ avgload: 0.01,
  currentload: 1.626097810793696,
  currentload_user: 1.3879184615924518,
  currentload_system: 0.23817065404900373,
  currentload_nice: 0.000008695152240464225,
  currentload_irq: 0,
  cpus:
   [ { load: 1.695626019518337,
       load_user: 1.4198611313380949,
       load_system: 0.27576488818024214,
       load_nice: 0,
       load_irq: 0 },
     { load: 1.6678888639965714,
       load_user: 1.4158193615538,
       load_system: 0.2520695024427712,
       load_nice: 0,
       load_irq: 0 },
     { load: 1.5743637678396865,
       load_user: 1.350108120155257,
       load_system: 0.2242208890670384,
       load_nice: 0.00003475861739118224,
       load_irq: 0 },
     { load: 1.566688428641539,
       load_user: 1.3659722643595498,
       load_system: 0.20071616428198932,
       load_nice: 0,
       load_irq: 0 } ] }



> { total: 16595820544,
  free: 3444150272,
  used: 13151092736,
  active: 10958938112,
  available: 5177061376,
  buffcache: 2192732160,
  swaptotal: 8388603904,
  swapused: 0,
  swapfree: 8388603904 }
*/


function getMetrics(step, funcStream, ecxSessionId, oFromNg, cb)
{
    var result = {
      statusCode: 500,
      response: {
        cpuUtil: 0,
        memory: {
          size: 0,
          util: 0,
        },
        data: {
          size: 0,
          util: 0,
        },
        data2: {
          size: 0,
          util: 0,
        },
        data3: {
          size: 0,
          util: 0,
        },
      },
      
      countdownLatch: 3,                                                                                  // i.e., 3 calls to si
    };

    ngp.si.currentLoad().then(data => {    
            extractMetrics('cpu', data, result, step, funcStream, ecxSessionId, oFromNg, cb);
        }).catch(error =>{
            ngp.logMsg('error', 'CPU utilization metrics error=' + error);
            ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, result, false);	
        });

        ngp.si.mem().then(data => {    
              extractMetrics('mem', data, result, step, funcStream, ecxSessionId, oFromNg, cb);
          }).catch(error =>{
              ngp.logMsg('error', 'Memory utilization metrics error=' + error);
              ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, result, false);	
          });
        ngp.si.fsSize().then(data => {    
            extractMetrics('fsz', data, result, step, funcStream, ecxSessionId, oFromNg, cb);
        }).catch(error =>{
            ngp.logMsg('error', 'File system utilization metrics error=' + error);
            ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, result, false);	
        });          
}


function extractMetrics(mType, mData, result, step, funcStream, ecxSessionId, oFromNg, cb)
{
    if ( mType == 'cpu' )
    {
        result.response.cpuUtil = parseFloat(mData.currentLoad).toFixed(2);
        result.countdownLatch--;
    }

    if ( mType == 'mem' )
    {
      var memO = result.response.memory;

      var trueUsed = mData.used - mData.buffcache;                                                        // used includes buffers/cache, so remove that amount to get true used
      memO.util = parseFloat(trueUsed/(mData.total)).toFixed(2);
      memO.size = mData.total;
      
      result.countdownLatch--;
    }

    if ( mType == 'fsz' )
    {
        var fszHash = ngp.arrayToHash(mData, 'mount');
        
        var fsmO = result.response.data;

        fsmO.util = fszHash['/data'].use;
        fsmO.size = fszHash['/data'].size;

        fsmO = result.response.data2;
        fsmO.util = fszHash['/data2'].use;
        fsmO.size = fszHash['/data2'].size

        if ( fszHash['/data3'])
        {
          fsmO = result.response.data3;
          fsmO.util = fszHash['/data3'].use;
          fsmO.size = fszHash['/data3'].size;
        }

        result.countdownLatch--;
    }

    if ( result.countdownLatch < 1 )
    {
        delete result.countdownLatch;
        result.statusCode = 200;
        ngp.setStepSuccessResult(step, funcStream, ecxSessionId, oFromNg, cb, result);
    }
}
