if (!('contains' in String.prototype)) {
    String.prototype.contains = function(str, startIndex) {
        return ''.indexOf.call(this, str, startIndex) !== -1;
    };
}

if (typeof String.prototype.startsWith != 'function') {
    String.prototype.startsWith = function(prefix) {
        return this.slice(0, prefix.length) == prefix;
    };
}

if (typeof String.prototype.endsWith != 'function') {
    String.prototype.endsWith = function(suffix) {
        return this.slice(-suffix.length) == suffix;
    };
}

var ngp = require('../ecxngp');
var reportProvider = require('./report-provider');

var reportRouter = ngp.express.Router();

const reportlicitQueries = {
    action: ['schedule']
};

const getReportlicitQueries = {
    userId: [/.*/,],
    filter: [/.*/,],
};

const deleteReportlicitQueries = {
    userId: [/.*/,],    
};

//middleware which is specific to the report router
reportRouter.use(function timeLog(req, res, next) {
    ngp.logMsg('info','Handling /ecxngp/report request Time(%s) METHOD(%s)', Date.now().toString(), req.method.toString());
    next();
});

/*
  =========================
  ADD REPORT ENDPOINTS HERE
  =========================
*/

/*
SAMPLE POST BODY FROM UI
========================
{
    "userId: "1000",
    "resources": [
        {
            "name": "Report Name 1",
             "href": "http://ecxbed3/api/endeavour/report/10",
             "id": "10"
        },
        {
            "name": "Report Name 2",
             "href": "http://ecxbed3/api/endeavour/report/11",
             "id": "11"
        }
    ],
    "trigger": { 
      "type":"DAILY",
      "frequency":1,
      "domList": [],
      "dowList": []
    },
    "notification": [
        "email1",
        "email2"
    ]
}
*/
/**
 * @swagger
 * /ngp/report:
 *   post:
 *     tags: ["Reports"]
 */
reportRouter.post('/', function(req, res) {
    var reportQueryValidationResult = ngp.validateQs(req, reportlicitQueries);

    if ( reportQueryValidationResult.rc == 2 )
    {
        ngp.logMsg('warn', 'Detected invalid value among keys=' + JSON.stringify(reportQueryValidationResult.keyVals));
        ngp.returnBadRequestError(res);
        return;
    }

    if ( reportQueryValidationResult.rc == 0 ){
        ngp.logMsg('warn', 'Detected no query parameters');
        ngp.returnBadRequestError(res);
        return;
    }
    var stepsStream = null;

    if (req.query['action']){
        if (req.query['action'] == 'schedule'){            
            stepsStream = reportProvider.init(req, res, 'createReportSchedule');
            ngp.startFunc(req, res, stepsStream);       
        }else{
            ngp.returnBadRequestError(res);
            return;            
        }
    }
});

/**
 * @swagger
 * /ngp/report:
 *   get:
 *     tags: ["Reports"]
 */
reportRouter.get('/', function(req, res) {    
    var getReportQueryValidationResult = ngp.validateQs(req, getReportlicitQueries);

    if ( getReportQueryValidationResult.rc == 2 )
    {
        ngp.logMsg('warn', 'Detected invalid value among keys=' + JSON.stringify(getReportQueryValidationResult.keyVals));
        ngp.returnBadRequestError(res);
        return;
    }

    if ( getReportQueryValidationResult.rc == 0 ){
        ngp.logMsg('warn', 'Detected no query parameters');
        ngp.returnBadRequestError(res);
        return;
    }

    if (req.query['userId']){
        var stepsStream = null;
        var reportCategoryValue = null;
        if (req.query['filter']){
            var filterValue = JSON.parse(req.query['filter']);
            if (filterValue[0]['value']){
                reportCategoryValue = filterValue[0]['value'];
                if (reportCategoryValue.contains(' ')){
                    ngp.logMsg('info','Report Category Value('+ reportCategoryValue + ') contains a space');
                    reportCategoryValue = ngp.escape(reportCategoryValue);
                    ngp.logMsg('info','New Report Category Value(' + reportCategoryValue + ')');
                    filterValue[0]['value'] = reportCategoryValue;
                    ngp.logMsg('info','New Filter Value(' + JSON.stringify(filterValue) + ')');
                    req.query['filter'] = filterValue;
                }
            }
        }
        stepsStream = reportProvider.init(req, res, 'getReports');        
        ngp.startFunc(req, res, stepsStream);
    }    
});

/**
 * @swagger
 * /ngp/report:
 *   delete:
 *     tags: ["Reports"]
 */
reportRouter.delete('/:id', function(req, res) {    
    var deleteReportValidationResult = ngp.validateQs(req, deleteReportlicitQueries);

    if ( deleteReportValidationResult.rc == 2 )
    {
        ngp.logMsg('warn', 'Detected invalid value among keys=' + JSON.stringify(deleteReportValidationResult.keyVals));
        ngp.returnBadRequestError(res);
        return;
    }

    if ( deleteReportValidationResult.rc == 0 ){
        ngp.logMsg('warn', 'Detected no query parameters');
        ngp.returnBadRequestError(res);
        return;
    }

    if (req.query['userId']){
        var stepsStream = null;
        stepsStream = reportProvider.init(req, res, 'deleteReportSchedule');        
        ngp.startFunc(req, res, stepsStream);
    }
});

module.exports = reportRouter;