if (!('contains' in String.prototype)) {
    String.prototype.contains = function(str, startIndex) {
        return ''.indexOf.call(this, str, startIndex) !== -1;
    };
}

if (typeof String.prototype.startsWith != 'function') {
    String.prototype.startsWith = function(prefix) {
        return this.slice(0, prefix.length) == prefix;
    };
}
     
if (typeof String.prototype.endsWith != 'function') {
    String.prototype.endsWith = function(suffix) {
        return this.slice(-suffix.length) == suffix;
    };
}


var ngp = require('./ecxngp');                                                                


var funcSteps = [
    {                                                                                              
        continue: 'next',                                                                         
        name: 'create_missing_target',
        verb: 'POST',
        path: '/api/endeavour/policy',                                                                       
        ip: null,                                                                                                                                   
        port: null, 
        type: null,
        func: null,                                                                                 
        cb: function(res, data){ logMsg('info', 'Step=' + this.name + ' finished');}
    },                                                                                     
    {                                                                                              
        continue: 'next',                                                                         
        name: 'get_secperms_target',
        verb: 'GET',
        path: null,                                                                                          
        ip: null,                                                                                                                                   
        port: null, 
        type: null,
        func: null,                                                                                 
        cb: function(res, data){ logMsg('info', 'Step=' + this.name + ' finished');}
    },     
    {                                                                                              
        continue: 'next',                                                                          
        name: 'apply_secperms',
        verb: 'POST',
        path: '/api/security/permissions?action=qqVVqq',                                           
        ip: null,                                                                                                                                   
        port: null, 
        type: null,
        func: null,                                                                                
        cb: function(res, data){ logMsg('info', 'Step=' + this.name + ' finished');}
    }, 
];


const doSecPermsStepFuncs = {
    create_missing_target: function(step, funcStream, ecxSessionId, oFromNg, cb){                            
        createMissingTargets(step, funcStream, ecxSessionId, oFromNg, cb);   
    },     
    get_secperms_target: function(step, funcStream, ecxSessionId, oFromNg, cb){                            
        getSecPermsTarget(step, funcStream, ecxSessionId, oFromNg, cb);   
    },      
    apply_secperms: function(step, funcStream, ecxSessionId, oFromNg, cb){                            
        applySecPerms(step, funcStream, ecxSessionId, oFromNg, cb);   
    },
};


exports.init = init;


function init(req, res, keyVals)                                                    
{
    var ngState = ngp.getSessionState(req, res);

    var ecxSid = ngState.ecxSid;
    res.ecxSid = ecxSid;

    var stepsStream = JSON.parse(JSON.stringify(funcSteps));                                    

    stepsStream.forEach(function(step){
        step.ip = ngp.localIp;
        step.port = ngp.ecxExternal;

        if ( step.name == 'create_missing_target' && keyVals.action != 'add' )                             // don't do this for update case
            step.continue = 'skip';

        if ( step.name == 'apply_secperms' )
        {
            step.path = step.path.replace('qqVVqq', keyVals.action);
        }
        
        step.func = doSecPermsStepFuncs[step.name];        
        
        step.cb = function(ecxSid, data){ 
			ngp.logMsg('info', "Finished step '" + ngp.getCurrentFunc(step.reqId, ecxSid) + '(' + this.name + ")' in session ID '" + ecxSid + "'");

            if ( data )
            {
                ngp.logMsg('debug', "Data returned by step '" + ngState.currentFunc + '(' + this.name + ")' in session ID '" + ecxSid + "'=" + JSON.stringify(data));
            }
        };
    });

    ngp.createRequestStore(stepsStream, ecxSid, req.reqId, 'apply_secperms');  

    ngState.currentFunc = 'apply_secperms';                                                        
    return stepsStream;
}


var getSecPermsTarget = function(step, funcStream, ecxSessionId, oFromNg, cb)
{
  var scrubbedNgInput = ngp.scrubPostBody(oFromNg, true);                      
  ngp.logMsg('warn', "Input from client for security permissions in session ID='" + ecxSessionId + "'=" + JSON.stringify(scrubbedNgInput));
    
  var ngState = ngp.ecxSidHash[ecxSessionId];

  if ( ! oFromNg.hasOwnProperty('permissionData') )                                                        // FTI the update case which lacks permissionData array; not for long though
  {
      oFromNg = normalizePermissionsInput(oFromNg);                                                        // house lone permission inside permissionData array and then use common code
  }

  var permsData = JSON.parse(JSON.stringify(oFromNg.permissionData));

  var permState = [];                                                                                      // array of objects that will be POSTed to SPP
  var reportTriggerPerms = [];

  getSecurityTargetEntities(oFromNg, permsData, ngState, permState, reportTriggerPerms, function(rc, errEntity){
      if ( rc == 'ok' )
      {
          if ( reportTriggerPerms.length > 0 )                                                             // NB: if this is true, then permState must contain report policies
          {
              getReportTriggers(reportTriggerPerms, permState, ngState, function(rc, result){
                  if ( rc == 'ok' )
                  {
                      ngp.setRequestState(step.reqId, ecxSessionId, 'addPermissions', permState);
          
                      step.cb(ecxSessionId, null);                                                  
                      ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb); 	                           
                  }
                  else
                  {
                      var failResult = ngp.getGenericEcxErrorResponse(errEntity, ecxSessionId, step); 
                      ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false); 
                  }
              });
          }
          else                                                                                             // EHI no reports among the permissions we must assert
          {
              ngp.setRequestState(step.reqId, ecxSessionId, 'addPermissions', permState);              
  
              step.cb(ecxSessionId, null);                                                  
              ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb); 
          }
      }
      else
      {
          var failResult = ngp.getGenericEcxErrorResponse(errEntity, ecxSessionId, step); 
          ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);                
      }   
  });
};


function getReportTriggers(reportTriggerPerms, permState, ngState, cb)
{
  if ( reportTriggerPerms.length < 1 )
  {
      cb('ok', null);
  }
  else
  {
      var reportTriggerPerm = reportTriggerPerms.shift();

// !! REWRITE??: get all triggers, convert triggers array to hash using its name prop; then find one trigger in hash whose name matches reportTriggerPerm.reportName

      var path = '/api/endeavour/trigger';                                                                 // NO filter support for triggers

      ngp.sendRequestToEcx('GET', ngp.ecxExternal, path, ngState.ecxSid, null, function(trigO){ 
          if ( trigO.e || trigO.response.statusCode != 200 )  
          {
              var failResult = ngp.getGenericEcxErrorResponse(trigO, ecxSessionId, step); 
              cb('ng', failResult);
          }
          else                                                                                             // EHI 0 or more trigger in the triggers array (FILTER DON'T WORK)      
          {
              var triggers = trigO.rO.triggers;                                                            // exactly 1 trigger in the array

              triggers.forEach(function(trigger){
                  if ( trigger.name == reportTriggerPerm.reportName )
                  {
                      delete reportTriggerPerm.reportName;
                      reportTriggerPerm.rbacPath = trigger.rbacPath;
                      permState.push(reportTriggerPerm);
                  }
              });

              getReportTriggers(reportTriggerPerms, permState, ngState, cb);                               // RECURSE
          }
      }); 
  }
}


function getSecurityTargetEntities(oFromNg, perms, ngState, permState, reportTriggerPerms, cb)
{
  if ( perms.length < 1 )
  {
      cb('ok');
      return;
  }

  var perm = perms.shift();

// Vide ECC-11530. In ALL cases we must apply permissions to the 'base' case. Then, for slapolicy & report get related entities (policies and report triggers) and apply permissions to them.

    var permO = {
        rbacPath: perm.rbacPath,
        permissions: perm.permissions,
    };
  
    permState.push(permO); 

    if ( perm.resourceType != 'slapolicy' && perm.resourceType != 'report' )                 // any more work to be done?
    {
        getSecurityTargetEntities(oFromNg, perms, ngState, permState, reportTriggerPerms, cb);   // RECURSE
        return;
    }

  var isReport = false;

  if ( perm.resourceType == 'slapolicy' )   
  {
      //ngp.logMsg('warn', 'PERM=' + JSON.stringify(perm));
      var temp = '*' + perm.name + '*';                                                        // e.g., 'Gold' alone won't cut it, need to find, say, vmware_Gold

      var selector = {
          path: '/api/endeavour/policy?filter=',
          pathFilter: '[{"property":"name","value":"' + temp + '"}]',
          arrayName: 'policies',                                                               // look at objects in this array
          propName: 'name',                                                                    // match value of this object property
          filter: new RegExp('^\\S+?_' + perm.name),                                           // against this
      };
  }

  if ( perm.resourceType == 'report' )   
  {
      isReport = true;

      var temp = 'Report_' + perm.id + '_' + perm.userId;

      var selector = {
          path: '/api/endeavour/policy?filter=',
          pathFilter: '[{"property":"name","value":"' + temp + '"}]',
          arrayName: 'policies',                                                               // look at objects in this array
          propName: 'name',                                                                    // match value of this object property
          filter: new RegExp('^\\S+?_' + perm.name),                                           // against this
      };
  }

  var path = selector.path + require('querystring').escape(selector.pathFilter);   

  ngp.sendRequestToEcx('GET', ngp.ecxExternal, path, ngState.ecxSid, null, function(entity){ 
      if ( entity.e || entity.response.statusCode != 200 )  
      {
          cb('ng', entity);
          return;
      }
      else
      {
        var entities = entity.rO[selector.arrayName];

// NB: if I fail to match the indicated resourceType, then we must still form a permissions object that consists of the rbacPath (from the NG input, not from a matching entity) and
// the permissions array (always from NG input).

        if ( entities.length < 1 )
        {
              ngp.logMsg('warn', 'No entities match path=' + path); //!! ALSO, what is there to do with report triggers in the 'no entities' case?

              var permO = {
                  rbacPath: perm.rbacPath,
                  permissions: perm.permissions,
              };

              permState.push(permO);
        }
        else
        {

// FOREACH entity that matched we must send to SPP a post body of {rbacPath, permissions} where rbacPath is from the entity & permissions from the perm object
          
          entities.forEach(function(entity){
              var permO = {
                  rbacPath: entity.rbacPath,                                                   
                  permissions: perm.permissions,
              };    

              if ( selector.path.contains('filter=') )                                         // FTI SPP did filtering for us
              {
                  permState.push(permO);
              }
              else if ( entity[selector.propName].match(selector.filter) )
              {
                  permState.push(permO);
              }  
              
              if ( isReport )                                                                  // store a placeholder for the report's trigger which we'll get shortly
              {
                  var rptPermO = {
                      reportName: perm.name,
                      rbacPath: 'REPORT_RBACPATH',
                      permissions: perm.permissions,
                  };
                  
                  reportTriggerPerms.push(rptPermO); 
              }
          });
        }
      }

      getSecurityTargetEntities(oFromNg, perms, ngState, permState, reportTriggerPerms, cb);   // RECURSE
  });    
}


function createMissingTargets(step, funcStream, ecxSessionId, oFromNg, cb)
{
    var scrubbedNgInput = ngp.scrubPostBody(oFromNg, true);                      
    ngp.logMsg('warn', "Input from client for security permissions in session ID='" + ecxSessionId + "'=" + JSON.stringify(scrubbedNgInput));
      
    var ngState = ngp.ecxSidHash[ecxSessionId];
  
    if ( ! oFromNg.hasOwnProperty('permissionData') )                                                        // FTI the update case which lacks permissionData array; not for long though
    {
        oFromNg = normalizePermissionsInput(oFromNg);                                                        // house lone permission inside permissionData array and then use common code
    }
  
    var permsData = JSON.parse(JSON.stringify(oFromNg.permissionData));
  
    var missingTargets = [];                                                                                 // array of objects that will be POSTed to SPP
    var missingPolicyNames = {};
  
    getMissingSecurityTargetEntities(oFromNg, permsData, ngState, missingTargets, missingPolicyNames, function(rc, errEntity){
        ngp.logMsg('warn', "Must pre-create policies for=" + JSON.stringify(missingTargets));                // i.e., sprofs and reports w/o related endeavour policies
        ngp.logMsg('warn', "Pre-created policy names will be=" + JSON.stringify(missingPolicyNames));
        
        if ( rc == 'ok' )
        {
            if ( missingTargets.length > 0 )
            {
                var simFromNg = createSlapArray(missingTargets);                                             // analog to what is consumed in crhyppolicy, BUT ONLY FOR slapolicy, not reports
                
                createMissingPolicies(step, funcStream, ecxSessionId, simFromNg, missingPolicyNames, function(rc, failResult){    // actually create them
                    if ( rc == 'ng' )
                    {
                        ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false); 
                    }
                    else                                                                                      // EHI we created the missing PJ targets, continue with funk
                    {
                        step.cb(ecxSessionId, null);                                                  
                        ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb); 
                    }
                });    
            }
            else                                                                                              // EHI all slapolicies had related endeavour policies
            {
                step.cb(ecxSessionId, null);                                                  
                ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb); 
            }
        }
        else
        {
            var failResult = ngp.getGenericEcxErrorResponse(errEntity, ecxSessionId, step);                    // NB: errEntity came from SPP, so this is safe...
            ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);                
        }   
    });    
}


/*  A LIFE-LIKE SLAP
{
    "name": "Gold", // Name of SLA Policy
    "href": "https://172.20.39.2:8443/api/spec/storageprofile/8",
    "id": "8"
}
*/

function createSlapArray(missingTargets)
{
    var slapolicies = [];
    var missingTargetType = null;

    missingTargets.forEach(function(missingTarget){
        if ( ! missingTargetType )
            missingTargetType = missingTarget.type;

        slapolicies.push(
            {
                name: missingTarget.name,
                id: missingTarget.id,
                href: missingTarget.href
            }
        );
    });

    if ( missingTargetType == 'slapolicy' )
        return( {slapolicies: slapolicies});

    return( {reports: slapolicies});
}


// Find both SLA policies and reports that lack a related Endeavour policy and create the policy & job. This happens when user "manages permissions" before creating hypervisor protection 
// policy (sla case) or before scheduling a report (report case).

function getMissingSecurityTargetEntities(oFromNg, perms, ngState, missingTargets, missingPolicyNames, cb)
{
  if ( perms.length < 1 )
  {
      cb('ok');
      return;
  }

  var perm = perms.shift();

  if ( perm.resourceType == 'slapolicy' )   
  {
      var polName = '*' + perm.name + '*';                                                     // e.g., 'Gold' alone won't cut it, need to find, say, vmware_Gold

      var selector = {
          path: '/api/endeavour/policy?filter=',
          pathFilter: '[{"property":"name","value":"' + polName + '"}]',
          arrayName: 'policies',                                                               // look at objects in this array
          propName: 'name',                                                                    // match value of this object property
          filter: new RegExp('^\\S+?_' + perm.name),                                           // against this
      };

    var path = selector.path + require('querystring').escape(selector.pathFilter);   

    ngp.sendRequestToEcx('GET', ngp.ecxExternal, path, ngState.ecxSid, null, function(entity){ 
        if ( entity.e || entity.response.statusCode != 200 )  
        {
            cb('ng', entity);
            return;
        }
        else
        {
            var entities = entity.rO[selector.arrayName];

            if ( entities.length < 1 )                                                         // FTI case we're interested in, the slapolicy lacks endeavour policy
            {
                var missingTarget = {
                    name: perm.name,
                    id: perm.id,
                    href: perm.href,
                    type: 'slapolicy',
                };

                missingTargets.push(missingTarget);
                missingPolicyNames['vmware_' + missingTarget.name] = 1;                        // 2 PJs, one for vmware, one for its competitor
                missingPolicyNames['hyperv_' + missingTarget.name] = 1;
            }
        }

        getMissingSecurityTargetEntities(oFromNg, perms, ngState, missingTargets, missingPolicyNames, cb);   // RECURSE
    });
  }
  else if ( perm.resourceType == 'report' )   
  {
      var reportName = 'Report_' + perm.id + '_' + perm.userId;
      
      var selector = {
          path: '/api/endeavour/policy?filter=',
          pathFilter: '[{"property":"name","value":"' + reportName + '"}]',
          arrayName: 'policies',                                                               // look at objects in this array
          propName: 'name',                                                                    // match value of this object property
          filter: new RegExp('^\\S+?_' + perm.name),                                           // against this
      };

    var path = selector.path + require('querystring').escape(selector.pathFilter);   

    ngp.sendRequestToEcx('GET', ngp.ecxExternal, path, ngState.ecxSid, null, function(entity){ 
        if ( entity.e || entity.response.statusCode != 200 )  
        {
            cb('ng', entity);
            return;
        }
        else
        {
            var entities = entity.rO[selector.arrayName];

            if ( entities.length < 1 )                                                         // FTI case we're interested in, the report lacks endeavour policy
            {
                var missingTarget = {
                    name: reportName,
                    id: perm.id,
                    href: perm.href,
                    type: 'report',
                };

                missingTargets.push(missingTarget);
                missingPolicyNames[reportName] = 1;
            }
        }

        getMissingSecurityTargetEntities(oFromNg, perms, ngState, missingTargets, missingPolicyNames, cb);   // RECURSE
    });
  }  
  else
      getMissingSecurityTargetEntities(oFromNg, perms, ngState, missingTargets, missingPolicyNames, cb);     // RECURSE
}


/*
  The addPermissions state saved by previous step will contain objects of form
  {
      rbacPath: <FROM_SPP_ENTITY>,
      permissions: <FROM_NG_PERMISSIONS>,
  }

  Present them 1 by 1 to SPP !! 
*/

var applySecPerms = function(step, funcStream, ecxSessionId, oFromNg, cb)
{
  var ngState = ngp.ecxSidHash[ecxSessionId];

  var temp = ngp.getRequestState(step.reqId, ngState.ecxSid, 'addPermissions');

  var permPostBodies = JSON.parse(JSON.stringify(temp));                                       // !!?? WHY CLONING 
  var sppResults = [];                                                                         // result from each POST to SPP accumulate here

  postPermissionsToSpp(permPostBodies, step, ecxSessionId, sppResults, function(rc, result){
      if ( rc == 'ng' )
      {
          ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, result, false); 
      }
      else
      {
          var successResult = { 
              statusCode: 201,
              response: sppResults,
          };

          ngp.setStepSuccessResult(step, funcStream, ecxSessionId, oFromNg, cb, successResult);  
      }
  });
};


//!! accumulate per call to ecx result and return in array to caller, thence to its caller. You need to POST to action=add | action=update depending on what came from GUI
function postPermissionsToSpp(permBodies, step, ecxSessionId, sppResults, cb)
{
  if ( permBodies.length < 1 )
  {
      cb('ok');
  }
  else
  {
      var permBody = permBodies.shift();

      ngp.sendRequestToEcx(step.verb, ngp.ecxExternal, step.path, ecxSessionId, permBody, function(entity){ 
          if ( entity.e || ( entity.response.statusCode != 201 && entity.response.statusCode != 200 ) )             
          {
              var failResult = ngp.getGenericEcxErrorResponse(entity, ecxSessionId, step);
              cb('ng', failResult);
          }
          else
          {
              sppResults.push(entity.rO);
              postPermissionsToSpp(permBodies, step, ecxSessionId, sppResults, cb);            // RECURSE
          }
      });         
  }
}


function normalizePermissionsInput(oFromNg)
{
    ngp.logMsg('debug', 'NG INPUT BEFORE=' + JSON.stringify(oFromNg));
    var permissionData = [];

    permissionData.push(oFromNg);

    oFromNg = {permissionData: permissionData};
    ngp.logMsg('debug', 'NG INPUT AFTER=' + JSON.stringify(oFromNg));
    return oFromNg;
}


// permissionData element will soon have property .href (you also need its .id and .name which is the sprof name)

function createMissingPolicies(step, funcStream, ecxSessionId, oFromNg, createPolNamesHash, driverCb)                           
{
    ngp.logMsg('warn', 'FAUXFROMNG=' + JSON.stringify(oFromNg));
    ngp.logMsg('debug', 'Policy names=' + JSON.stringify(createPolNamesHash));

    if  ( oFromNg.reports )
    {
        var rptHash = ngp.arrayToHash(oFromNg.reports, 'name');                                     // Input from client
        var pjEntries = [];                                                                                                                                         
        
        var rollback = {
            policies: [],
            jobs: [],
        };
    
        var policyNow = (new Date).getTime();
    
        for ( policyName in createPolNamesHash )
        {
            var pjEntry = {};
            var rpt = rptHash[policyName];

            var rptpolTplClone = JSON.parse(JSON.stringify(ngp.rptpolTpl));
            rptpolTplClone.name = policyName;
            rptpolTplClone.spec.source[0].id = rpt.id;
            rptpolTplClone.spec.source[0].name = rpt.name;                                                  // same as policy name
            rptpolTplClone.spec.source[0].href = rpt.href;                                                  

            var pjEntry = {
                policyName: policyName,
                policyCreated: false,
                policyId: null,
                jobCreated: false,
                jobId: null,
                policyPostBody: rptpolTplClone,                                                          // created up front, outbound to ECX via POST
                policyRollbackAttempted: false,
                policyRollbackSuccessful: false,
                jobRollbackAttempted: false,
                jobRollbackSuccessful: false,	                                                         // NB: set on 204 or 404 from DELETE
            };
            
            pjEntries.push(pjEntry);                                                                     // unlike recovery case, here we can have multiple policy/job pairs to create
        }
        
        ngp.logMsg('warn', 'REPORT POLICY POST BODY=' + JSON.stringify(pjEntry.policyPostBody));
    }
    else                                                                                                 // EHI slapolicies
    {
        var slapHash = ngp.arrayToHash(oFromNg.slapolicies, 'name');                                     // Input from client
        var pjEntries = [];                                                                                                                                         
        
        var rollback = {
            policies: [],
            jobs: [],
        };

        var policyNow = (new Date).getTime();

        for ( policyName in createPolNamesHash )
        {
            var pjEntry = {};
            var temp = policyName.split('_');                                                           // type, e.g., vmware, in element 0
            var entityType = temp.shift();
            var slapMetal = temp.join('_');

            var slap = slapHash[slapMetal];

            var hyppolTplClone = JSON.parse(JSON.stringify(ngp.hyppolTpl));
            hyppolTplClone.name = policyName;
            hyppolTplClone.subType = entityType;
            hyppolTplClone.spec.storageworkflow[0].id = slap.id;
            hyppolTplClone.spec.storageworkflow[0].name = slap.name;
            hyppolTplClone.spec.storageworkflow[0].href = slap.href;

            delete hyppolTplClone.spec.storageworkflow[0].trigger;
            
            var pjEntry = {
                policyName: policyName,
                policyCreated: false,
                policyId: null,
                jobCreated: false,
                jobId: null,
                policyPostBody: hyppolTplClone,                                                         // created up front, outbound to ECX via POST
                policyRollbackAttempted: false,
                policyRollbackSuccessful: false,
                jobRollbackAttempted: false,
                jobRollbackSuccessful: false,	                                                        // NB: set on 204 or 404 from DELETE
            };
            
            pjEntries.push(pjEntry);                                                                    // unlike recovery case, here we can have multiple policy/job pairs to create
        }
    }
    
    //ngp.logMsg('warn', 'HMR=' + JSON.stringify(pjEntries));
	var numPoliciesToCreate = pjEntries.length;

	var finishedPjEntries = [];                                                                         // 'finished' as in done with; maybe entity created, maybe not
	var crSummaryResults = [];                                                                          // will contain results of successful policy/job creation in pairs

// At any point in time pjEntries will contain unfinished (not yet presented to ECX pjEntries each of which represents a policy/job pair) and finishedPjEntries will contain finished. 
// The entities represented by an element in finishedPjEntries may or may not have been created. Stuff remaining in pjentries never got created and thus can't be rolled back.

	createPolicyAndJob(pjEntries, step, ecxSessionId, finishedPjEntries, crSummaryResults, function(failResult){
		if ( failResult )                                                                               // FTI rollback anything in finishedPjEntries that indicates it was created
		//if ( true )                                                                                   // uncomment to force rollback               
		{
			ngp.logMsg('warn', 'Detected failure of policy and job creation in session ' + ecxSessionId + '; attempting rollback'); 

			var rolledbackPolJobsAr = [];
			
			rollbackPolicyAndJob(finishedPjEntries, ecxSessionId, rolledbackPolJobsAr, function(){    
				var failResult = {
					statusCode: 500,                                                              
					response: {
						rollbackResults: rolledbackPolJobsAr,
					},                                                       
				};
                
                driverCb('ng', failResult);
			});
		}
		else
		{
			var rollbackState = {
				createdPolJobsAr: finishedPjEntries,
				rolledbackPolJobsAr: rolledbackPolJobsAr,                                            // !! perforce empty cuz all were created OK, but make explicit
            };
            
			ngp.setRequestState(step.reqId, ecxSessionId, 'policiesCreationResults', crSummaryResults);         // push to next step in case needed (successful) 
			ngp.setRequestState(step.reqId, ecxSessionId, 'finishedPjEntries', finishedPjEntries);              // push to next step in case it fails
            
            driverCb('ok');
    	}		
	});
}


function KEEP_createMissingPolicies(step, funcStream, ecxSessionId, oFromNg, createPolNamesHash, driverCb)                           
{
    ngp.logMsg('warn', 'FAUXFROMNG=' + JSON.stringify(oFromNg));
	ngp.logMsg('debug', 'Policy names=' + JSON.stringify(createPolNamesHash));

	var slapHash = ngp.arrayToHash(oFromNg.slapolicies, 'name');                                     // Input from client
	var pjEntries = [];                                                                                                                                         
	
	var rollback = {
		policies: [],
		jobs: [],
	};

	var policyNow = (new Date).getTime();

	for ( policyName in createPolNamesHash )
	{
		var pjEntry = {};
		var temp = policyName.split('_');                                                           // type, e.g., vmware, in element 0
		var entityType = temp.shift();
		var slapMetal = temp.join('_');

		var slap = slapHash[slapMetal];

		var hyppolTplClone = JSON.parse(JSON.stringify(ngp.hyppolTpl));
		hyppolTplClone.name = policyName;
		hyppolTplClone.subType = entityType;
		hyppolTplClone.spec.storageworkflow[0].id = slap.id;
		hyppolTplClone.spec.storageworkflow[0].name = slap.name;
		hyppolTplClone.spec.storageworkflow[0].href = slap.href;

		delete hyppolTplClone.spec.storageworkflow[0].trigger;
		
		var pjEntry = {
			policyName: policyName,
			policyCreated: false,
			policyId: null,
			jobCreated: false,
			jobId: null,
			policyPostBody: hyppolTplClone,                                                          // created up front, outbound to ECX via POST
			policyRollbackAttempted: false,
			policyRollbackSuccessful: false,
			jobRollbackAttempted: false,
			jobRollbackSuccessful: false,	                                                         // NB: set on 204 or 404 from DELETE
		};
		
		pjEntries.push(pjEntry);                                                                     // unlike recovery case, here we can have multiple policy/job pairs to create
	}

	//ngp.logMsg('warn', 'HMR=' + JSON.stringify(pjEntries));
	var numPoliciesToCreate = pjEntries.length;

	var finishedPjEntries = [];                                                                      // 'finished' as in done with; maybe entity created, maybe not
	var crSummaryResults = [];                                                                       // will contain results of successful policy/job creation in pairs

// At any point in time pjEntries will contain unfinished (not yet presented to ECX pjEntries each of which represents a policy/job pair) and finishedPjEntries will contain finished. 
// The entities represented by an element in finishedPjEntries may or may not have been created. Stuff remaining in pjentries never got created and thus can't be rolled back.

	createPolicyAndJob(pjEntries, step, ecxSessionId, finishedPjEntries, crSummaryResults, function(failResult){
		if ( failResult )                                                                            // FTI rollback anything in finishedPjEntries that indicates it was created
		//if ( true )                        // uncomment to force rollback               
		{
			ngp.logMsg('warn', 'Detected failure of policy and job creation in session ' + ecxSessionId + '; attempting rollback'); 

			var rolledbackPolJobsAr = [];
			
			rollbackPolicyAndJob(finishedPjEntries, ecxSessionId, rolledbackPolJobsAr, function(){    
				var failResult = {
					statusCode: 500,                                                              
					response: {
						rollbackResults: rolledbackPolJobsAr,
					},                                                       
				};
                
                driverCb('ng', failResult);
			});
		}
		else
		{
			var rollbackState = {
				createdPolJobsAr: finishedPjEntries,
				rolledbackPolJobsAr: rolledbackPolJobsAr,                                            // !! perforce empty cuz all were created OK, but make explicit
            };
            
			ngp.setRequestState(step.reqId, ecxSessionId, 'policiesCreationResults', crSummaryResults);         // push to next step in case needed (successful) 
			ngp.setRequestState(step.reqId, ecxSessionId, 'finishedPjEntries', finishedPjEntries);              // push to next step in case it fails
            
            driverCb('ok');
    	}		
	});
}

// createPolicyAndJob only returns after all entries in pjEntries (each entry represents a policy/job pair that must be created) have been processed; or after a failure

function createPolicyAndJob(pjEntries, driverStep, ecxSessionId, finishedPjEntries, crSummaryResults, policiesCreatedCb)    // RECURSIVE function
{
	if ( pjEntries.length < 1 )
	{
		policiesCreatedCb();
	}
	else
	{
		var pjEntry = pjEntries.shift();                                                             // leftmost element in array of pjEntry objects
		
		finishedPjEntries.push(pjEntry);                                                             // flags set (or not) below will determine need for rollback, i.e., something created but then some failure

		ngp.sendRequestToEcx(driverStep.verb, ngp.ecxExternal, driverStep.path, ecxSessionId, pjEntry.policyPostBody, function(hypprObj){    // Create POLICY
			if ( hypprObj.e || hypprObj.response.statusCode != 201 )                                 // failed to create the ECX policy  
			{
				var failResult = ngp.getGenericEcxErrorResponse(hypprObj, ecxSessionId, driverStep);
				policiesCreatedCb(failResult);			                                             // return to rollback first jobs, then policies
			}   
			else                                                                                     // a single policy was created, but non-terminal cuz we still need to drive link_resources
			{
				crSummaryResults.push(hypprObj.rO);                                                  // save to give back to caller; an array of the created policy/job pairs as returned by ECX

				pjEntry.policyId = hypprObj.rO.id;
				pjEntry.policyCreated = true;
				
				ngp.logMsg('debug', "Hypervisor protection policy creation response=" + JSON.stringify(hypprObj.rO)); 

				var jobTplClone = JSON.parse(JSON.stringify(ngp.jobTpl));

				jobTplClone.name = pjEntry.policyName;
				jobTplClone.description += pjEntry.policyName;
				jobTplClone.policyId = hypprObj.rO.id;

				ngp.sendRequestToEcx(driverStep.verb, ngp.ecxExternal, '/api/endeavour/job', ecxSessionId, jobTplClone, function(hyppjObj){    // create JOB
					if ( hyppjObj.e || hyppjObj.response.statusCode != 201 )
					{
						var failResult = ngp.getGenericEcxErrorResponse(hyppjObj, ecxSessionId, driverStep);
						policiesCreatedCb(failResult);
					}
					else
					{
						crSummaryResults.push(hyppjObj.rO);                                          // save to give back to caller 

						pjEntry.jobId = hyppjObj.rO.id;
						pjEntry.jobCreated = true;
						
						ngp.logMsg('debug', "Created hypervisor protection job=" + JSON.stringify(hyppjObj.rO)); 

						createPolicyAndJob(pjEntries, driverStep, ecxSessionId, finishedPjEntries, crSummaryResults, policiesCreatedCb);    // RECURSE
					}
				});
			}
		});		
	}
}