if (!('contains' in String.prototype)) {
	  String.prototype.contains = function(str, startIndex) {
	    return ''.indexOf.call(this, str, startIndex) !== -1;
	  };
	}

if (typeof String.prototype.startsWith != 'function') {
    String.prototype.startsWith = function(prefix) {
        return this.slice(0, prefix.length) == prefix;
    };
}
		 
if (typeof String.prototype.endsWith != 'function') {
    String.prototype.endsWith = function(suffix) {
        return this.slice(-suffix.length) == suffix;
    };
}


var ngp = require('./ecxngp');                                                                           


var funcSteps = [                                                                                 
	{                                                                                              
		continue: 'next',                                                                          
		name: 'registration_keyExists',                                                                      
		verb: 'GET',
		path: '/api/identity/key',
		ip: null,                                                                                                                                       
		port: null, 
		type: null,
		func: null,                                                                                 
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	}, 
	{                                                                                              
		continue: 'next',                                                                          
		name: 'registration_createKey',                                                                    
		verb: 'POST',
		path: '/api/identity/key',                                                            
		ip: null, 
		port: null, 
		type: null,
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	}, 
	{                                                                                              
		continue: 'next',         
		name: 'update_cloudProvider',                                                                      
		verb: 'PUT',    
		path: '/api/cloud',                                                                                
		ip: null, 
		port: null, 
		type: null,
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	}, 
	{                                                                                              
		continue: 'next',                                                                         
		name: 'registration_deleteKey',                                                                    
		verb: 'DELETE',    
		path: null,   
		ip: null, 
		port: null, 
		type: null,
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	}, 
];


const regProviderStepFuncs = {
	registration_keyExists: function(step, funcStream, ecxSessionId, oFromNg, cb){                                 
		getKeys(step, funcStream, ecxSessionId, oFromNg, cb);                                       
	},
	registration_createKey: function(step, funcStream, ecxSessionId, oFromNg, cb){                               
		createKey(step, funcStream, ecxSessionId, oFromNg, cb);                                     
	},
	update_cloudProvider: function(step, funcStream, ecxSessionId, oFromNg, cb){                                 
		updateCloudProvider(step, funcStream, ecxSessionId, oFromNg, cb);		
	},
	registration_deleteKey: function(step, funcStream, ecxSessionId, oFromNg, cb){                               
		regDeleteKey(step, funcStream, ecxSessionId, oFromNg, cb);
	},
};


exports.init = init;


function init(req, res)                                                      
{
	var ngState = ngp.getSessionState(req, res);

	if ( ngState.stale )                                                                     
		return;                                                                                

	var ecxSid = ngState.ecxSid;
	res.ecxSid = ecxSid;

	var stepsStream = JSON.parse(JSON.stringify(funcSteps));                                    

	stepsStream.forEach(function(step) {
		step.ip = ngp.localIp;
		step.port = ngp.ecxExternal;	
		step.func = regProviderStepFuncs[step.name];    
		
		if(step.name == 'update_cloudProvider')
		step.path = step.path + '/' + req.params.id;                                       
		
		step.cb = function(ecxSid, data){ 
			ngp.logMsg('info', "Finished step '" + ngp.getCurrentFunc(step.reqId, ecxSid) + '(' + this.name + ")' in session ID '" + ecxSid + "'");

			if ( data )
			{
				ngp.logMsg('debug', "Data returned by step '" + ngState.currentFunc + '(' + this.name + ")' in session ID '" + ecxSid + "'=" + JSON.stringify(data));
			}
		};		
	});

	ngp.createRequestStore(stepsStream, ecxSid, req.reqId, 'update_cloudProvider');  
	ngState.currentFunc = 'update_cloudProvider';         
	return stepsStream;
}

var updateCloudProvider = function(step, funcStream, ecxSessionId, oFromNg, cb)
{
	var scrubbedNgInput = ngp.scrubPostBody(oFromNg, true);
	ngp.logMsg('info', "Input from client for cloud provider update in session ID='" + ecxSessionId + "'=" + JSON.stringify(scrubbedNgInput));
	
	var ngState = ngp.ecxSidHash[ecxSessionId];
	var postBody = getProviderUpdatePostBody(ecxSessionId, step, ngState, oFromNg);
	
	ngp.sendRequestToEcx(step.verb, ngp.ecxExternal, step.path, ecxSessionId, postBody, function(regO) {
		if ( regO.e || regO.response.statusCode != 200 ) {
			var failResult = ngp.getGenericEcxErrorResponse(regO, ecxSessionId, step); 
			var createdKeys = ngp.getRequestState(step.reqId, ecxSessionId, 'createdKeys');
			   
			if ( ! createdKeys || createdKeys.length < 1 ) {
				if ( funcStream.length > 0 ) 
				funcStream[0].continue = 'skip'; 	
				ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, true);       
			}
			else {
				ngp.setSerialStreamResult(ecxSessionId, failResult);                               
				step.cb(ecxSessionId, regO.rO);                                                  
				ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb);				    
			}  			   
		}			   		    
		else {   			
			var finalResponce = regO.rO;

			ngp.logMsg('warn', "Update OK response=" + JSON.stringify(regO.rO));  	
			var successResult = {
				statusCode: regO.response.statusCode,
			   	response: finalResponce,			
			};

			ngp.setSerialStreamResult(ecxSessionId, successResult);                              
		    if ( funcStream.length > 0 ) {				
				funcStream[0].continue = 'skip';                                                   
	        }

	 	    step.cb(ecxSessionId, regO.rO);                                                  
		    ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb); 				
	    }
    });    
};

var getKeys = function(step, funcStream, ecxSessionId, oFromNg, cb)
{
	if(! oFromNg.accesskey.name) {
		ngp.setRequestState(step.reqId, ecxSessionId, 'accessKeyHREF', oFromNg.accesskey);
		if ( funcStream.length > 0 ) {
			funcStream[0].continue = 'skip';                                                   
		}

		step.cb(ecxSessionId, true);                                                  
		ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb); 	
	} 
	else {
		var ngState = ngp.ecxSidHash[ecxSessionId];
		var keyExists = false;
		ngp.ngpCoreHelper.getKeys(ecxSessionId, function(getKeysResObj) {
			var arrayOfKeys = getKeysResObj.rO['keys'];
						
			for (var i = 0; i < arrayOfKeys.length; i++) {
				if(arrayOfKeys[i]['name']==oFromNg.accesskey.name) {
					ngp.setRequestState(step.reqId, ecxSessionId, 'accessKeyHREF',arrayOfKeys[i]['links']['self']['href']);					
					keyExists = true;
					break;
				}
			}

			if(keyExists == true) {
				if ( funcStream.length > 0 ) {
					funcStream[0].continue = 'skip';                                                   
				}
		
				step.cb(ecxSessionId, true);                                                  
				ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb); 
			}
			else {
				step.cb(ecxSessionId, true);                                                  
				ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb); 
			}
		});

	}   
};

function getProviderUpdatePostBody(ecxSessionId, step, ngState, oFromNg)             
{
	var postBody = null;

	postBody = {  
		name: oFromNg.name,
		properties: oFromNg.properties,
		accesskey: {
			href: ngp.getRequestState(step.reqId, ecxSessionId, 'accessKeyHREF'),                                
		}
	};

	return postBody;	
}

var createKey = function(step, funcStream, ecxSessionId, oFromNg, cb) {  
	var postBody = {  
			name: oFromNg.accesskey.name,
			keytype: oFromNg.accesskey.keytype,
			secret: oFromNg.accesskey.secret,
			access: oFromNg.accesskey.access
		};

	ngp.sendRequestToEcx(step.verb, step.port, step.path, ecxSessionId, postBody, function(rObj){           
		if ( rObj.e ) {
			var failResult = ngp.getGenericEcxErrorResponse(rObj, ecxSessionId, step); 
			ngp.logMsg('error', 'Failed to create Access Key ' + postBody.name + ' for session ID=' + ecxSessionId + ' because of error=' + JSON.stringify(e)); 
			ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);			
			return;
		}   
		
		var crO = rObj.rO;

		if ( rObj.response.statusCode != 200 ) {
			var failResult = ngp.getGenericEcxErrorResponse(rObj, ecxSessionId, step); 
			ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);
		}
		else {									   
			ngp.logMsg('warn', 'Created Access Key with name=' + postBody.name + ' and href=' + JSON.stringify(crO.links.self.href));
			ngp.setRequestState(step.reqId, ecxSessionId, 'accessKeyHREF',crO.links.self.href);

			var createdKeys = ngp.getRequestState(step.reqId, ecxSessionId, 'createdKeys');

			if ( ! createdKeys ) {
				createdKeys = [crO.links.self.href];
				ngp.setRequestState(step.reqId, ecxSessionId, 'createdKeys', createdKeys); 
			}
			else {
				createdKeys.push(crO.links.self.href);
			}

			step.cb(ecxSessionId, crO);                                                   
			ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb); 			
		}
	});
}

var regDeleteKey = function(step, funcStream, ecxSessionId, oFromNg, cb)                          
{
	var createdKeys = ngp.getRequestState(step.reqId, ecxSessionId, 'createdKeys');                   
	if ( createdKeys && createdKeys.length > 0 ) {
		ngp.logMsg('debug', 'Created Access Keys to delete=' + JSON.stringify(createdKeys));
		
		deleteOneKey(createdKeys, ecxSessionId, function(){
			
			step.cb(ecxSessionId, null); 
			ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb); 			
		});
	}
	else 
	{
		step.cb(ecxSessionId, null); 
		ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb); 
	}
};

function deleteOneKey(keysToDelete, ecxSessionId, cb)
{
	if ( keysToDelete.length < 1 )
		cb();
	else {		
		var keyToDelete = keysToDelete.shift();
		const urlParse = require('url-parse');
		var path = urlParse(keyToDelete).pathname;

		ngp.sendRequestToEcx('DELETE', ngp.ecxExternal, path, ecxSessionId, null, function(dlObj){           
			if ( dlObj.response.statusCode != 204 )
				ngp.logMsg('error', "Deletion of SPP Access Key '" + keyToDelete + "' failed because of error=" + JSON.stringify(dlObj.rO));                         
			
			deleteOneKey(keysToDelete, ecxSessionId, cb);
		});
	}
}