if (!('contains' in String.prototype)) {
	  String.prototype.contains = function(str, startIndex) {
	    return ''.indexOf.call(this, str, startIndex) !== -1;
	  };
	}

if (typeof String.prototype.startsWith != 'function') {
    String.prototype.startsWith = function(prefix) {
        return this.slice(0, prefix.length) == prefix;
    };
}
		 
if (typeof String.prototype.endsWith != 'function') {
    String.prototype.endsWith = function(suffix) {
        return this.slice(-suffix.length) == suffix;
    };
}


var ngp = require('./ecxngp');                                                                           


var funcSteps = [                                                                                  // generalized user handling + provider registration
	{                                                                                              // NB: nothing in this block will change
		continue: 'next',                                                                          // first constituent always continues
		name: 'upd_isprovider',
		verb: 'GET',
		path: null,                                                                                // further qualified by init: /api/hypervisor, /api/storage, /api/application
		ip: null,                                                                                  //!! set by ecxngp. but maybe you don't really need per step ip and ort                                                     
		port: null, 
		type: null,
		func: null,                                                                                // see initialization below 
		cb: function(res, data){ logMsg('info', 'Step=' + this.name + ' finished');}
	}, 
	{                                                                                              // NB: nothing in this block will change
		continue: 'next',                                                                          // first constituent always continues
		name: 'upd_usermode',
		verb: 'GET',
		path: '/api/identity/user',
		ip: null,                                                                                  //!! set by ecxngp. but maybe you don't really need per step ip and ort                                                     
		port: null, 
		type: null,
		func: null,                                                                                // see initialization in regProviderCommon 
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	}, 
	{                                                                                              // NB: nothing in this block will change
		continue: 'next',                                                                          // set by previous step [ next | skip | fail ]     
		name: 'upd_createuser',   
		verb: 'POST',
		path: '/api/identity/user',                                                            
		ip: null, 
		port: null, 
		type: null,
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	}, 
	{                                                                                              // NB: this block is changed before registration to conform to entity being registered
		continue: 'next',         
		name: 'upd_provider', 
		verb: 'PUT',    
		path: null,                                                                                // NB: depends on type of entity we are registering, further qualified by init
		ip: null, 
		port: null, 
		type: null,
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	}, 
	{                                                                                              // NB: this block is changed before registration to conform to entity being registered
		continue: 'next',                                                                          // set to skip if reg_register succeeds, but if driven it will delete identity only if we created one  
		name: 'upd_deleteuser',                                                                    
		verb: 'DELETE',    
		path: null,   
		ip: null, 
		port: null, 
		type: null,
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	}, 	
];


const updProviderStepFuncs = {
	upd_isprovider: function(step, funcStream, ecxSessionId, oFromNg, cb){ 
		isProvider(step, funcStream, ecxSessionId, oFromNg, cb);
	},    
	upd_usermode: function(step, funcStream, ecxSessionId, oFromNg, cb){ 
		ngp.userMode(step, funcStream, ecxSessionId, oFromNg, cb);
	},
	upd_createuser: function(step, funcStream, ecxSessionId, oFromNg, cb){ 
		ngp.createUser(step, funcStream, ecxSessionId, oFromNg, cb); 		
	},
	upd_provider: function(step, funcStream, ecxSessionId, oFromNg, cb){ 
		updateProvider(step, funcStream, ecxSessionId, oFromNg, cb);		
	},
	upd_deleteuser: function(step, funcStream, ecxSessionId, oFromNg, cb){ 
		ngp.regDeleteUser(step, funcStream, ecxSessionId, oFromNg, cb);
	},			
};


exports.init = init;


function init(req, res, providerType)                                    
{
	var ngState = ngp.getSessionState(req, res);

	if ( ngState.stale )                                                                                    // FTI we got bounced while client remained up. AND response already sent.
		return;                                                                                             // !!! R U CERTAIN THAT RESPONSE SENT IN ALL CASES HERE?

	var ecxSid = ngState.ecxSid;
	res.ecxSid = ecxSid;

	var typology = ngp.getRegistrationTypology(req, providerType);                                        

	var stepsStream = JSON.parse(JSON.stringify(funcSteps));                                    

	var updStep = null;
    var isProviderStep = null;

	stepsStream.forEach(function(step){
		step.ip = ngp.localIp;
		step.port = ngp.ecxExternal;

		step.resourceType = typology.resourceType;
		step.type = typology.type;                                                                          // NB: we need to be told the API path for type we are registering

        if ( step.name == 'upd_isprovider' || step.name == 'upd_provider' )
			step.path = typology.path + '/' + req.params.id;	

		step.func = updProviderStepFuncs[step.name];                                                        // hook up the step's real workhorse
		
		step.cb = function(ecxSid, data){ 
			ngp.logMsg('info', "Finished step '" + ngState.currentFunc + '(' + this.name + ")' in session ID '" + ecxSid + "'");

			if ( data )
			{
				ngp.logMsg('debug', "Data returned by step '" + ngState.currentFunc + '(' + this.name + ")' in session ID '" + ecxSid + "'=" + JSON.stringify(data));
			}
		};
	});

	ngState.currentFunc = 'update_' + typology.resourceType + '_' + typology.type;                        // e.g., register_storage_netapp
	return stepsStream;
}


/*
  Despite this being PUT logic, we still GET the object before updating it to capture the current (perhaps soon to be previous) user associated with the provider. Later, we may use try to 
  delete this previous user.
*/

function isProvider(step, funcStream, ecxSessionId, oFromNg, cb) {                       //!! BECUZ used by serveral submodules, keep in ecxngp   
	var scrubbedNgInput = ngp.scrubPostBody(oFromNg, true);
	ngp.logMsg('warn', "Input from client for provider existence in session ID='" + ecxSessionId + "'=" + JSON.stringify(scrubbedNgInput));

	ngp.sendRequestToEcx(step.verb, ngp.ecxExternal, step.path, ecxSessionId, null, function(provObj){   
		if ( provObj.e || provObj.response.statusCode != 200 )                               
		{
			var failResult = ngp.getGenericEcxErrorResponse(provObj, ecxSessionId, step);
			ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult);
		}
		else                                                                                   
		{
			var ngState = ngp.ecxSidHash[ecxSessionId];    

			var effUserProp = 'user';

			if ( (step.path.contains('appserver')) || ( step.path.contains('scriptserver') ) )
				effUserProp = 'osuser';
			
				var prov = provObj.rO;                                                             // no longer compare name property; the URL :id gave us what it did

			if ( ngState.currentFunc.startsWith('update') )                                    //!! ALWAYS SO, AT LEAST UNTIL i MOVE THIS CODE TO ecxngp and use w/other funcs
			{
				ngp.setEndpointState(ecxSessionId, 'editHref', prov.links.edit.href);				
				ngp.logMsg('debug', 'Extant provider=' + JSON.stringify(prov));
				
				ngp.setEndpointState(ecxSessionId, 'prevUserHref', prov[effUserProp].href);      // we will try to delete this if update has diff user		

				step.cb(ecxSessionId, provObj.rO);                                                   
				ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb); 	   
			}
		}
	});	
};


var updateProvider = function(step, funcStream, ecxSessionId, oFromNg, cb)
{
	var scrubbedNgInput = ngp.scrubPostBody(oFromNg, true);
	ngp.logMsg('warn', "Input from client for provider update in session ID='" + ecxSessionId + "'=" + JSON.stringify(scrubbedNgInput));

	var ngState = ngp.ecxSidHash[ecxSessionId];

	var postBody = getProviderUpdatePutBody(step, ngState, oFromNg);  //!! LOSES COMMENT
    
	var path = ngp.getEndpointState(ecxSessionId, 'editHref').split(/\:\d{4}/)[1];
	
	ngp.sendRequestToEcx(step.verb, ngp.ecxExternal, path, ecxSessionId, postBody, function(regO){ 
		if ( regO.e || regO.response.statusCode != 200 )   
		{
			var failResult = ngp.getGenericEcxErrorResponse(regO, ecxSessionId, step); 
			
			var createdIdentities = ngp.getEndpointState(ecxSessionId, 'createdIdentities'); 

			if ( ( ! createdIdentities || createdIdentities.length < 1 ) && funcStream.length > 0 )     // FTI we never created an identity AND subsequent step is present                                  
				funcStream[0].continue = 'skip'; 
			else                                                                                 // FTI PUT failed && we created a user; delete the user (rollback)
			{
				ngp.logMsg('warn', 'Detected PUT failure in session ' + ecxSessionId + '; attempting rollback of user identity=' + identityToDelete); 
				
				ngp.sendRequestToEcx('DELETE', ngp.ecxExternal, identityToDelete, ecxSessionId, null, function(delO){  
					/* SRTE has already complained.
					if ( delO.response && delO.response.statusCode != 204 )
					{
						ngp.logMsg('warn', 'Deletion of user identity=' + identityToDelete + ' failed because of error=' + JSON.stringify(delO.rO));
					}
					*/
				});
			}

			ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, true);    // boolean arg causes ssfr not to set 'fail' (we've already set 'skip')
		}
		else                                                                                      // EHI PUT worked; maybe we delete the provider's previous user if we replaced it with another user
		{
			ngp.logMsg('debug', "Update OK response=" + JSON.stringify(regO.rO));  

			var oldUserHref = ngp.getEndpointState(ecxSessionId, 'prevUserHref').split(/\:\d{4}/)[1];	

			var effUserProp = 'user';

			if ( (step.path.contains('appserver')) || ( step.path.contains('scriptserver') ) )
				effUserProp = 'osuser';

			var prov = regO.rO;
			/* PUSH MORNING OF 8/4 (see above)
			if ( ! prov[effUserProp] )                                                                 // argghh...some providers have top-level user.href
			{
				var perhapsNewUserHref = prov.links[effUserProp].href.split(/\:\d{4}/)[1];	
			}
			else
			*/
			var perhapsNewUserHref = prov[effUserProp].href.split(/\:\d{4}/)[1];					

// NB: if ECX R.I. associations are intact, it cannot hurt to try to delete the previous user identity if we just applied a new user. This is FIRE & FORGET; it works or it don't.

			if ( oldUserHref != perhapsNewUserHref )                                             // FTI old user may no longer be attached to any other resource; try to delete it
			{
				ngp.sendRequestToEcx('DELETE', ngp.ecxExternal, oldUserHref, ecxSessionId, null, function(delO){  
					if ( delO.response && delO.response.statusCode == 204 )
						ngp.logMsg('debug', "Deleted provider's previous user association=" + oldUserHref);
				});
			}

			var successResult = {
				statusCode: regO.response.statusCode,
				response: regO.rO,
			};

			ngp.setSerialStreamResult(ecxSessionId, successResult);                              // NB: let caller see result of successful update

			if ( funcStream.length > 0 )                                                         // FTI there is a next step (always first element in what remains of array)
			{
				funcStream[0].continue = 'skip';                                                 // cause skip of next serial step (deletion of identity)                        
			}

			step.cb(ecxSessionId, regO.rO);                                                      // From ECX callback now drive the recursive driver to move to next constituent function, if any                                    
			ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb); 					
		}
	});
};


function getProviderUpdatePutBody(step, ngState, oFromNg)                                  
{
	var postBody = null;

	postBody = {  
		addToCatJob: true,
		hostAddress: oFromNg.hostAddress,                                                             
		//type: oFromNg.type,		//!! SHOULD BE COMMON TO ALL, but isn't yet	
		portNumber: oFromNg.portNumber,                                                        // use whatever GUI gives us, if anything; else backend decides
		sslConnection: oFromNg.sslConnection,                                                  // ditto
        comment: oFromNg.comment,                                                              //!! backend seems to exclude now; too bad, useful to test update
		user: {
			href: ngp.getEndpointState(ngState.ecxSid, 'userHref'),	                           //!! NOT NECESSARILY EVIDENCE THAT WE CREATED THE USER!						
		},
	};

// Handle DIFFERENTIAE here. Note that funcO.resourceType is assigned by regProviderCommon based on the typology. It does NOT come from GUI.

	if ( step.resourceType == 'hypervisor' )
	{
		delete postBody.type;
		postBody.type = oFromNg.type;
		postBody.name = oFromNg.hostAddress; 
		postBody.opProperties = oFromNg.opProperties;                                          // always, even in 'simple' (no VE) update case [ this module will be replaced by VE version]
	}

	if ( step.resourceType == 'storage' )
	{
		var hrefUrl = postBody.user.href;
		postBody.user = {href: hrefUrl};
		postBody.type = oFromNg.type;
		postBody.siteId = oFromNg.siteId;
		postBody.name = oFromNg.hostAddress;  
	}

//!! only HANA requires appCredentials: oFromNg.appCredentials,
	if (step.resourceType == 'appserver' )
	{
		postBody.osuser = postBody.user;
        postBody.addToCatJob = oFromNg.addToCatJob === undefined ? true : oFromNg.addToCatJob;
        postBody.application = oFromNg.application === undefined ? false : oFromNg.application;
        postBody.script = oFromNg.script === undefined ? true : oFromNg.script;
		delete postBody.user;

		postBody.osType = oFromNg.osType;
		postBody.name = oFromNg.hostAddress;  
		postBody.useKeyAuthentication = oFromNg.useKeyAuthentication;
		postBody.opProperties = oFromNg.opProperties;

		if(!oFromNg.portNumber)
		delete postBody.portNumber;
		
		delete postBody.sslConnection;
	}

	if (step.resourceType == 'scriptserver' )
	{
		postBody.osuser = postBody.user;
		
		delete postBody.user;

		postBody.osType = oFromNg.osType;
		postBody.name = oFromNg.hostAddress;  
		postBody.useKeyAuthentication = oFromNg.useKeyAuthentication;

		delete postBody.portNumber;
		delete postBody.sslConnection;
		delete postBody.addToCatJob;
	}


	return postBody;
}

