if (!('contains' in String.prototype)) {
	  String.prototype.contains = function(str, startIndex) {
	    return ''.indexOf.call(this, str, startIndex) !== -1;
	  };
	}

if (typeof String.prototype.startsWith != 'function') {
    String.prototype.startsWith = function(prefix) {
        return this.slice(0, prefix.length) == prefix;
    };
}
		 
if (typeof String.prototype.endsWith != 'function') {
    String.prototype.endsWith = function(suffix) {
        return this.slice(-suffix.length) == suffix;
    };
}

var ngp = require('./ecxngp');
var url = require('url');

var vadpProxyAvailabilityCheckCount = 0;

const vadpRegisterTplFile = ngp.PUBLICDIR + '/vadpregistertpl.json';
var vadpRegisterTpl = null;

getConfiguration();

function getConfiguration()
{
    try
    {
        var vadpRegistertemp = ngp.fs.readFileSync(vadpRegisterTplFile, 'utf8');
        vadpRegisterTpl = JSON.parse(vadpRegistertemp);
    }
    catch(e)
    {
        ngp.logMsg('error', "Failed to read VADP register template file because of error=" + e.toString());
        return;
    }
}

var installAndRegisterFunctionSteps = [
	{                                                                                              
		continue: 'next',                                                                          
		name: 'registration_identityExists',                                                                      
		verb: 'GET',
		path: '/api/identity/user',
		ip: null,                                                                                                                                       
		port: null, 
		type: null,
		func: null,                                                                                 
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	}, 
	{                                                                                              
		continue: 'next',                                                                          
		name: 'registration_createIdentity',                                                                    
		verb: 'POST',
		path: '/api/identity/user',                                                            
		ip: null, 
		port: null, 
		type: null,
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
    }, 
    {
		continue: 'next',
		name: 'vadp_register',
		verb: null,  
		path: null,  
		ip: null,
		port: null, 
        type: null,
        skipInstall: 'false',
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
    },
    {                                                                                              
		continue: 'next',                                                                         
		name: 'vadp_register_deleteIdentity',                                                                    
		verb: 'DELETE',    
		path: null,   
		ip: null, 
		port: null, 
		type: null,
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	}  
];

const installAndRegisterFunctions = {
    registration_identityExists: function(step, funcStream, ecxSessionId, oFromNg, cb){                                 
		getIdentities(step, funcStream, ecxSessionId, oFromNg, cb);                                       
	},
	registration_createIdentity: function(step, funcStream, ecxSessionId, oFromNg, cb){                               
		createIdentity(step, funcStream, ecxSessionId, oFromNg, cb);                                     
	},	
    vadp_register: function(step, funcStream, ecxSessionId, oFromNg, cb){
		registerVadpProxy(step, funcStream, ecxSessionId, oFromNg, cb);
    },
    vadp_register_deleteIdentity: function(step, funcStream, ecxSessionId, oFromNg, cb){                                 
		regDeleteIdentity(step, funcStream, ecxSessionId, oFromNg, cb);                                       
	}	
};

/**
 * Init function thats sets the steps and its values
 *
 * @param req
 * @param res
 * @param functionType
 */
function init(req, res)                                    
{
    ngp.logMsg('info', 'init()  ' + JSON.stringify(req.query));
	var ngState = ngp.getSessionState(req, res);

	if ( ngState.stale ) 
		return;

	var ecxSid = ngState.ecxSid;
	res.ecxSid = ecxSid;

    var stepsStream = JSON.parse(JSON.stringify(installAndRegisterFunctionSteps));
    ngp.createRequestStore(stepsStream, ecxSid, req.reqId, 'vadp_installandregister');
    ngState.currentFunc = 'vadp_installandregister';

	stepsStream.forEach(function(step) {
		step.ip = ngp.localIp;
        step.port = ngp.ecxExternal;
        
        step.func = installAndRegisterFunctions[step.name];
        
        if (step.name == 'vadp_register_deleteIdentity') {
            if(!req.body['identityId']['username'])			
			step.continue = 'skip';
		}

        step.cb = function(ecxSid, data) {            			
			ngp.logMsg('info', "Finished step '" + ngp.getCurrentFunc(step.reqId, ecxSid) + '(' + this.name + ")' in session ID '" + ecxSid + "'");

			if ( data )
			{
				ngp.logMsg('debug', "Data returned by step '" + ngState.currentFunc + '(' + this.name + ")' in session ID '" + ecxSid + "'=" + JSON.stringify(data));
			}
		};
	});

	return stepsStream;
}

function registerVadpProxy(step, funcStream, ecxSessionId, oFromNg, cb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'STARTING REGISTRATION OF VADP PROXY ['+ oFromNg['pushinstall']['hostAddress'] + ']');
    ngp.addDoubleSpace();
  
    if (oFromNg['registration']['siteId']) { 
         
        ngp.ngpCoreHelper.registerVadpProxy(ecxSessionId, createVadpRegisterPostBody(step, ecxSessionId, oFromNg['registration']['siteId'],oFromNg['pushinstall']['hostAddress']), function(registerVadpProxyResObj){
            if (registerVadpProxyResObj.response.statusCode != 201) {
                ngp.logMsg('error', '   Registeration of VADP proxy ['+ oFromNg['pushinstall']['hostAddress'] + '] failed. Error ==> ' + JSON.stringify(registerVadpProxyResObj.rO));
                var failResult = {
                    statusCode: 500,
                    response: {
                        response: registerVadpProxyResObj.rO
                    }
                };

                ngp.setRequestState(step.reqId, ecxSessionId, 'failureResult', failResult);

                if(!oFromNg['identityId']['username'])
                ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);
                else
                ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, true);    
            }
            else {
                ngp.addDoubleSpace();
                ngp.logMsg('info', '==> Done installing and registering VADP proxy [' + oFromNg['pushinstall']['hostAddress'] +']');

                var successResult = {
                        statusCode: 201,
                        response: registerVadpProxyResObj.rO,
                };
                 if ( funcStream.length > 0 ) {
                        funcStream[0].continue = 'skip';                                                   
                    }
                   
                ngp.setStepSuccessResult(step, funcStream, ecxSessionId, oFromNg, cb, successResult);                
            }
        });
    }
}

var getIdentities = function(step, funcStream, ecxSessionId, oFromNg, cb)
{
	if(! oFromNg['identityId']['username']) {
		ngp.setRequestState(step.reqId, ecxSessionId, 'identityId', oFromNg['identityId']);
		if ( funcStream.length > 0 ) {
			funcStream[0].continue = 'skip';                                                   
		}

		step.cb(ecxSessionId, true);                                                  
		ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb); 	
	} 
	else {
		var ngState = ngp.ecxSidHash[ecxSessionId];
		var identityExists = false;
		ngp.ngpCoreHelper.getIdentities(ecxSessionId, function(getIdentitiesResObj) {
            
			var arrayOfIdentities = getIdentitiesResObj.rO['users'];
						
			for (var i = 0; i < arrayOfIdentities.length; i++) {
                var identityName = require('querystring').unescape(oFromNg['identityId']['username'] + '_' + oFromNg['pushinstall']['hostAddress']);
				if(arrayOfIdentities[i]['name']==identityName) {
                    ngp.setRequestState(step.reqId, ecxSessionId, 'identityId',arrayOfIdentities[i]['id']);					
					identityExists = true;
					break;
				}
			}

			if(identityExists == true) {
				if ( funcStream.length > 0 ) {
					funcStream[0].continue = 'skip';                                                   
				}
			}

            step.cb(ecxSessionId, true);                                                  
			ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb); 
		});

	}   
};

var createIdentity = function(step, funcStream, ecxSessionId, oFromNg, cb) {  
	var postBody = {  
        name: require('querystring').unescape(oFromNg['identityId']['username'] + '_' + oFromNg['pushinstall']['hostAddress']),
        password: oFromNg['identityId']['password'],
        username: require('querystring').unescape(oFromNg['identityId']['username']),
    };    
    
	ngp.sendRequestToEcx(step.verb, step.port, step.path, ecxSessionId, postBody, function(rObj){           
		if ( rObj.e ) {
			var failResult = ngp.getGenericEcxErrorResponse(rObj, ecxSessionId, step); 
			ngp.logMsg('error', 'Failed to create Identity ' + postBody.username + ' for session ID=' + ecxSessionId + ' because of error=' + JSON.stringify(e)); 
			ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);			
			return;
		}   
		
        var crO = rObj.rO;
                
		if ( rObj.response.statusCode != 201 ) {
			var failResult = ngp.getGenericEcxErrorResponse(rObj, ecxSessionId, step); 
			ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);
		}
		else {									   
			ngp.logMsg('warn', 'Created Identity with name=' + postBody.username + ' and href=' + JSON.stringify(crO.links.self.href));
            ngp.setRequestState(step.reqId, ecxSessionId, 'identityId', crO.id);
            
			var createdIdentities = ngp.getRequestState(step.reqId, ecxSessionId, 'createdIdentities');

			if ( ! createdIdentities ) {
				createdIdentities = [crO.links.self.href];
				ngp.setRequestState(step.reqId, ecxSessionId, 'createdIdentities', createdIdentities); 
			}
			else {
				createdIdentities.push(crO.links.self.href);
			}

			step.cb(ecxSessionId, true);                                                   
			ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb); 			
		}
	});
}

var regDeleteIdentity = function(step, funcStream, ecxSessionId, oFromNg, cb)                          
{
    var failResult = ngp.getRequestState(step.reqId, ecxSessionId, 'failureResult');
    
    var createdIdentities = ngp.getRequestState(step.reqId, ecxSessionId, 'createdIdentities');                   
	if ( createdIdentities && createdIdentities.length > 0 ) {
		ngp.logMsg('debug', 'Created Identities to delete=' + JSON.stringify(createdIdentities));
		
		deleteOneIdentity(createdIdentities, ecxSessionId, function(){
			
			ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);		
		});
	}
	else 
	{
		ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);
	}
};

function deleteOneIdentity(identitiesToDelete, ecxSessionId, cb)
{
	if ( identitiesToDelete.length < 1 )
		cb();
	else {
        
        
		var identityToDelete = identitiesToDelete.shift();

        const urlParse = require('url-parse');
		var path = urlParse(identityToDelete).pathname;

		ngp.sendRequestToEcx('DELETE', ngp.ecxExternal, path, ecxSessionId, null, function(dlObj){           
			if ( dlObj.response.statusCode != 204 )
				ngp.logMsg('error', "Deletion of SPP Identity '" + identityToDelete + "' failed because of error=" + JSON.stringify(dlObj.rO));                         
			
                deleteOneIdentity(identitiesToDelete, ecxSessionId, cb);
		});
	}
}
/**
 * Create VADP proxy POST body
 * 
 * @param step 
 * @param ecxSessionId 
 * @param siteId 
 */
function createVadpRegisterPostBody(step, ecxSessionId, siteId, hostAddress){
    var identityIdForPostBody = ngp.getRequestState(step.reqId, ecxSessionId, 'identityId');
    var vadpRegisterTplClone = JSON.parse(JSON.stringify(vadpRegisterTpl));
    vadpRegisterTplClone.ipAddr = hostAddress;
    vadpRegisterTplClone.siteId = siteId; 
    vadpRegisterTplClone.identityId = identityIdForPostBody;     
    return vadpRegisterTplClone;
}

exports.init = init;