if (!('contains' in String.prototype)) {
	  String.prototype.contains = function(str, startIndex) {
	    return ''.indexOf.call(this, str, startIndex) !== -1;
	  };
	}

if (typeof String.prototype.startsWith != 'function') {
    String.prototype.startsWith = function(prefix) {
        return this.slice(0, prefix.length) == prefix;
    };
}
		 
if (typeof String.prototype.endsWith != 'function') {
    String.prototype.endsWith = function(suffix) {
        return this.slice(-suffix.length) == suffix;
    };
}


var ngp = require('./ecxngp');                                                                           
const vadpRegisterTplFile = ngp.PUBLICDIR + '/vadpregistertpl.json';
var vadpRegisterTpl = null;

getConfiguration();

function getConfiguration()
{
    try
    {
        var vadpRegistertemp = ngp.fs.readFileSync(vadpRegisterTplFile, 'utf8');
        vadpRegisterTpl = JSON.parse(vadpRegistertemp);
    }
    catch(e)
    {
        ngp.logMsg('error', "Failed to read VADP register template file because of error=" + e.toString());
        return;
    }
}

var funcSteps = [                                                                                 
	{                                                                                              
		continue: 'next',                                                                          
		name: 'registration_identityExists',                                                                      
		verb: 'GET',
		path: '/api/identity/user',
		ip: null,                                                                                                                                       
		port: null, 
		type: null,
		func: null,                                                                                 
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	}, 
	{                                                                                              
		continue: 'next',                                                                          
		name: 'registration_createIdentity',                                                                    
		verb: 'POST',
		path: '/api/identity/user',                                                            
		ip: null, 
		port: null, 
		type: null,
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	}, 
	{                                                                                              
		continue: 'next',         
		name: 'update_VADP',                                                                      
		verb: 'PUT',    
		path: '/api/vadp',                                                                                
		ip: null, 
		port: null, 
		type: null,
		id: null,
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	}, 
	{                                                                                              
		continue: 'next',                                                                         
		name: 'registration_deleteIdentity',                                                                    
		verb: 'DELETE',    
		path: null,   
		ip: null, 
		port: null, 
		type: null,
		cb: function(res, data){ ngp.logMsg('info', 'Step=' + this.name + ' finished');}
	}, 
];


const updateVADPStepFuncs = {
	registration_identityExists: function(step, funcStream, ecxSessionId, oFromNg, cb){                                 
		getIdentities(step, funcStream, ecxSessionId, oFromNg, cb);                                       
	},
	registration_createIdentity: function(step, funcStream, ecxSessionId, oFromNg, cb){                               
		createIdentity(step, funcStream, ecxSessionId, oFromNg, cb);                                     
	},
	update_VADP: function(step, funcStream, ecxSessionId, oFromNg, cb){                                 
		updateVADP(step, funcStream, ecxSessionId, oFromNg, cb);		
	},
	registration_deleteIdentity: function(step, funcStream, ecxSessionId, oFromNg, cb){                               
		regDeleteIdentity(step, funcStream, ecxSessionId, oFromNg, cb);
	},
};


exports.init = init;


function init(req, res)                                                      
{
	var ngState = ngp.getSessionState(req, res);

	if ( ngState.stale )                                                                     
		return;                                                                                

	var ecxSid = ngState.ecxSid;
	res.ecxSid = ecxSid;

	var stepsStream = JSON.parse(JSON.stringify(funcSteps));                                    

	stepsStream.forEach(function(step) {
		step.ip = ngp.localIp;
		step.port = ngp.ecxExternal;	
		step.func = updateVADPStepFuncs[step.name];    
		
		if(step.name == 'update_VADP')
		step.id = req.params.id;                                       
		
		step.cb = function(ecxSid, data){ 
			ngp.logMsg('info', "Finished step '" + ngp.getCurrentFunc(step.reqId, ecxSid) + '(' + this.name + ")' in session ID '" + ecxSid + "'");

			if ( data )
			{
				ngp.logMsg('debug', "Data returned by step '" + ngState.currentFunc + '(' + this.name + ")' in session ID '" + ecxSid + "'=" + JSON.stringify(data));
			}
		};		
	});

	ngp.createRequestStore(stepsStream, ecxSid, req.reqId, 'update_VADP');  
	ngState.currentFunc = 'update_VADP';         
	return stepsStream;
}

function updateVADP(step, funcStream, ecxSessionId, oFromNg, cb){
    ngp.addDoubleSpace();
    ngp.logMsg('info', 'STARTING UPDATE OF VADP PROXY ['+ oFromNg['pushinstall']['hostAddress'] + ']');
    ngp.addDoubleSpace();
  
    if (oFromNg['registration']['siteId']) { 
         
        ngp.ngpCoreHelper.updateVadpProxy(ecxSessionId, createVadpUpdatePutBody(step, ecxSessionId, oFromNg['registration']['siteId'],oFromNg['pushinstall']['hostAddress']), step.id, function(updateVadpProxyResObj){
            if (updateVadpProxyResObj.response.statusCode != 200) {
                ngp.logMsg('error', '   Update of VADP proxy ['+ oFromNg['pushinstall']['hostAddress'] + '] failed. Error ==> ' + JSON.stringify(updateVadpProxyResObj.rO));
                var failResult = {
                    statusCode: 500,
                    response: {
                        response: updateVadpProxyResObj.rO
                    }
                };

                ngp.setRequestState(step.reqId, ecxSessionId, 'failureResult', failResult);

                if(!oFromNg['identityId']['username'])
                ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);
                else
                ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, true);    
            }
            else {
                ngp.addDoubleSpace();
                ngp.logMsg('info', '==> Done updating VADP proxy [' + oFromNg['pushinstall']['hostAddress'] +']');

                var successResult = {
                        statusCode: 201,
                        response: updateVadpProxyResObj.rO,
                };
                 if ( funcStream.length > 0 ) {
                        funcStream[0].continue = 'skip';                                                   
                    }
                   
                ngp.setStepSuccessResult(step, funcStream, ecxSessionId, oFromNg, cb, successResult);                
            }
        });
    }
}

var getIdentities = function(step, funcStream, ecxSessionId, oFromNg, cb)
{
	if(! oFromNg['identityId']['username']) {
		ngp.setRequestState(step.reqId, ecxSessionId, 'identityId', oFromNg['identityId']);
		if ( funcStream.length > 0 ) {
			funcStream[0].continue = 'skip';                                                   
		}

		step.cb(ecxSessionId, true);                                                  
		ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb); 	
	} 
	else {
		var ngState = ngp.ecxSidHash[ecxSessionId];
		var identityExists = false;
		ngp.ngpCoreHelper.getIdentities(ecxSessionId, function(getIdentitiesResObj) {
            
			var arrayOfIdentities = getIdentitiesResObj.rO['users'];
						
			for (var i = 0; i < arrayOfIdentities.length; i++) {
                var identityName = require('querystring').unescape(oFromNg['identityId']['username'] + '_' + oFromNg['pushinstall']['hostAddress']);
				if(arrayOfIdentities[i]['name']==identityName) {
                    ngp.setRequestState(step.reqId, ecxSessionId, 'identityId',arrayOfIdentities[i]['id']);					
					identityExists = true;
					break;
				}
			}

			if(identityExists == true) {
				if ( funcStream.length > 0 ) {
					funcStream[0].continue = 'skip';                                                   
				}
			}

            step.cb(ecxSessionId, true);                                                  
			ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb); 
		});

	}   
};

function createVadpUpdatePutBody(step, ecxSessionId, siteId, hostAddress){
    var identityIdForPostBody = ngp.getRequestState(step.reqId, ecxSessionId, 'identityId');
    var vadpRegisterTplClone = JSON.parse(JSON.stringify(vadpRegisterTpl));
    vadpRegisterTplClone.ipAddr = hostAddress;
    vadpRegisterTplClone.siteId = siteId; 
    vadpRegisterTplClone.identityId = identityIdForPostBody;     
    return vadpRegisterTplClone;
}

var createIdentity = function(step, funcStream, ecxSessionId, oFromNg, cb) {  
	var postBody = {  
        name: require('querystring').unescape(oFromNg['identityId']['username'] + '_' + oFromNg['pushinstall']['hostAddress']),
        password: oFromNg['identityId']['password'],
        username: require('querystring').unescape(oFromNg['identityId']['username']),
    };    
    
	ngp.sendRequestToEcx(step.verb, step.port, step.path, ecxSessionId, postBody, function(rObj){           
		if ( rObj.e ) {
			var failResult = ngp.getGenericEcxErrorResponse(rObj, ecxSessionId, step); 
			ngp.logMsg('error', 'Failed to create Identity ' + postBody.username + ' for session ID=' + ecxSessionId + ' because of error=' + JSON.stringify(e)); 
			ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);			
			return;
		}   
		
        var crO = rObj.rO;
                
		if ( rObj.response.statusCode != 201 ) {
			var failResult = ngp.getGenericEcxErrorResponse(rObj, ecxSessionId, step); 
			ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);
		}
		else {									   
			ngp.logMsg('warn', 'Created Identity with name=' + postBody.username + ' and href=' + JSON.stringify(crO.links.self.href));
            ngp.setRequestState(step.reqId, ecxSessionId, 'identityId', crO.id);
            
			var createdIdentities = ngp.getRequestState(step.reqId, ecxSessionId, 'createdIdentities');

			if ( ! createdIdentities ) {
				createdIdentities = [crO.links.self.href];
				ngp.setRequestState(step.reqId, ecxSessionId, 'createdIdentities', createdIdentities); 
			}
			else {
				createdIdentities.push(crO.links.self.href);
			}

			step.cb(ecxSessionId, true);                                                   
			ngp.runSerialFuncs(funcStream, ecxSessionId, oFromNg, cb); 			
		}
	});
}

var regDeleteIdentity = function(step, funcStream, ecxSessionId, oFromNg, cb)                          
{
    var failResult = ngp.getRequestState(step.reqId, ecxSessionId, 'failureResult');
    
    var createdIdentities = ngp.getRequestState(step.reqId, ecxSessionId, 'createdIdentities');                   
	if ( createdIdentities && createdIdentities.length > 0 ) {
		ngp.logMsg('debug', 'Created Identities to delete=' + JSON.stringify(createdIdentities));
		
		deleteOneIdentity(createdIdentities, ecxSessionId, function(){
			
			ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);		
		});
	}
	else 
	{
		ngp.setStepFailResult(step, funcStream, ecxSessionId, oFromNg, cb, failResult, false);
	}
};

function deleteOneIdentity(identitiesToDelete, ecxSessionId, cb)
{
	if ( identitiesToDelete.length < 1 )
		cb();
	else {
        
        
		var identityToDelete = identitiesToDelete.shift();

        const urlParse = require('url-parse');
		var path = urlParse(identityToDelete).pathname;

		ngp.sendRequestToEcx('DELETE', ngp.ecxExternal, path, ecxSessionId, null, function(dlObj){           
			if ( dlObj.response.statusCode != 204 )
				ngp.logMsg('error', "Deletion of SPP Identity '" + identityToDelete + "' failed because of error=" + JSON.stringify(dlObj.rO));                         
			
                deleteOneIdentity(identitiesToDelete, ecxSessionId, cb);
		});
	}
}
